@@ -5,7 +5,7 @@
 # backend_aml.py: support for running jobs under Azure Machine Learning
 
 import os
-from platform import python_version
+import time
 import shutil
 import logging
 import urllib.request
@@ -14,6 +14,7 @@ from hyperopt import hp
 
 from xtlib import utils, xt_vault
 from xtlib import errors
+from xtlib import scriptor
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import run_helper
@@ -24,6 +25,8 @@ from xtlib.console import console
 from xtlib.hparams import hp_helper
 from xtlib.report_builder import ReportBuilder
 from xtlib.helpers.feedbackParts import feedback as fb
+from xtlib.backends.mount_helper import MountHelper
+from xtlib.backends.node_scriptor import NodeScriptor
 from xtlib.helpers.notebook_builder import NotebookBuilder
 
 from xtlib.backends.backend_base import BackendBase
@@ -64,49 +67,51 @@ class AzureML(BackendBase):
     def get_name(self):
         return "aml"
 
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
+    def build_node_script_and_adjust_runs(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args, cmds=None):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  At the
         time this is called, files can still be added to snapshot_dir.
         '''
-        store_data_dir, data_action, data_writable, store_model_dir, model_action, model_writable,  \
-            storage_name, storage_key = self.get_action_args(args)
+        # store_data_dir, data_action, data_writable, store_model_dir, model_action, model_writable,  \
+        #     storage_name, storage_key = self.get_action_args(args)
 
         # local or POOL of vm's
-        fn_wrapped = None     # we use same script for each box (but with different ARGS)
-        username = args["username"]
+        fn_script = None     # we use same script for each box (but with different ARGS)
+        # username = args["username"]
         fn_shim = constants.FN_AML_SHIM
+        is_direct_run = args["direct_run"]
 
-        for i, box_runs in enumerate(job_runs):
+        for i, box_run in enumerate(job_runs):
             # wrap the user commands in FIRST RUN of each box (apply data/model actions)
-            br = box_runs[0]
-            box_info = br["box_info"]
-            actions = ["data", "model"]
-            run_name = br["run_name"]
-            is_windows = False
-            node_id = utils.node_id(i)
+            # box_info = box_run["box_info"]
+            # actions = ["data", "model"]
+            run_name = box_run["run_name"]
+            # is_windows = False
+            # node_id = utils.node_id(i)
 
-            run_specs = br["run_specs"]
+            run_specs = box_run["run_specs"]
             cmd_parts = run_specs["cmd_parts"]
 
-            if not fn_wrapped:
+            if not fn_script:
                 # just wrap the user cmd once (shared by all boxes/nodes)
                 assert cmd_parts[0] == "python"
                 assert cmd_parts[1] == "-u"
                 assert len(cmd_parts[2]) > 0 
 
-                # update the target_fn (might have been switched to the xt controller)
-                target_fn = cmd_parts[2]
-                arg_parts = cmd_parts[3:]
+                # # update the target_fn (might have been switched to the xt controller)
+                # target_fn = cmd_parts[2]
+                # arg_parts = cmd_parts[3:]
 
                 setup = self.config.get_setup_from_target_def(self.compute_def)
-                use_sudo = utils.safe_value(setup, "use-sudo")       # some AML clusters need sudo
+                # use_sudo = utils.safe_value(setup, "use-sudo")       # some AML clusters need sudo
 
-                fn_wrapped = self.create_wrapper_script(cmd_parts, snapshot_dir, store_data_dir, data_action, 
-                    data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
-                    use_sudo, username, setup, use_allow_other=use_sudo, args=args)
+                # fn_wrapped = self.create_wrapper_script(cmd_parts, snapshot_dir, store_data_dir, data_action, 
+                #     data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
+                #     use_sudo, username, setup, use_allow_other=use_sudo, args=args)
 
-                # AML wants a python script, so use our tiny python shim to run __wrapped__.sh 
+                fn_script, fn_inner = self.build_node_script(box_run, args)
+
+                # AML wants a python script, so use our tiny python shim to run FN_NODE_SCRIPT
                 bootstrap_dir = args["bootstrap_dir"]
                 fn_from = file_utils.get_xtlib_dir() + "/backends/" + constants.FN_AML_SHIM
                 fn_to = bootstrap_dir + "/" + fn_shim
@@ -115,17 +120,88 @@ class AzureML(BackendBase):
                 # copy to submit-logs
                 utils.copy_to_submit_logs(args, fn_from)
 
-            # we update each box's command (passing RUN_NAME as arg to __wrapped__.sh)
-            script_part = "{} {} {}".format(os.path.basename(fn_wrapped), i, run_name)
+                # 1/2 calls to this (TODO: remove one)
+                # this first call forces singularity to write out the long credential env vars to: constants.FN_SET_ENV_VARS
+                dummy_env_vars = {}
+                self.get_controller_env_vars(dummy_env_vars, box_run, node_index=i, args=args)
+
+            # NOTE: we pass the NODE_INDEX, RUN_NAME, and DIRECT_CMD as parameters to the fn_wrapped script
+            # so that we can share a single script among all nodes
+            if args["direct_run"]:
+                direct_cmd = cmds[i] if i < len(cmds) else cmds[0]
+                run_specs["run_cmd"] = direct_cmd
+
+            script_parts = '{}'.format(os.path.basename(fn_script))
 
             # using "source" to the script is failing on ITP (Nov-4-2022, so changing it back to /bin/bash)
             #sh_parts = ['source', script_part]
-            sh_parts = ['/bin/bash', '--login', script_part]
+            bash_parts = ['/bin/bash', '--login', script_parts]
 
             # pass sh_parts as a single argument to avoid wierd "arg": 1 problems with AML estimators
-            wrapped_parts = ["python", "-u", fn_shim, " ".join(sh_parts)]
-            run_specs["cmd_parts"] = wrapped_parts
+            shim_parts = ["python", "-u", fn_shim, " ".join(bash_parts)]
+            run_specs["cmd_parts"] = shim_parts
 
+    def build_node_script(self, first_run, args):
+
+        run_specs = first_run["run_specs"]
+
+        # username = args["username"]ssss
+        # cwd = "$HOME/.xt/cwd"
+
+        cmd_parts = run_specs["cmd_parts"]
+        node_cmd = " ".join(cmd_parts)
+        actions = ["data", "model"]
+
+        # sudo is available, but causes problems with subsequent MNT directory access
+        setup = self.config.get_setup_from_target_def(self.compute_def)
+        use_sudo = utils.safe_value(setup, "use-sudo")       # some AML clusters need sudo  
+        # turn it off for now: (creates mount dirs that normal user code cannot access)
+        #if self.get_name() != "singularity":
+        use_sudo = False     
+
+        # we need to move our cwd (default working dir is blob-mapped and slow)
+        # as of 10/21/2020, ITP recommends we use "/var/tmp" until they disable the NFS mount for $HOME
+        # homebase = "$HOME/{}".format(guid)
+        homebase = "/var/tmp"
+        mountbase = "/var/tmp/.xt"      
+        cwd = "{}/.xt/cwd".format(homebase)
+        get_env_vars_from_file = (self.get_name() == "singularity")
+
+        mount_helper = MountHelper(compute_def=self.compute_def, homebase=homebase, mountbase=mountbase, sudo_available=use_sudo, actions=actions, 
+            use_username=False, use_allow_other=False, nonempty=True, backend=self, config=self.config, args=args)
+        
+        # native or docker, AML deliveers the code to us as: unzipped bootstrap files and a code .zip file
+        use_gpu = args["use_gpu"]
+        default_docker_image = self.get_default_docker_image(use_gpu)
+
+        node_scriptor = NodeScriptor(homebase=homebase, cwd=cwd, controller_cmd=node_cmd, 
+            manual_docker=False, mount_helper=mount_helper, backend=self, compute_def=self.compute_def, use_sudo=use_sudo, 
+            default_docker_image=default_docker_image, get_env_vars_from_file=get_env_vars_from_file, config=self.config, args=args)
+        
+        fn_script, fn_inner = node_scriptor.generate_script()
+        return fn_script, fn_inner
+
+    def gen_code_prep_cmds(self, ca, cmds, cwd, using_docker):
+
+        code_dir = "/usr/src" if using_docker else "."
+        code_fn = code_dir + "/" + constants.CODE_ZIP_FN
+
+        ca.append_title(cmds, "CODE PREP: UNZIP code files to cwd:")
+
+        # AML reuses directories, so we need to clean up first
+        ca.append_dir_clean(cmds, cwd)
+
+        # unzip and remove the code.zip file
+        ca.append_unzip(cmds, code_fn, cwd)
+        if not using_docker:
+            ca.append(cmds, "rm {}".format(constants.CODE_ZIP_FN))
+
+        # this step not needed, since bootstrap files are also in the .zip file
+        # copy the bootstrap files __*__ type files
+        #ca.append(cmds, "cp {}/__* {}".format(code_dir, cwd))
+
+        # move to CWD
+        ca.append(cmds, "cd {}".format(cwd))
 
     def get_default_docker_image(self, use_gpu):
         if use_gpu:
@@ -264,6 +340,17 @@ class AzureML(BackendBase):
         names = [ name for name,value in services.items() if "type" in value and value["type"] == "aml"  ]
         return names
 
+    # API
+    def supports_setting_location(self):
+        return True
+
+    # API 
+    def get_location(self, ws_name):
+        ws = self.get_aml_ws(ws_name)
+        location = ws.location
+
+        return location
+
     def get_experiments(self, ws_name):
         ws = self.get_aml_ws(ws_name)
         return ws.experiments
@@ -457,7 +544,7 @@ class AzureML(BackendBase):
                 if service_info_by_node:
                     for node, node_service_info in service_info_by_node.items():
                         if node_service_info.get("run_name") in run_names:
-                            cancel_result = self.cancel_node(node_service_info)
+                            cancel_result, node_end_time = self.cancel_node(node_service_info)
                             cancel_results.append(cancel_result)
 
         return cancel_results
@@ -694,7 +781,7 @@ class AzureML(BackendBase):
 
         return container_registry, docker_image, sing_dict
 
-    def create_estimator(self, job_id, workspace, aml_ws_name, xt_exper_name, aml_exper_name, run_name, code_dir, target_fn, arg_dict, 
+    def create_estimator(self, job_id, workspace, node_run, aml_ws_name, xt_exper_name, aml_exper_name, run_name, code_dir, target_fn, arg_dict, 
         compute_target, node_id, nodes, fake_submit, args):
 
         config = self.config
@@ -756,7 +843,21 @@ class AzureML(BackendBase):
         env_vars = self.build_env_vars(workspace, aml_ws_name, xt_exper_name, aml_exper_name, run_name, job_id=job_id, 
             compute_target=compute_target, username=username, description=description, aggregate_dest=aggregate_dest, 
             node_id=node_id, args=args)
+        
+        node_index = utils.node_index(node_id)
+        is_direct_run = args["direct_run"]
 
+        box_secret = None     # obsoleted
+        batch_key = None      # doesn't apply to AML
+        direct_cmd = None
+        if is_direct_run:
+            run_specs = node_run["run_specs"]
+            direct_cmd = run_specs["run_cmd"]
+
+        # 2/2 calls to this (TODO: remove one)
+        # this second call really updates the env_vars to be passed to singularity for this node run
+        self.get_controller_env_vars(env_vars, node_run, node_index, args)
+       
         container_registry, docker_image, _, = self.get_docker_container_registry(args)
 
         framework = args["framework"]
@@ -799,9 +900,8 @@ class AzureML(BackendBase):
             distributed_obj = None
 
         compute_def = args["compute_def"]
-        direct_run = args["direct_run"]
 
-        if direct_run:
+        if is_direct_run:
             # relying on AML for full control (not using XT controller)
             node_count = utils.safe_value(compute_def, "nodes")
 
@@ -889,6 +989,9 @@ class AzureML(BackendBase):
                         # specify additional docker run flags so that we can use blobfuse
                         other_args += ["--privileged"]
 
+                    # AML doesn't support $PWD (why?)
+                    other_args = [arg for arg in other_args if not "$PWD" in arg]
+
                     environment.docker.arguments = other_args
 
                 if docker_image:
@@ -930,10 +1033,31 @@ class AzureML(BackendBase):
 
             low_pri = utils.safe_value(self.compute_def, "low-pri", False)
 
+            # give sublclasses a chance to chime in
             self.update_estimator(estimator, gpu_count, low_pri)
 
         return estimator, experiment
 
+    def get_controller_env_vars(self, env_vars, node_run, node_index, args):
+        #node_index = node_run["node_index"]
+        run_name = node_run["run_name"]
+        is_direct_run = args["direct_run"]
+
+        batch_key = None      # doesn't apply to AML
+        direct_cmd = None
+        if is_direct_run:
+            run_specs = node_run["run_specs"]
+            direct_cmd = run_specs["run_cmd"]
+
+        scriptor.add_controller_env_vars(env_vars, self.config, node_index=node_index, run_name=run_name, 
+            batch_key=batch_key, direct_cmd=direct_cmd, args=args)
+        
+        # give subclass a chance to update env vars
+        self.update_env_vars(env_vars, args)
+
+    def update_env_vars(self, env_vars, args):
+        pass
+
     def update_estimator(self, estimator, gpu_count, preemption_allowed):
         pass
 
@@ -958,8 +1082,8 @@ class AzureML(BackendBase):
         nodes = len(job_runs)
         service_info_by_node = {}
 
-        for i, node_runs in enumerate(job_runs):
-            node_info = self.submit_node_runs(job_id, node_runs, workspace, aml_ws_name, experiment, aml_exper_name, 
+        for i, node_run in enumerate(job_runs):
+            node_info = self.submit_node_run(job_id, node_run, workspace, aml_ws_name, experiment, aml_exper_name, 
                 compute_def, resume_name, repeat_count, using_hp, compute, runs_by_box, snapshot_dir, i, 
                 show_aml_run_name, nodes, args)
 
@@ -972,15 +1096,14 @@ class AzureML(BackendBase):
         fb.reset_feedback()
         fb.feedback("submitted")
 
-        service_run_info = {}
-        return service_run_info, service_info_by_node     
+        service_job_info = {}    # AML has updated the job, so don't return this
+        return service_job_info, service_info_by_node     
 
-    def submit_node_runs(self, job_id, node_runs, ws_name, aml_ws_name, xt_exper_name, aml_exper_name, 
+    def submit_node_run(self, job_id, node_run, ws_name, aml_ws_name, xt_exper_name, aml_exper_name, 
         compute_def, resume_name, repeat_count, using_hp, compute, runs_by_box, code_dir, node_index, 
         show_aml_run_name, nodes, args):
 
-        first_run = node_runs[0]
-        first_run_name = first_run["run_name"]
+        first_run_name = node_run["run_name"]
         fake_submit = args["fake_submit"]
         trainer = None
 
@@ -990,9 +1113,9 @@ class AzureML(BackendBase):
         self.serializable_estimator = None
         self.serializable_trainer = None
         
-        box_name = first_run["box_name"]
+        box_name = node_run["box_name"]
 
-        run_specs = first_run["run_specs"]
+        run_specs = node_run["run_specs"]
         cmd_parts = run_specs["cmd_parts"]
         target_fn = args["script"]
         node_id = "node" + str(node_index)
@@ -1024,7 +1147,7 @@ class AzureML(BackendBase):
         # append job/run name to AML experiment name (make locating XT runs in AML portal easier)
         aml_exper_name += "__{}__{}".format(job_id, first_run_name)
 
-        estimator, experiment = self.create_estimator(job_id, ws_name, aml_ws_name, xt_exper_name, aml_exper_name, first_run_name, 
+        estimator, experiment = self.create_estimator(job_id, ws_name, node_run, aml_ws_name, xt_exper_name, aml_exper_name, first_run_name, 
             code_dir, target_fn, arg_dict, compute_target, node_id, nodes, fake_submit, args)
 
         hp_config = args["hp_config"]
@@ -1094,27 +1217,15 @@ class AzureML(BackendBase):
         else:
             fb.feedback("{}".format(run_name))
 
-        run_names = []
-        for run in node_runs:
-            run_name = run["run_name"]
-            run_names.append(run_name)
-
+        run_name = node_run["run_name"]
         node_info = {"ws": ws_name}
 
-        for run_name in run_names:
-            # we only have 1 run, so OK to hold info in flat dict here
-            node_info["aml_exper_name"] = aml_exper_name
-            node_info["aml_run_number"] = aml_run_number
-            node_info["aml_run_id"] = aml_run_id
-            node_info["run_name"] = run_name
-            node_info["job_id"] = job_id
-            node_info["node_id"] = node_id
-
-            # update mongo db info for run with cluster and service_job_id
-            # DO WE STILL NEED THIS?  Aug-08-2020
-            # I think not, since this is now covered by service_node_info 
-            #db = self.store.get_database()
-            #mongo.update_run_info(workspace, run_name, {"aml_exper_name": aml_exper_name, "aml_run_number": aml_run_number})
+        node_info["aml_exper_name"] = aml_exper_name
+        node_info["aml_run_number"] = aml_run_number
+        node_info["aml_run_id"] = aml_run_id
+        node_info["run_name"] = run_name
+        node_info["job_id"] = job_id
+        node_info["node_id"] = node_id
 
         if monitor_cmd:
             console.print("monitoring notebook created; to run:")
@@ -1330,16 +1441,112 @@ class AzureML(BackendBase):
 
         return run
 
+    # API call
+    def get_timestamped_status_of_nodes(self, service_name, service_infos):
+        '''
+        nodes must all be from the same AML experiment
+        '''
+        first_si = None
+
+        for si in service_infos:
+            if si:
+                first_si = si
+                break
+
+        if not first_si or not service_name:
+            sd = {"node"+str(i): (None,None) for i in range(len(service_infos))}
+            return sd
+        
+        experiment_name = first_si["aml_exper_name"]
+
+        ws = self.get_aml_ws(service_name)
+        sd = {}
+
+        experiment = Experiment(ws, name=experiment_name)
+        #run = Run(experiment, aml_node_id_list[0])
+
+        # enumerate just specified runs
+        for si in service_infos:
+            if not si:
+                sd[node_id] = (None, None)
+            
+            else:
+                aml_run_id = si["aml_run_id"]
+                node_id = si["node_id"]
+
+                run = Run(experiment, aml_run_id)
+
+                status = self.get_simple_status(run.status)
+                details = run.get_details()
+
+                if not "endTimeUtc" in details:
+                    print("----> WARNING: AML/ITP/SINGULARITY node: {} has not terminated according to service.  status: {}".format(aml_run_id, status))
+                    sd[node_id] = (status, None)
+
+                else:
+                    end_time_utc = details["endTimeUtc"]
+                    sd[node_id] = (status, end_time_utc)
+
+        return sd
+         
+    # API call
+    def get_status_of_nodes(self, service_name, experiment_name, service_info_list):
+        ws = self.get_aml_ws(service_name)
+        sd = {}
+
+        experiment = Experiment(ws, name=experiment_name)
+        #run = Run(experiment, aml_node_id_list[0])
+
+        # enumerate just specified runs
+        started = time.time()
+        print("  getting {:,} specified AML runs".format(len(service_info_list)))
+        
+        for si in service_info_list:
+            aml_run_id = si["aml_run_id"]
+            node_id = si["node_id"]
+
+            run = Run(experiment, aml_run_id)
+            print("    id: {}, status: {}".format(run.id, run.status))
+            sd[node_id] = run.status
+        
+        elapsed = time.time() - started
+        print("  enumerated {:,} desired runs ({:.2f} secs)".format(len(service_info_list), elapsed))
+
+        # # enumerate all runs in experiment
+        # started = time.time()
+        # run_count = 0
+        # match_count = 0
+
+        # # pd = {"id": {"$in": aml_node_id_list}}
+        # # runs = experiment.get_runs(properties=pd)
+
+        # print("getting all AML runs for exper: {}".format(experiment_name))
+        # runs = experiment.get_runs()
+
+        # for run in runs:
+        #     run_count += 1
+        #     if run["id"] in aml_node_id_list:
+        #         print("id: {}, status: {}".format(run.id, run.status))
+        #         match_count += 1
+
+        # elapsed = time.time() - started
+        # print("enumerated {:,} total runs, {:,} desired runs ({:.2f} secs".format(run_count, match_count, elapsed))
+
+        return sd
+
     # API call
     def cancel_job(self, service_job_info, service_info_by_node):
         result_by_node = {}
+        node_end_times = {}
 
         for node_id, node_info in service_info_by_node.items():
-            result = self.cancel_node(node_info)
+            result, node_end_time = self.cancel_node(node_info)
+
             if result is not None:
                 result_by_node[node_id] = result
+                node_end_times[node_id] = node_end_time
 
-        return result_by_node
+        return result_by_node, node_end_times
 
     # API call
     def cancel_node(self, service_node_info):
@@ -1352,27 +1559,39 @@ class AzureML(BackendBase):
             before_status = simple_status
             cancelled = False
 
-            if simple_status != "completed":
+            if simple_status in ["error", "completed", "cancelled"]:
+                # node already terminated
+                details = run.get_details()
+
+                if not "endTimeUtc" in details:
+                    print("----> WARNING: AML/ITP/SINGULARITY node: {} has not terminated according to service.  status: {}".format(service_node_info, service_status))
+                    node_end_time = None
+                else:
+                    node_end_time_str = details["endTimeUtc"]
+                    node_end_time = utils.get_time_from_arrow_str(node_end_time_str)
                 
+            else:
+                # node is alive - cancel it
                 run.cancel()
 
                 service_status = run.get_status()
                 simple_status = self.get_simple_status(service_status)
-                cancelled = (service_status == "completed")     
+                cancelled = (simple_status == "completed") 
+                node_end_time = time.time()    
 
             result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
 
-        return result
+        return result, node_end_time
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
 
-        self.append(cmds, "mkdir -p {}".format(dest_dir))
-        #self.append(cmds, "cp -r -v $AZUREML_LOGDIRECTORY_PATH/* {}".format(dest_dir))
-        #self.append(cmds, "cp -r -v $AZ_BATCHAI_STDOUTERR_DIR/* {}".format(dest_dir))
-        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml_compute_logs/* {}".format(dest_dir))
-        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml-logs/* {}".format(dest_dir))
-        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/logs/* {}".format(dest_dir))
+        ca.append(cmds, "mkdir -p {}".format(dest_dir))
+        #ca.append(cmds, "cp -r -v $AZUREML_LOGDIRECTORY_PATH/* {}".format(dest_dir))
+        #ca.append(cmds, "cp -r -v $AZ_BATCHAI_STDOUTERR_DIR/* {}".format(dest_dir))
+        ca.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml_compute_logs/. {}".format(dest_dir))
+        ca.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml-logs/. {}".format(dest_dir))
+        ca.append(cmds, "cp -r $XT_ORIG_WORKDIR/logs/. {}".format(dest_dir))
 
     # API call
     def get_log_files_dir(self, args):
