@@ -4,10 +4,10 @@
 #
 # backend_batch.py: support for running a XT job on a 1-N Azure Batch Compute boxes.
 
-from calendar import c
 import os
 import io
 import sys
+import arrow
 import shutil
 import time
 import datetime
@@ -27,6 +27,8 @@ from xtlib.console import console
 from xtlib.helpers.xt_config import XTConfig
 from xtlib.report_builder import ReportBuilder
 from xtlib.helpers.feedbackParts import feedback as fb
+from xtlib.backends.mount_helper import MountHelper
+from xtlib.backends.node_scriptor import NodeScriptor
 from xtlib.helpers.key_press_checker import KeyPressChecker
 
 from .backend_interface import BackendInterface
@@ -44,14 +46,11 @@ class AzureBatch(BackendBase):
         - building a "node_record" that describes how to launch each node
         - call the appropriate Batch SDK API's to build a batch job and launch it.
 
-    Building a node record:
-        - create a Batch ResourceFile for each input file and the expected output files
-        - wraps the command(s) for the node by prefixing these commands:
-
-            unzip CODE_ZIP_FN               (the user's source tree)
-            conda activate py36             (activate py36 known good ML environment)
-            pip install xtlib               (for controller and/or user's script)
-            export XT_NODE_ID=nodeNN        (for controller to get his run info from the MRC file)
+    Features:
+        - Build the node script (shared by all nodes of the job)
+        - Building a node record:
+            - create a Batch ResourceFile for each input file and the expected output files
+            - include the node script 
     '''
 
     def __init__(self, compute, compute_def, core, config, username=None, arg_dict=None):
@@ -110,67 +109,62 @@ class AzureBatch(BackendBase):
         '''
         return False
 
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
+    def build_node_script_and_adjust_runs(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args, cmds=None):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  At the
         time this is called, files can still be added to snapshot_dir.
         '''
-        store_data_dir, data_action, data_writable, store_model_dir, model_action, model_writable,  \
-            storage_name, storage_key = self.get_action_args(args)
 
-        # local or POOL of vm's
-        fn_wrapped = None     # we use same cmd on each box/job
-        username = args["username"]
         nowrap = args["nowrap"]
+        is_direct_run = args["direct_run"]
 
-        for i, box_runs in enumerate(job_runs):
-            node_id = "node" + str(i)
-            br = box_runs[0]
-            box_info = br["box_info"]
-            run_name = br["run_name"]
-            actions = ["data", "model"]
-            box_secret = br["box_secret"]
+        if nowrap:
+            # user script is the node script
+            fn_script = args["script"]
+            fn_inner = None
+        else:
+            # build the node script
+            first_run = job_runs[0]
+            fn_script, fn_inner = self.build_node_script(first_run, args)
 
-            is_windows = False
+        # adjust each job run to run fn_script
+        for i, box_run in enumerate(job_runs):
+            run_name = box_run["run_name"]
 
-            run_specs = br["run_specs"]
-            cmd_parts = run_specs["cmd_parts"]
+            run_specs = box_run["run_specs"]
+            #cmd_parts = run_specs["cmd_parts"]
 
             if nowrap:
                 # run user's script directly
-                fn_direct = os.path.basename(args["script"])
+                fn_direct = os.path.basename(fn_script)
                 #run_specs["cmd_parts"] = ["echo i was here; ls -lt; echo bye now"]   # [fn_direct]
                 cmd = "/bin/bash --login -c 'echo starting script: {}; {}; echo script ended'".format(fn_direct, fn_direct)
                 run_specs["cmd_parts"] = [cmd]
             
             else:
-                # just create the fn_wrapped for the first node (but use for all nodes)
-                if i == 0:
-                    node_cmd = " ".join(cmd_parts)
-
-                    fn_wrapped = self.wrap_cmd_for_batch(node_cmd, node_id, store_data_dir, data_action, data_writable, 
-                        store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, 
-                        box_secret, args)
-
-                # NOTE: we pass the NODE_INDEX and RUN_NAME as parameters to the fn_wrapped script
+                # NOTE: we pass the NODE_INDEX, RUN_NAME, and DIRECT_CMD as environment variables
                 # so that we can share a single script among all nodes
-                wrapped_parts = ['/bin/bash', '--login', '{} {} {}'.format(os.path.basename(fn_wrapped), i, run_name)]
-                run_specs["cmd_parts"] = wrapped_parts
+                if is_direct_run:
+                    direct_cmd = cmds[i] if i < len(cmds) else cmds[0]
+                    run_specs["run_cmd"] = direct_cmd
+
+                bash_parts = [ '/bin/bash', '--login', '{}'.format(os.path.basename(fn_script)) ]
+                run_specs["cmd_parts"] = bash_parts
 
                 merge_batch_logs = args["merge_batch_logs"]
                 if merge_batch_logs:
                     # recompose this command to pass along all script arguments
-                    wrapped_parts = ['/bin/bash', '--login', '{} $*'.format(os.path.basename(fn_wrapped))]
+                    bash_parts = ['/bin/bash', '--login', '{} $*'.format(os.path.basename(fn_script))]
         
                     # create a STDBOTH.txt that merged stdout and stderr
-                    #node_cmd = "{} > ../stdboth_tmp.txt 2>&1 && cp ../stdboth_tmp.txt ../stdboth.txt".format(" ".join(wrapped_parts))
-                    node_cmd = "{} > ../stdboth.txt 2>&1".format(" ".join(wrapped_parts))
+                    node_cmd = "{} > ../stdboth.txt 2>&1".format(" ".join(bash_parts))
 
                     # create a helper file to call our node cmd
                     fn_helper = snapshot_dir + "/" + constants.FN_BATCH_HELPER
                     file_utils.write_text_file(fn_helper, node_cmd)
 
-                    run_specs["cmd_parts"] = ["bash", constants.FN_BATCH_HELPER, str(i), run_name]
+                    outer_bash_parts = ["bash", constants.FN_BATCH_HELPER, str(i), run_name]
+                    run_specs["cmd_parts"] = outer_bash_parts
 
                     console.diag(run_specs)
 
@@ -179,76 +173,119 @@ class AzureBatch(BackendBase):
         fn_to = snapshot_dir + "/" + constants.FN_BATCH_NODE_ID
         shutil.copyfile(fn_from, fn_to)
 
-    def wrap_cmd_for_batch(self, user_cmd, node_id, store_data_dir, data_action, data_writable, 
-        store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, box_secret, args):
+    def build_node_script(self, first_run, args):
 
-        cmds = []
-        
-        gen_for_windows = False
-        self.capture_setup_cmds = args["capture_setup_cmds"]
+        run_specs = first_run["run_specs"]
+        box_secret = first_run["box_secret"]
 
-        # default for backends is to pass these 2 args to their __wrapped__.xx script (philly is different)
-        node_id = "node$1"
-        run_name = "$2"
-        docker_cmd = args["docker_cmd"]
-        job_id = args["job_id"]
-
-        # for wrapper building functions to access 
-        self.node_id = node_id
-        self.run_name = run_name
-        
-        add_pre_cmds = False
-
-        if add_pre_cmds:
-            # add env vars (the Batch Task environment_setting property seems to be broken
-            env_vars = dict(args["env_vars"])
-            scriptor.add_controller_env_vars(env_vars, self.config, box_secret, None)
-
-            self.append_title(cmds, "PRE-SETUP cmds:")
-            for name, value in env_vars.items():
-                cmd = "export {}={}".format(name, value)
-                self.append(cmds, cmd)
-
-        # our initial working dir: .../wd
-        # this will change us to: .../wd/.xt/cwd
-        self.add_first_cmds(cmds, homebase="$HOME", cwd="$HOME/.xt/cwd", args=args)
-
-        self.append_title(cmds, "UNZIP code/bootstrap files:")
-
-        # unzip files from wd to current directory
-        if docker_cmd:
-            fn_bootstrap = "/usr/src/" + constants.CODE_ZIP_FN
-            self.append_unzip(cmds, fn_bootstrap, ".")
-        else:
-            fn_bootstrap = "../../" + constants.CODE_ZIP_FN
-            self.append_unzip(cmds, fn_bootstrap, ".")
-            self.append(cmds, "rm {}".format(fn_bootstrap))
-
-        self.append_dir(cmds)
-
-        super().add_setup_cmds(cmds, remove_zip=False, gen_for_windows=gen_for_windows, 
-            args=args)
-
-        super().add_other_cmds(cmds, args)
-        super().add_report_cmds(cmds, args=args)
-
-        # since node id is the only different between nodes (in this __wrapped__.sh file), make it a parameter
-        self.append(cmds, 'export XT_NODE_ID=node$1')       #  + node_id)
+        # username = args["username"]
+        # cwd = "$HOME/.xt/cwd"
 
+        cmd_parts = run_specs["cmd_parts"]
+        node_cmd = " ".join(cmd_parts)
+        homebase = "$HOME"
         actions = ["data", "model"]
-
-        # try to install blobfuse, but it may already be installed on some images
-        install_blobfuse = True
+        manual_docker = bool(args["docker_cmd"]) 
 
         # sudo is available, but causes problems with subsequent MNT directory access
         sudo_available = False     
 
-        self.add_mount_cmds(cmds, sudo_available, storage_name, storage_key, 
-            actions, data_action, model_action, store_data_dir, store_model_dir, data_writable, model_writable,
-            use_username=False, use_allow_other=False, install_blobfuse=install_blobfuse, args=args)
+        mount_helper = MountHelper(compute_def=self.compute_def, homebase=homebase, sudo_available=sudo_available, actions=actions,
+            use_username=False, use_allow_other=False, nonempty=True, backend=self, config=self.config, args=args)
+        
+        cwd="$HOME/.xt/cwd"
+        default_docker_image = None
 
-        fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, sudo_available, args)
-        return fn_wrapped
+        node_scriptor = NodeScriptor(homebase=homebase, cwd=cwd, controller_cmd=node_cmd, 
+            manual_docker=manual_docker, mount_helper=mount_helper, backend=self, compute_def=self.compute_def, default_docker_image=default_docker_image,
+            config=self.config, args=args)
+        
+        fn_script, fn_inner = node_scriptor.generate_script()
+        return fn_script, fn_inner
+    
+    def gen_code_prep_cmds(self, ca, cmds, cwd, using_docker):
+        code_dir = "/usr/src" if using_docker else "."
+        code_fn = code_dir + "/" + constants.CODE_ZIP_FN
+
+        ca.append_title(cmds, "CODE PREP: UNZIP code files to cwd:")
+        ca.append_unzip(cmds, code_fn, cwd)
+
+        if not using_docker:
+            ca.append(cmds, "rm {}".format(constants.CODE_ZIP_FN))
+
+        # move to CWD
+        ca.append(cmds, "cd {}".format(cwd))
+
+    # def wrap_cmd_for_batch(self, user_cmd, node_id, store_data_dir, data_action, data_writable, 
+    #     store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, box_secret, args):
+
+    #     cmds = []
+        
+    #     gen_for_windows = False
+    #     self.capture_setup_cmds = args["capture_setup_cmds"]
+
+    #     # default for backends is to pass these 2 args to their __wrapped__.xx script (philly is different)
+    #     node_id = "node$1"
+    #     run_name = "$2"
+    #     docker_cmd = args["docker_cmd"]
+    #     job_id = args["job_id"]
+
+    #     # for wrapper building functions to access 
+    #     self.node_id = node_id
+    #     self.run_name = run_name
+        
+    #     add_pre_cmds = False
+
+    #     if add_pre_cmds:
+    #         # add env vars (the Batch Task environment_setting property seems to be broken
+    #         env_vars = dict(args["env_vars"])
+    #         scriptor.add_controller_env_vars(env_vars, self.config, box_secret, None)
+
+    #         self.append_title(cmds, "PRE-SETUP cmds:")
+    #         for name, value in env_vars.items():
+    #             cmd = "export {}={}".format(name, value)
+    #             self.append(cmds, cmd)
+
+    #     # our initial working dir: .../wd
+    #     # this will change us to: .../wd/.xt/cwd
+    #     self.add_first_cmds(cmds, homebase="$HOME", cwd="$HOME/.xt/cwd", args=args)
+
+    #     self.append_title(cmds, "UNZIP code/bootstrap files:")
+
+    #     # unzip files from wd to current directory
+    #     if docker_cmd:
+    #         fn_bootstrap = "/usr/src/" + constants.CODE_ZIP_FN
+    #         self.append_unzip(cmds, fn_bootstrap, ".")
+    #     else:
+    #         fn_bootstrap = "../../" + constants.CODE_ZIP_FN
+    #         self.append_unzip(cmds, fn_bootstrap, ".")
+    #         self.append(cmds, "rm {}".format(fn_bootstrap))
+
+    #     self.append_dir(cmds)
+
+    #     super().add_setup_cmds(cmds, remove_zip=False, gen_for_windows=gen_for_windows, 
+    #         args=args)
+
+    #     super().add_other_cmds(cmds, args)
+    #     super().add_report_cmds(cmds, args=args)
+
+    #     # since node id is the only different between nodes (in this __wrapped__.sh file), make it a parameter
+    #     self.append(cmds, 'export XT_NODE_ID=node$1')       #  + node_id)
+
+    #     actions = ["data", "model"]
+
+    #     # try to install blobfuse, but it may already be installed on some images
+    #     install_blobfuse = True
+
+    #     # sudo is available, but causes problems with subsequent MNT directory access
+    #     sudo_available = False     
+
+    #     self.add_mount_cmds(cmds, sudo_available, storage_name, storage_key, 
+    #         actions, data_action, model_action, store_data_dir, store_model_dir, data_writable, model_writable,
+    #         use_username=False, use_allow_other=False, install_blobfuse=install_blobfuse, args=args)
+
+    #     fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, sudo_available, args)
+    #     return fn_wrapped
 
     def print_things(self, things, columns):
         
@@ -358,9 +395,9 @@ class AzureBatch(BackendBase):
         pool_id = self.launch(job_id, node_records, auto_pool=auto_pool, ws_name=ws_name, vm_size=vm_size, 
             vm_image=vm_image, num_nodes=num_nodes, use_low_pri=use_low_pri, is_distributed=is_distributed, args=args)
 
-        service_run_info = {"batch_job_id": self.batch_job_id, "pool_id": pool_id}
+        service_job_info = {"batch_job_id": self.batch_job_id, "pool_id": pool_id}
 
-        return service_run_info, service_info_by_node    
+        return service_job_info, service_info_by_node    
 
     def prep_all_nodes(self, job_id, job_runs_list, num_nodes, use_low_pri, using_hp, exper_name, 
         static_run_cmds, node_records, workspace, service_info_by_node, merge_batch_logs, args):
@@ -373,12 +410,13 @@ class AzureBatch(BackendBase):
         def thread_worker(job_runs, job_id, num_nodes, use_low_pri, 
                 using_hp, exper_name, static_run_cmds, args):
 
-            # process a subset of all jobs on this thread
-            for i, run_data_list in enumerate(job_runs):
+            # process a subset of all job_runs (nodes) on this thread
+            for i, node_run in enumerate(job_runs):
                 
-                node_index = run_data_list[0]["node_index"]
+                # if this is a direct run (no controller), use "box_index" instead of "node_index"
+                node_index = node_run["node_index"] if "node_index" in node_run else node_run["box_index"]
 
-                node_record = self.prep_node(job_id, node_index, run_data_list, num_nodes, use_low_pri, 
+                node_record = self.prep_node(job_id, node_index, node_run, num_nodes, use_low_pri, 
                     using_hp=using_hp, exper_name=exper_name, static_run_cmds=static_run_cmds, args=args)
 
                 with thread_lock:
@@ -393,9 +431,11 @@ class AzureBatch(BackendBase):
 
                     node_id = "node" + str(node_index)
                     batch_job_id = self.make_batch_job_id(workspace, job_id)
+                    run_name = "run_{}".format(node_index)
 
                     node_info = {"workspace": workspace, "job_id": job_id, "batch_job_id": batch_job_id, 
-                        "node_id": node_id, "task_index": node_index, "merge_batch_logs": merge_batch_logs}
+                        "node_id": node_id, "task_index": node_index, "merge_batch_logs": merge_batch_logs,
+                        "run_name": run_name}
 
                     service_info_by_node[node_id] = node_info
 
@@ -411,18 +451,16 @@ class AzureBatch(BackendBase):
         # sort node_records to normal order
         node_records.sort(key=lambda r: r["node_index"])
 
-    def prep_node(self, job_id, node_index, run_data_list, num_nodes, use_low_pri, using_hp,  
+    def prep_node(self, job_id, node_index, node_run, num_nodes, use_low_pri, using_hp,  
             exper_name, static_run_cmds, args):
         '''prepare runs for node node_index'''
 
-        first_run = run_data_list[0]
-        box_info = first_run["box_info"]
-
-        box_name = "{}-{}-{}".format(job_id, "batch", node_index)
+        # box_info = node_run["box_info"]
+        # box_name = "{}-{}-{}".format(job_id, "batch", node_index)
         
         # target_sh, node_cmd, fn_context, fn_sh, run_names = self.prep_for_controller_run(run_data_list, node_index, job_id, tmp_dir, using_hp, 
         #     box_info, exper_name, args=args)
-        run_names = ", ".join([rd["run_name"] for rd in run_data_list])
+        run_name = node_run["run_name"]
 
         # build RESOURCE FILES for each file we have uploaded into "before"  directory
         before_path = "before/code"      # job store
@@ -468,12 +506,13 @@ class AzureBatch(BackendBase):
         # if static_run_cmds:
         #     fb.feedback("  adding node: {}, run(s): {}".format(box_name.upper(), run_names.upper()), is_final=True)
 
-        node_specs = first_run["run_specs"]
-        box_secret = first_run["box_secret"]
-        node_cmd = " ".join(node_specs["cmd_parts"])
+        node_specs = node_run["run_specs"]
+        box_secret = node_run["box_secret"]
+        cmd_parts = node_specs["cmd_parts"]
+        node_cmd = " ".join(cmd_parts)
 
         return {"node_cmd": node_cmd, "node_res_files": node_res_files, "node_output_files": node_output_files, 
-            "box_secret": box_secret, "node_index": node_index}
+            "box_secret": box_secret, "node_index": node_index, "run_name": run_name}
 
     def launch(self, job_id, node_records, auto_pool=True, description=None, ws_name=None, 
             vm_size=None, vm_image=None, num_nodes=1, use_low_pri=True, is_distributed=False, args=None):
@@ -540,6 +579,23 @@ class AzureBatch(BackendBase):
         batch_client.retry = utils.make_retry_func()
         self.batch_client = batch_client
 
+    # API
+    def supports_setting_location(self):
+        # not currently supported
+        return False
+    
+    # API 
+    def get_location(self, service):
+        if not self.batch_client:
+           self.create_batch_client()
+
+        # cannot find a proper API for this, so just parse the URL
+        url = self.batch_client.config.batch_url
+        parts = url.split(".")
+        location = parts[1]
+
+        return location
+
     def get_external_port(self, port_name, node):
         port = None
         ip_addr = None
@@ -815,10 +871,23 @@ class AzureBatch(BackendBase):
             node_cmd = node_record["node_cmd"]
             node_res_files = node_record["node_res_files"]
             node_output_files = node_record["node_output_files"]
+            run_name = node_record["run_name"]
 
-            env_vars = dict(args["env_vars"])
-            env_vars["XT_NODE_ID"] = "node" + str(idx)
-            env_vars["XT_BOX_SECRET"] = node_record["box_secret"]
+            #env_vars = dict(args["env_vars"])
+            # env_vars["XT_NODE_ID"] = "node" + str(idx)
+            # env_vars["XT_BOX_SECRET"] = node_record["box_secret"]
+
+            # user's env vars
+            node_env_vars = dict(args["env_vars"])
+
+            # add NODE-SPECIFIC (and shared) XT env vars 
+            service_name = utils.safe_value(self.compute_def, "service")
+            batch_creds = self.config.get_service(service_name)
+            batch_key = batch_creds["key"]
+            direct_cmd = node_cmd if args["direct_run"] else None
+
+            scriptor.add_controller_env_vars(node_env_vars, self.config, node_index=idx, run_name=run_name, 
+                batch_key=batch_key, direct_cmd=direct_cmd, args=args)
 
             task_id = "task{}".format(idx)
             #console.print("add task: id=", task_id, ", node_cmd=", node_cmd)
@@ -828,7 +897,7 @@ class AzureBatch(BackendBase):
 
             # add user-specified environment variables
             env_var_list = []
-            for key, value in env_vars.items():
+            for key, value in node_env_vars.items():
                 # wierd API; need to specify params by name or get wierd errors
                 es = batchmodels.EnvironmentSetting(name=key, value=str(value))
                 env_var_list.append(es)
@@ -1452,6 +1521,7 @@ class AzureBatch(BackendBase):
         simple_status = None
         next_offset = 0
         found_file = False
+        from_storage = True
 
         if log_name is None:
             log_name = "stdout.txt"
@@ -1479,6 +1549,7 @@ class AzureBatch(BackendBase):
                     log_source = "after_logs"
 
         if not found_file:
+            from_storage = False
 
             # this call can take up to 15 secs
             if not task:
@@ -1486,6 +1557,7 @@ class AzureBatch(BackendBase):
                 for i in range(5):
                     try:
                         task = self.get_node_task(service_node_info)
+                        break
                     except BaseException as ex:
                         console.print("monitor exception: {}".format(ex))
                         time.sleep(3)
@@ -1527,7 +1599,7 @@ class AzureBatch(BackendBase):
 
         return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
             "service_status": batch_status, "file_path": file_path, "service_context": {"task": task}, "found_file": found_file, 
-            "log_source": log_source}
+            "log_source": log_source, "from_storage": from_storage}
 
     # API call
     def get_simple_status(self, status):
@@ -1551,12 +1623,17 @@ class AzureBatch(BackendBase):
     # API call
     def cancel_job(self, service_job_info, service_info_by_node):
         result_by_node = {}
+        node_end_times = {}
 
         for node_id, node_info in service_info_by_node.items():
-            result = self.cancel_node(node_info)
-            result_by_node[node_id] = result
+            result, node_end_time = self.cancel_node(node_info)
 
-        return result_by_node
+            result_by_node[node_id] = result
+            node_end_times[node_id] = node_end_time
+
+        # NOTE: we currently don't explictly cancel the job (since it is cancelled when all tasks are cancelled)
+
+        return result_by_node, node_end_times
 
     # API call
     def cancel_node(self, service_node_info):
@@ -1565,9 +1642,14 @@ class AzureBatch(BackendBase):
         service_status = task.state.value
         simple_status = self.get_simple_status(service_status)
         cancelled = False
+        node_end_time = None
 
-        if simple_status != "completed":
+        if simple_status in ["error", "completed", "cancelled"]:
+            # node already ended
+            node_end_time = task.state_transition_time.timestamp()
             
+        else:
+            # node is alive - cancel it
             batch_job_id = service_node_info["batch_job_id"]
             self.batch_client.task.terminate(batch_job_id, task.id)
 
@@ -1576,13 +1658,14 @@ class AzureBatch(BackendBase):
 
             service_status = task.state.value
             simple_status = self.get_simple_status(service_status)
-            cancelled = (service_status == "completed")     
+            cancelled = (simple_status == "completed")   
+            node_end_time = time.time()  
 
         result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
-        return result
+        return result, node_end_time
 
     # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
         # this is done by the batch service at the end of our batch task
         pass            
 
@@ -1643,3 +1726,101 @@ class AzureBatch(BackendBase):
 
         path = "/usr/logs" if docker_cmd else "$XT_ORIG_WORKDIR/.."
         return path
+
+    # API call
+    def get_timestamped_status_of_nodes(self, service_name, service_info_list):
+        sd = {}
+
+        if self.batch_client is None:
+            self.create_batch_client()
+    
+        missing_tasks = 0
+
+        for si in service_info_list:
+            if not si:
+                sd[node_id] = (None, None)
+
+            else:
+                batch_job_id = si["batch_job_id"]
+                task_index = si["task_index"]
+
+                task_id = "task" + str(task_index)
+                node_id = si["node_id"]
+
+                try:
+                    task = self.batch_client.task.get(batch_job_id, task_id)
+                    status = self.get_simple_status(task.state)
+                    #print("    id: {}, status: {}".format(task.id, status))
+                    dt_end = task.state_transition_time
+                    end_time = arrow.get(dt_end)
+                    sd[node_id] = (status, end_time)
+                except BaseException as ex:
+                    #print("error trying to access batch task_id: {}, {}".format(task_id, ex.error.code))
+                    sd[node_id] = (None, None)
+
+        return sd
+
+    # API call
+    def get_status_of_nodes(self, service_name, experiment_name, service_info_list):
+        sd = {}
+
+        started = time.time()
+        print("  getting {:,} specified BATCH tasks".format(len(service_info_list)))
+
+        if self.batch_client is None:
+            self.create_batch_client()
+    
+        missing_tasks = 0
+
+        for si in service_info_list:
+            batch_job_id = si["batch_job_id"]
+            task_index = si["task_index"]
+
+            task_id = "task" + str(task_index)
+            node_id = si["node_id"]
+
+            try:
+                task = self.batch_client.task.get(batch_job_id, task_id)
+            except BaseException as ex:
+                #print("error trying to access batch task_id: {}, {}".format(task_id, ex.error.code))
+                sd[node_id] = None
+                missing_tasks += 1
+            else:
+                status = str(task.state)
+                print("    id: {}, status: {}".format(task.id, status))
+                sd[node_id] = status
+
+        elapsed = time.time() - started
+        print("  missing batch tasks: {:,}".format(missing_tasks))
+        print("  enumerated {:,} desired runs ({:.2f} secs)".format(len(service_info_list), elapsed))
+
+        return sd
+
+    # API call
+    def get_status_of_jobs(self, workspace, jobs):
+        sd = {}
+
+        if self.batch_client is None:
+            self.create_batch_client()
+
+        job_error_count = 0
+
+        for job in jobs:
+            job_id = job["job_id"]
+
+            try:
+                job_obj = self.batch_client.job.get(job_id)
+            except BaseException as ex:
+                #print("error getting Batch JOB for: {}, ex: {}".format(job_id, ex.error.code))
+                sd[job_id] = None        # user deleted job from Batch Explorer; which would have cancelled it if it was running
+                job_error_count += 1
+            else:
+                status = job_obj.status
+                print("job: {}, status: {}".format(job_id, status))
+                sd[job_id] = job_obj.status
+
+        if job_error_count:
+            print("Batch jobs not found: {:,}".format(job_error_count))
+
+        return sd
+
