@@ -25,7 +25,7 @@ class BackendInterface(Interface):
         pass
     
     # API call
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
+    def build_node_script_and_adjust_runs(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args, cmds=None):
         '''
         Args:
             job_id: the name of job that is being prepared to be submitted
@@ -258,7 +258,7 @@ class BackendInterface(Interface):
         pass
     
     # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
         '''
         Args:
             cmds - list of cmds for appending generated cmds to
