@@ -116,12 +116,12 @@ class ITP(AzureML):
         # self.append(cmds, "xt --version")
         # self.append(cmds, "python $xt_file --version")
 
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
 
-        self.append(cmds, "mkdir -p {}".format(dest_dir))
-        self.append(cmds, "cp -r -v /var/log/compute/00_stdout.txt {}".format(dest_dir))
-        #self.append(cmds, "cp -r -v /var/log {}".format(dest_dir))
-        self.append(cmds, "cp -r -v $XT_ORIG_WORKDIR/logs {}".format(dest_dir))
+        ca.append(cmds, "mkdir -p {}".format(dest_dir))
+        ca.append(cmds, "cp -r -v /var/log/compute/00_stdout.txt {}".format(dest_dir))
+        #ca.append(cmds, "cp -r -v /var/log {}".format(dest_dir))
+        ca.append(cmds, "cp -r -v $XT_ORIG_WORKDIR/logs {}".format(dest_dir))
 
     # API call
     def get_log_files_dir(self, args):
