@@ -51,10 +51,21 @@ class Philly(BackendBase):
         self.vc = utils.safe_value(compute_def, "vc")
         self.queue = utils.safe_value(compute_def, "queue")
         
+    # API 
     def get_name(self):
         return "philly"
 
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
+    # API call
+    def get_timestamped_status_of_nodes(self, service_name, service_infos):
+        # return "nodes not recognized"
+        sd = {"node"+str(i): (None,None) for i in range(len(service_infos))}
+        return sd
+    
+    # API 
+    def get_location(self, service):
+        return None
+        
+    def build_node_script_and_adjust_runs(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args, cmds=None):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  This 
         base implementation does so by generating a new script file and adding it to the snapshot_dir.
@@ -930,20 +941,20 @@ class Philly(BackendBase):
         file_utils.write_text_file(fn_result, result)
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/logs/1")
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/stdout/1")
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/amlogs/1")
 
         # wait for stdout to flush to disk (unreliable with 10 secs)
         sleep_secs = 30
-        self.append(cmds, "echo wait 30 secs for our STDOUT to flush to file", echo=False)
-        self.append(cmds, "sleep {}".format(sleep_secs))
+        ca.append(cmds, "echo wait 30 secs for our STDOUT to flush to file", echo=False)
+        ca.append(cmds, "sleep {}".format(sleep_secs))
 
-        self.append(cmds, "tail $PHILLY_JOB_DIRECTORY/stdout/1/stdout.txt")
+        ca.append(cmds, "tail $PHILLY_JOB_DIRECTORY/stdout/1/stdout.txt")
 
         cmd = "cp -r $PHILLY_JOB_DIRECTORY/stdout/* {}".format(dest_dir)
-        self.append(cmds, cmd)
+        ca.append(cmds, cmd)
 
     # NEW API call
     def get_queue_jobs(self, target_def, username, queues, workspace):
