@@ -21,6 +21,8 @@ from xtlib import run_helper
 from xtlib import process_utils
 from xtlib import box_information
 from xtlib.xt_client import XTClient
+from xtlib.backends.mount_helper import MountHelper
+from xtlib.backends.node_scriptor import NodeScriptor
 
 from xtlib.console import console
 from xtlib.helpers.feedbackParts import feedback as fb
@@ -49,6 +51,16 @@ class PoolBackend(BackendBase):
     def get_name(self):
         return "pool"
 
+    # API call
+    def get_timestamped_status_of_nodes(self, service_name, service_infos):
+        # return "nodes not recognized"
+        sd = {"node"+str(i): (None,None) for i in range(len(service_infos))}
+        return sd
+
+    # API 
+    def get_location(self, service):
+        return None
+
     # API 
     def provides_container_support(self):
         '''
@@ -57,111 +69,143 @@ class PoolBackend(BackendBase):
         '''
         return False
 
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
+    def build_node_script_and_adjust_runs(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args, cmds=None):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  This 
         base implementation does so by generating a new script file and adding it to the snapshot_dir.
         '''
-        gen_for_windows = self.set_gen_for_windows(self.is_windows, args)
-
-        store_data_dir, data_action, data_writable, store_model_dir, model_action, model_writable,  \
-            storage_name, storage_key = self.get_action_args(args)
-
+        nowrap = args["nowrap"]
+        is_direct_run = args["direct_run"]
         docker_cmd = args["docker_cmd"]
 
         # local or POOL of vm's
-        fn_wrapped = None     # we use same generated script on each box/job
         data_local = args["data_local"]
         model_local = args["model_local"]
 
-        # if data_local and docker_cmd:
-        #     data_local = "/usr/data"
+        # build the node script
+        first_run = job_runs[0]
+        fn_script, fn_inner = self.build_node_script(first_run, args)
 
         if docker_cmd:
             # best to use blobfuse since it will be available and will be closer to testing remote node 
             data_local = None
             model_local = None
 
-        for i, box_runs in enumerate(job_runs):
-            # wrap the user commands in FIRST RUN of each box (apply data/model actions)
-            br = box_runs[0]
-            box_info = br["box_info"]
+        for i, box_run in enumerate(job_runs):
+            box_info = box_run["box_info"]
             box_name = box_info.box_name
-            box_secret = br["box_secret"]
+            box_secret = box_run["box_secret"]
             actions = box_info.actions
             node_id = utils.node_id(i)
 
-            is_windows = box_info.box_os == "windows"
-
-            run_specs = br["run_specs"]
+            run_specs = box_run["run_specs"]
             cmd_parts = run_specs["cmd_parts"]
-            run_name = br["run_name"]
+            run_name = box_run["run_name"]
 
-            if not fn_wrapped:
+            if is_direct_run:
+                direct_cmd = cmds[i] if i < len(cmds) else cmds[0]
+                run_specs["run_cmd"] = direct_cmd
+            else:
+                direct_cmd = None
+                    
+            self.gather_env_vars_and_store_in_file(i, run_name, direct_cmd, args)
 
-                # we only do this once (for the first box/job)
-                using_localhost =  pc_utils.is_localhost(box_name, box_info.address)
+            # if not fn_wrapped:
 
-                if docker_cmd:
-                    actions = ["data", "model"]
-                else:
-                    # data_local overrides store_data_dir for LOCAL machine
-                    if using_localhost and data_local:
-                        store_data_dir = os.path.join(os.path.expanduser(data_local), store_data_dir)
-                        store_data_dir = file_utils.fix_slashes(store_data_dir, is_linux=not self.gen_for_windows)
+                #     # we only do this once (for the first box/job)
+                #     using_localhost =  pc_utils.is_localhost(box_name, box_info.address)
 
-                        data_action = "use_local"
-                        if not "data" in actions:
-                            actions.append("data")
+                #     if docker_cmd:
+                #         actions = ["data", "model"]
+                #     else:
+                #         # data_local overrides store_data_dir for LOCAL machine
+                #         if using_localhost and data_local:
+                #             store_data_dir = os.path.join(os.path.expanduser(data_local), store_data_dir)
+                #             store_data_dir = file_utils.fix_slashes(store_data_dir, is_linux=not self.gen_for_windows)
 
-                    # model_local overrides store_model_dir for LOCAL machine
-                    if using_localhost and model_local:
-                        store_model_dir = os.path.join(os.path.expanduser(model_local), store_model_dir)
-                        model_action = "use_local"
-                        if not "model" in actions:
-                            actions.append("model")
+                #             data_action = "use_local"
+                #             if not "data" in actions:
+                #                 actions.append("data")
 
-                setup_name = args["setup"]
-                setup = self.config.get_setup_from_target_def(self.compute_def, setup_name)
+                #         # model_local overrides store_model_dir for LOCAL machine
+                #         if using_localhost and model_local:
+                #             store_model_dir = os.path.join(os.path.expanduser(model_local), store_model_dir)
+                #             model_action = "use_local"
+                #             if not "model" in actions:
+                #                 actions.append("model")
 
-                env_vars = self.get_env_vars_for_box(box_name, box_info, i, box_secret, args)
-                pre_cmds = []
+                #     setup_name = args["setup"]
+                #     setup = self.config.get_setup_from_target_def(self.compute_def, setup_name)
 
-                # add commands to set ENV VARs
-                self.append_title(pre_cmds, "SET/EXPORT POOL environment vars:")
+                #     env_vars = self.get_env_vars_for_box(box_name, box_info, i, box_secret, args)
+                #     pre_cmds = []
 
-                for name, value in env_vars.items():
-                    self.append_export(pre_cmds, name, value)
+                #     # add commands to set ENV VARs
+                #     self.append_title(pre_cmds, "SET/EXPORT POOL environment vars:")
 
-                use_username = not docker_cmd
-                install_blobfuse = bool(docker_cmd)
+                #     for name, value in env_vars.items():
+                #         self.append_export(pre_cmds, name, value)
 
-                if self.gen_for_windows:
-                    node_index = "%1"
-                    run_name = "%2"
-                    homebase = os.getenv("HOMEDRIVE") + "\\."
-                    cwd = homebase + "\\.xt\\cwd"
-                else:
-                    node_index = "$1"
-                    run_name = "$2"
-                    homebase = "$HOME"
-                    cwd = "$HOME/.xt/cwd"
+                #     use_username = not docker_cmd
+                #     install_blobfuse = bool(docker_cmd)
 
-                fn_wrapped = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
-                    data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
-                    is_windows=is_windows, sudo_available=False, use_allow_other=False, use_username = use_username, 
-                    setup=setup, pre_setup_cmds=pre_cmds, post_setup_cmds=None, args=args, nonempty=True, 
-                    install_blobfuse=install_blobfuse, node_index=node_index,
-                    run_name=run_name, homebase=homebase, cwd=cwd)   
+                #     if self.gen_for_windows:
+                #         node_index = "%1"
+                #         run_name = "%2"
+                #         homebase = os.getenv("HOMEDRIVE") + "\\."
+                #         cwd = homebase + "\\.xt\\cwd"
+                #     else:
+                #         node_index = "$1"
+                #         run_name = "$2"
+                #         homebase = "$HOME"
+                #         cwd = "$HOME/.xt/cwd"
+
+                # fn_wrapped = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
+                #     data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
+                #     is_windows=is_windows, sudo_available=False, use_allow_other=False, use_username = use_username, 
+                #     setup=setup, pre_setup_cmds=pre_cmds, post_setup_cmds=None, args=args, nonempty=True, 
+                #     install_blobfuse=install_blobfuse, node_index=node_index,
+                #     run_name=run_name, homebase=homebase, cwd=cwd)   
 
             # we update each box's command
-            script_part = "{} {} {}".format(os.path.basename(fn_wrapped), i, run_name)
+            script_part = "{}".format(os.path.basename(fn_script))
             if self.is_windows:
                 sh_parts = [script_part]
             else:
                 sh_parts = ['/bin/bash', '--login', script_part]
             run_specs["cmd_parts"] = sh_parts
 
+    def build_node_script(self, first_run, args):
+
+        run_specs = first_run["run_specs"]
+
+        cmd_parts = run_specs["cmd_parts"]
+        node_cmd = " ".join(cmd_parts)
+        actions = ["data", "model"]
+        manual_docker = bool(args["docker_cmd"]) 
+        homebase = "$HOME"
+  
+        # sudo is available, but causes problems with subsequent MNT directory access
+        sudo_available = False     
+
+        mount_helper = MountHelper(compute_def=self.compute_def, homebase=homebase, sudo_available=sudo_available, actions=actions,
+            use_username=False, use_allow_other=False, nonempty=True, backend=self, config=self.config, args=args)
+        
+        cwd="$HOME/.xt/cwd"
+
+        node_scriptor = NodeScriptor(homebase=homebase, cwd=cwd, controller_cmd=node_cmd, 
+            manual_docker=manual_docker, mount_helper=mount_helper, backend=self, get_env_vars_from_file=True,
+            compute_def=self.compute_def, config=self.config, args=args)
+        
+        fn_script, fn_inner = node_scriptor.generate_script()
+        return fn_script, fn_inner
+
+    def gen_code_prep_cmds(self, ca, cmds, cwd, using_docker):
+
+        if not using_docker:
+            ca.append_title(cmds, "CODE PREP: remove the code zip file:")
+            ca.append(cmds, "rm {}".format(constants.CODE_ZIP_FN))
+
     def submit_job(self, job_id, job_runs, workspace, compute_def, resume_name, 
            repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
 
@@ -182,11 +226,11 @@ class PoolBackend(BackendBase):
                 #     errors.config_error("XT controller already running on box: " + box_name)
 
             # second pass - transfer the main script to each box and start it
-            for i, box_runs in enumerate(job_runs):
-                box_info = box_runs[0]["box_info"]
+            for i, box_run in enumerate(job_runs):
+                box_info = box_run["box_info"]
 
                 # start run on specified box
-                service_node_info = self.run_job_on_box(job_id, box_runs, box_index=i, box_info=box_info, app_info=None, 
+                service_node_info = self.run_job_on_box(job_id, box_run, box_index=i, box_info=box_info, app_info=None, 
                     pool_info=compute_def,  resume_name=resume_name, repeat=repeat_count, 
                     using_hp=using_hp, exper_name=experiment, snapshot_dir=snapshot_dir, args=args)
 
@@ -199,7 +243,7 @@ class PoolBackend(BackendBase):
 
         return job_info, info_by_node
 
-    def run_job_on_box(self, job_id, run_data_list, box_index, box_info, app_info, pool_info,  
+    def run_job_on_box(self, job_id, box_run, box_index, box_info, app_info, pool_info,  
             resume_name=None, repeat=None, using_hp=None, exper_name=None, snapshot_dir=None, args=None):
 
         '''
@@ -222,8 +266,7 @@ class PoolBackend(BackendBase):
         box_os = box_info.box_os
         is_box_windows = (box_os == "windows")
         
-        run_data = run_data_list[0]
-        run_name = run_data["run_name"]
+        run_name = box_run["run_name"]
 
         if pc_utils.is_localhost(box_addr=box_addr):
             psm_client = LocalPsmClient()
@@ -250,22 +293,27 @@ class PoolBackend(BackendBase):
 
         return service_node_info
 
-    def get_env_vars_for_box(self, box_name, box_info, nodex_index, box_secret, args):
-        #box_secrets.set_secret(box_name, box_secret)
+    def gather_env_vars_and_store_in_file(self, node_index, run_name, direct_cmd, args):
 
-        box_os = box_info.box_os
-        max_runs = box_info.max_runs
+        # TODO: figure out a schmeme for delivering node-specific env var to pool boxes
+        # one idea: create a new outer .zip file that contains the env var file + xt_code.zip
+        env_vars = {}
+        scriptor.add_controller_env_vars(env_vars, self.config, node_index=node_index, run_name=run_name, 
+            batch_key=None, direct_cmd=direct_cmd, args=args)
 
-        node_id = "node" + str(nodex_index)
+        # for pool, we move them all into the FN_SET_ENV_VARS file
+        text = ""
+        for key, value in env_vars.items():
+            text += "{}={}\n".format(key, value)
 
-        env_vars = dict(args["env_vars"])
-        scriptor.add_controller_env_vars(env_vars, self.config, box_secret, None)
-
-        # XT_RUN_NAME must be set thru environment variable so that script can be shared
-        #env_vars["XT_RUN_NAME"] = run_name
-
-        return env_vars
+        # write text to set_env file in bootstrap dir
+        bootstrap_dir = args["bootstrap_dir"]
+        fn = bootstrap_dir + "/" + constants.FN_SET_ENV_VARS
 
+        # specify "newline="" to ensure a \r is NOT written with \n char
+        with open(fn, "wt", newline="") as outfile:
+            outfile.write(text)
+            
     def get_client_cs(self, service_node_info):
         '''
         Args:
@@ -435,8 +483,8 @@ class PoolBackend(BackendBase):
             log_name = "stdboth.txt"
         file_path = os.path.basename(log_name)
 
-        # try to first read from job storage (if task has completed)
         if log_source != "live":
+            # try to read from service logs (job has completed), assuming we haven't already read some of LIVE log
             node_index = utils.node_index(node_id)
 
             job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
@@ -448,7 +496,7 @@ class PoolBackend(BackendBase):
                 log_source = "after_logs"
 
         if not found_file:
-            # read LOG FILE
+            # read log from live node
             #start = time.time()
             new_bytes = psm_client.read_log_file(fn_entry, start_offset, end_offset)
             next_offset = start_offset + len(new_bytes)
@@ -471,9 +519,10 @@ class PoolBackend(BackendBase):
                         break
 
             simple_status = self.get_simple_status(service_status)
+            log_source = "live"
 
         return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
-            "service_status": service_status}
+            "service_status": service_status, "log_source": log_source}
 
     # API call
     def get_simple_status(self, status):
@@ -558,17 +607,17 @@ class PoolBackend(BackendBase):
         return job_records
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
         # if using docker, must use /usr/src/logs for log dir
         docker_cmd = args["docker_cmd"]
         
         if self.gen_for_windows:
-            self.append(cmds, "xcopy logs\* {}\\".format(dest_dir))
+            ca.append(cmds, "xcopy logs\* {}\\".format(dest_dir))
         else:
             if docker_cmd:
-                self.append(cmds, "mkdir {} && cp /usr/src/logs/* {}".format(dest_dir, dest_dir))
+                ca.append(cmds, "mkdir {} && cp /usr/src/logs/* {}".format(dest_dir, dest_dir))
             else:
-                self.append(cmds, "mkdir {} && cp logs/* {}".format(dest_dir, dest_dir))
+                ca.append(cmds, "mkdir {} && cp logs/* {}".format(dest_dir, dest_dir))
     # API call
     def get_log_files_dir(self, args):
         docker_cmd = args["docker_cmd"]
