@@ -42,15 +42,12 @@ class Singularity(AzureML):
         '''
         return True   
         
-    # API call
-    def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
-
-        # call AML to do the normal work
-        super().adjust_run_commands(job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args)
+    def update_env_vars(self, env_vars, args):
 
         # move long/secret env vars into a file
         text = ""
-        for key in ["XT_BOX_SECRET", "XT_SERVER_CERT", "XT_STORE_CREDS", "XT_DB_CREDS"]:
+        #for key in ["XT_BOX_SECRET", "XT_SERVER_CERT", "XT_STORE_CREDS", "XT_DB_CREDS"]:
+        for key in ["XT_STORE_CREDS", "XT_DB_CREDS"]:
             value = env_vars[key]
             text += "{}={}\n".format(key, value)
 
@@ -133,18 +130,6 @@ class Singularity(AzureML):
         cmd += " > {} 2>&1".format(fn_log)
         os.system(cmd)
 
-        # # create Amlk8s config
-        # # this line must be included even if you don't need to reference AksCompute
-        # from azureml.contrib.core.compute.k8scompute import AksCompute
-        # from azureml.contrib.core.k8srunconfig import K8sComputeConfiguration
-
-        # k8sconfig = K8sComputeConfiguration()
-        # k8s = dict()
-        # k8s['gpu_count'] = gpu_count
-        # k8s['preemption_allowed'] = preemption_allowed
-        # k8sconfig.configuration = k8s      
-
-        # estimator.run_config.cmk8scompute = k8sconfig
 
     def run_job_on_singularity(self, experiment, trainer, arg_parts, run_name, node_index, args):
         ws = experiment.workspace
@@ -184,8 +169,13 @@ class Singularity(AzureML):
 
         ai = AISuperComputerConfiguration()
         rc.aisupercomputer = ai
-        ai.instance_type = vm_size       # "NC6_v3"     
-        ai.location = location
+        ai.instance_type = vm_size       # "NC6_v3"   
+        
+        if isinstance(location, list):  
+            ai.location = location
+        else:
+            ai.locations = location
+
         ai.sla_tier = sla_tier
         ai.image_version = '' 
         ai.scale_policy.auto_scale_interval_in_sec = 47
@@ -263,6 +253,8 @@ class Singularity(AzureML):
         FN_STDOUT2_LOG = "azureml-logs/70_driver_log.txt"
         FN_STD_OUT_TXT = "user_logs/std_out.txt"
 
+        from_storage = True
+
         for r in range(3):
             # try 3 times, then just bail
             try:
@@ -276,13 +268,6 @@ class Singularity(AzureML):
         node_id =  utils.safe_value(service_node_info, "node_id", "node0")
         workspace = service_node_info["ws"]
 
-        # aml bug workaround
-        #run._output_logs_pattern = "azureml-logs/[\d]{2}.+\.txt"
-
-        # this is the style of log output we want, but as a pull-model
-        # stream log to console
-        #run.wait_for_completion(show_output=True)
-
         new_text = None
         node_status = "queued"
         next_offset = None
@@ -292,13 +277,13 @@ class Singularity(AzureML):
             start_offset = 0
 
         if log_name is None:
-            file_path = "user_logs/std_log.txt"
+            file_path = "std_log.txt"
         else:
             file_path = log_name
 
-        # try to first read from job storage (if task has completed)
         if log_source != "live":
-            node_index = utils.node_index(node_id)
+            # try to read log from job storage (task has completed)
+            node_index = utils.node_index(node_id) 
 
             job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
             if self.store.does_job_file_exist(workspace, job_id, job_path):
@@ -309,7 +294,7 @@ class Singularity(AzureML):
                 log_source = "after_logs"
 
         if not found_file:
-            # refresh run details (status and logs)
+            # read log file from singularity service
             current_details = run.get_details() 
             aml_status = current_details["status"] 
             simple_status = self.get_simple_status(aml_status)
@@ -317,72 +302,55 @@ class Singularity(AzureML):
             available_logs = None
             next_log = None
             log_name = file_path
-
-            # if use_best_log:
-            #     # list of currently available logs
-            #     try:
-            #         available_logs = run._get_logs(current_details)
-            #         if FN_STDOUT_LOG in available_logs:
-            #             next_log = FN_STDOUT_LOG
-            #         elif FN_STDOUT2_LOG in available_logs:
-            #             next_log = FN_STDOUT2_LOG
-            #         else:
-            #             # let AML pick best log 
-            #             from azureml.core import Run
-            #             next_log = Run._get_last_log_primary_instance(available_logs) if available_logs else None
-            #     except BaseException as ex:
-            #         print("error getting available log info: {}".format(ex))
-
-            #     if not next_log:
-            #         next_log = file_path
-        
-            #     if available_logs and log_name != next_log:
-            #         # switching logs, so reset offset and remember new name
-            #         start_offset = 0
-            #         log_name = next_log
+            from_storage = False
+            log_source = "live"
 
             if log_name:
                 # reuse request for better perf (hopefully)
                 log_files = current_details["logFiles"]
-                if log_name in log_files:
-                    url = log_files[log_name]
+                aml_log_path = "user_logs/" + log_name
 
-                    if not self.request:
-                        self.request = urllib.request.Request(url)
-                    elif self.request.full_url != url:
-                        #self.request.close()
-                        self.request = urllib.request.Request(url)
+                # try to read one of Singularity-hosted service logs
+                for log_path in [log_name, aml_log_path]:
+                    if log_path in log_files:
+                        url = log_files[log_path]
 
-                    try:
-                        with urllib.request.urlopen(self.request) as response:
-                            all_bytes = response.read()
-                    except:
-                        # treat any error as "log not yet available"
-                        all_bytes = b""
+                        if not self.request:
+                            self.request = urllib.request.Request(url)
+                        elif self.request.full_url != url:
+                            #self.request.close()
+                            self.request = urllib.request.Request(url)
 
-                    if end_offset:
-                        new_bytes = all_bytes[start_offset:1+end_offset]
-                    else:
-                        new_bytes = all_bytes[start_offset:]
+                        try:
+                            with urllib.request.urlopen(self.request) as response:
+                                all_bytes = response.read()
+                        except:
+                            # treat any error as "log not yet available"
+                            all_bytes = b""
 
-                    next_offset = start_offset + len(new_bytes)
-                    new_text = new_bytes.decode(encoding)
+                        if end_offset:
+                            new_bytes = all_bytes[start_offset:1+end_offset]
+                        else:
+                            new_bytes = all_bytes[start_offset:]
+
+                        next_offset = start_offset + len(new_bytes)
+                        new_text = new_bytes.decode(encoding)
+                        break
 
         return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
-            "service_status": aml_status}
-
+            "service_status": aml_status, "from_storage": from_storage, "log_source": log_source}
 
     # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+    def add_service_log_copy_cmds(self, ca, cmds, dest_dir, args):
 
-        self.append(cmds, "mkdir -p {}".format(dest_dir))
+        ca.append(cmds, "mkdir -p {}".format(dest_dir))
 
         # copy known singularity log directories 
         for log_dir in ["user_logs"]:    # "azureml-logs", "logs",
             from_dir = "$AZUREML_CR_EXECUTION_WORKING_DIR_PATH/{}".format(log_dir)
 
-            self.append(cmds, "ls -R -lt {}/*".format(from_dir), echo=True)
-            self.append(cmds, "cp -r -v {}/* {}".format(from_dir, dest_dir))
+            ca.append(cmds, "ls -R -lt {}/*".format(from_dir), echo=True)
+            ca.append(cmds, "cp -r -v {}/* {}".format(from_dir, dest_dir))
 
         # this is now copy in user_logs above
         #self.append(cmds, "cp -r -v $AZUREML_CR_EXECUTION_WORKING_DIR_PATH/user_logs/std_log.txt {}".format(dest_dir))
