@@ -166,7 +166,7 @@ class CmdCore():
         cancel_results_by_boxes = {}
 
         if not job_helper.is_job_id(str(job_id)):
-            errors.syntax_error("not a valid job id: " + str(job_id))
+            errors.user_error("not a valid job id: " + str(job_id))
 
         #console.print("job_id=", job_id)
         text = self.store.read_job_info_file(job_id)
@@ -201,7 +201,7 @@ class CmdCore():
         return cancel_results_by_boxes
 
 
-    def create_context_file_core(self, run_data_list, node_index, job_id, using_hp,  
+    def create_context_file_core(self, run_data, node_index, job_id, using_hp,  
         app_info, exper_name, args):
         ''' create a "node context" JSON data object describing all of the runs we need to do for the 
         current node.  the xt controller will use this file to queue up all of the
@@ -213,35 +213,33 @@ class CmdCore():
         node_context = {"job_id": job_id, "node_index": node_index, "runs": node_runs}
         run_names = ""
 
-        for run_data in run_data_list:
+        run_specs = run_data["run_specs"] 
+        cmd_parts = run_specs["cmd_parts"]
 
-            run_specs = run_data["run_specs"] 
-            cmd_parts = run_specs["cmd_parts"]
+        run_name = run_data["run_name"]
+        box_name = run_data["box_name"]
+        repeat = run_data["repeat"]
+        box_info = run_data["box_info"]
 
-            run_name = run_data["run_name"]
-            box_name = run_data["box_name"]
-            repeat = run_data["repeat"]
-            box_info = run_data["box_info"]
+        #exper_name, app_name, app_info = self.get_exper_name(cmd_parts)
+        # exper_name = app_info.exper_name
+        # app_name = app_info.app_name
 
-            #exper_name, app_name, app_info = self.get_exper_name(cmd_parts)
-            # exper_name = app_info.exper_name
-            # app_name = app_info.app_name
+        # build the context for this run
+        run_context = self.get_client_context(exper_name, run_name, None, box_info, 
+            node_index=node_index, job_id=job_id, run_specs=run_specs, using_hp=using_hp, repeat=repeat,
+            args=args)
 
-            # build the context for this run
-            run_context = self.get_client_context(exper_name, run_name, None, box_info, 
-                node_index=node_index, job_id=job_id, run_specs=run_specs, using_hp=using_hp, repeat=repeat,
-                args=args)
+        #print("CmdCore: cmd_parts=", cmd_parts)
 
-            #print("CmdCore: cmd_parts=", cmd_parts)
+        run_context.cmd_parts = cmd_parts
+        run_context = run_context.__dict__
 
-            run_context.cmd_parts = cmd_parts
-            run_context = run_context.__dict__
-
-            node_runs.append(run_context)
-            if run_names == "":
-                run_names = run_name
-            else:
-                run_names += ", " + run_name
+        node_runs.append(run_context)
+        if run_names == "":
+            run_names = run_name
+        else:
+            run_names += ", " + run_name
 
         return node_context
         
@@ -293,11 +291,31 @@ class CmdCore():
         # fri, lri = self.compute_run_indexes_for_node(total_run_count, node_count, node_count-1)
         # assert lri==total_run_count-1
 
+        context.add_timestamps = self.config.get("logging", "add-timestamps")
         context.log_file_uploads = args["log_file_uploads"]
 
+        hb_str = self.config.get("general", "node-heartbeat")
+        # validate the unix time string
+        hb_secs = utils.shell_time_str_to_secs(hb_str)
+        context.node_heartbeat = hb_secs
+
+        # max_node_duration: validate before storing
+        duration_str = args["max_node_duration"]
+        _ = utils.shell_time_str_to_secs(duration_str)
+        context.max_node_duration = duration_str
+
+        # max_run_duration: validate before storing
+        duration_str = args["max_run_duration"]
+        _ = utils.shell_time_str_to_secs(duration_str)
+        context.max_run_duration = duration_str
+
         context.first_run_index = first_run_index
         context.last_run_index = last_run_index
 
+        context.log_db_stats = self.config.get("logging", "log-db-stats")
+        context.log_controller = self.config.get("logging", "log-controller-details")
+        context.snapshot_dirs = args["snapshot_dirs"]
+
         # notification data from config file
         context.config_events = dict(self.config.get("events"))
         context.team_dict = self.config.get("team")
@@ -312,10 +330,17 @@ class CmdCore():
         context.notify_list = args["notify"]
 
         # add email/sms creds
-        context.email_cs = self.config.get("external-services", "xt-email", "connection-string")
-        context.email_from = self.config.get("external-services", "xt-email", "from")
-        context.sms_cs = self.config.get("external-services", "xt-sms", "connection-string")
-        context.sms_from = self.config.get("external-services", "xt-sms", "from")
+        need_email_creds = bool(args["events"])
+        if need_email_creds:
+            context.email_cs = self.config.get("external-services", "xt-email", "connection-string")
+            context.email_from = self.config.get("external-services", "xt-email", "from")
+            context.sms_cs = self.config.get("external-services", "xt-sms", "connection-string")
+            context.sms_from = self.config.get("external-services", "xt-sms", "from")
+        else:
+            context.email_cs = None
+            context.email_from = None
+            context.sms_cs = None
+            context.sms_from = None
 
         # only apply run delays for jobs with multiple runs
         context.max_delay = args["max_run_delay"] if total_run_count > 1 else 0
@@ -396,7 +421,7 @@ class CmdCore():
         context.total_run_count = args["total_run_count"]
         context.search_style = args["search_style"]
         context.is_parent = context.search_style != "single"
-        context.create_time = utils.get_arrow_time_str()
+        context.create_time = utils.get_arrow_now_str()
 
         # controller sleep time
         context.sleep_on_exit = args["sleep_on_exit"]
@@ -441,8 +466,8 @@ class CmdCore():
 
         context.mirror_dest = args["mirror_dest"]
         context.mirror_request_list = mirror_request_list
-        context.mirror_delay_mins =  self.config.get("logging", "mirror-delay-mins")
-        context.show_mirror_calls =  self.config.get("logging", "show-mirror-calls")
+        context.mirror_delay_mins =  self.config.get("mirroring", "mirror-delay-mins")
+        context.show_mirror_calls =  self.config.get("mirroring", "show-mirror-calls")
         context.grok_server = None   # args["grok_server"]
 
         context.aggregate_dest = args["aggregate_dest"]
@@ -554,7 +579,6 @@ class CmdCore():
         fn_run_context = bootstrap_dir + "/" + constants.FN_RUN_CONTEXT 
         file_utils.ensure_dir_exists(file=fn_run_context)
         
-
         box_runs = job_runs[0]
         cfc = self.create_context_file_core(box_runs, 0, job_id, using_hp, app_info=None, exper_name=experiment, 
             args=args)
@@ -586,16 +610,16 @@ class CmdCore():
         # for EACH NODE, collect and adjust runs 
         context_by_nodes = {}
 
-        for i, box_runs in enumerate(job_runs):
+        for i, box_run in enumerate(job_runs):
 
-            node_context = self.create_context_file_core(box_runs, i, job_id, using_hp,  
+            node_context = self.create_context_file_core(box_run, i, job_id, using_hp,  
                 app_info=None, exper_name=experiment, args=args)
 
             node_id = "node" + str(i)
             context_by_nodes[node_id] = node_context
 
-            new_box_runs = self.adjust_box_runs_for_controller(i, box_runs)
-            job_runs[i] = new_box_runs
+            new_box_run = self.adjust_box_run_for_controller(i, box_run)
+            job_runs[i] = new_box_run
 
         # write 1st file to SNAPSHOT (MRC file)
         fn_context = bootstrap_dir + "/" + constants.FN_MULTI_RUN_CONTEXT 
@@ -640,12 +664,9 @@ class CmdCore():
 
         return [fn_context, fn_script]
 
-    def adjust_box_runs_for_controller(self, node_index, box_runs):
-        first_box_run = box_runs[0]
-        box_info = first_box_run["box_info"]
-
-        # use first run_name as name of the controller-mode run
-        run_name = first_box_run["run_name"]
+    def adjust_box_run_for_controller(self, node_index, box_run):
+        box_info = box_run["box_info"]
+        run_name = box_run["run_name"]
 
         # these fields are not used when running controller
         repeat_count = None 
@@ -660,10 +681,9 @@ class CmdCore():
 
         # create a single run_data that will run the controller for this node
         run_data = {"box_info": box_info, "box_name": box_info.box_name, "repeat": repeat_count, "run_name": run_name, 
-            "box_secret": first_box_run["box_secret"], "run_specs": run_specs, "node_index": node_index}
-        new_box_runs = [run_data]
+            "box_secret": box_run["box_secret"], "run_specs": run_specs, "node_index": node_index}
 
-        return new_box_runs
+        return run_data
 
     def upload_before_files_to_job(self, job_id, source_dir, store_dir, omit_list, zip_type, upload_type, args):
 
