@@ -11,7 +11,7 @@
 '''
 import sys
 import time
-import codecs
+import datetime
 
 class Console():
     '''
@@ -26,6 +26,7 @@ class Console():
     '''
     def __init__(self, level="normal"):
         self.level = level      
+        self.add_timestamps = False
 
         # we set these now for XTLib clients, but XT calls init_timing() to override these settings
         self.xt_started = time.time()
@@ -65,6 +66,7 @@ class Console():
         self.early_output.append( (target_level, text) )
 
     def print(self, *objects, sep=' ', end='\n', file=sys.stdout, flush=False, exit_time=0):
+
         if self.level == None:
             self.early_print("normal", *objects, sep=sep, end=end, file=file, flush=flush)
         elif self.level != "none":
@@ -89,6 +91,12 @@ class Console():
             if enc != 'UTF-8':
                 text = text.encode(enc, errors='backslashreplace').decode(enc)
 
+            # add time for platforms that don't log time for console output
+            if self.add_timestamps:
+                now = datetime.datetime.now()
+                now_str = now.strftime("%Y-%m-%d %H:%M:%S")
+                text = "@{}: {}".format(now_str, text) 
+
             print(text, end=end, file=file, flush=flush)
 
         if self.capturing_output:
