@@ -11,7 +11,7 @@ here, check to see if you need to also change it in psm.py.
 
 # Sync the BUILD string changes with: setup.py, CHANGELOG.md
 # Also, consider setting pytorch-xtlib docker image (in facotry_config.yaml) to new version just before releasing new XT
-BUILD = "version: 0.0.297, build: Dec-20-2022"
+BUILD = "version: 0.0.307, build: Mar-30-2023"
     
 INFO_CONTAINER_V1 = "xt-store-info"
 INFO_CONTAINER_V2 = "00-store-info"
@@ -35,8 +35,8 @@ AZURE_ERRORS_FN = "azure_errors.txt"
 
 AFTER = "__after__"
 WINDOWS_SERVICE_LOGS = "__after__\\service_logs"
-WINDOWS_XT_LOGS = "__after__\\xt_logs"
-LINUX_XT_LOGS = "__after__/xt_logs"
+WINDOWS_AFTER_LOGS = "__after__\\xt_logs"
+LINUX_AFTER_LOGS = "__after__/xt_logs"
 
 # workspace directory and files
 WORKSPACE_DIR = "__ws__"
@@ -133,10 +133,11 @@ FN_SERVER_CERT_PUBLIC = "~/.xt/cwd/__xt_server_cert_public__.pem"
 
 # primary setup cmd file
 FN_DOCKER_PULL_SH = "__docker_pull__.sh"
-FN_DOCKER_PULL_BAT = "__docker_pull__.bat"
-FN_WRAPPED_SH = "__wrapped__.sh"
-FN_WRAPPED_BAT = "__wrapped__.bat"
-FN_EV = "__dockev__.txt"
+#FN_DOCKER_PULL_BAT = "__docker_pull__.bat"
+#FN_WRAPPED_SH = "__wrapped__.sh"
+#FN_WRAPPED_BAT = "__wrapped__.bat"
+FN_DOCKER_ENV = "__docker_env__.txt"
+
 FN_AML_SHIM = "__aml_shim__.py"
 FN_SET_ENV_VARS = "__set_env_vars__.sh"
 FN_BUILD_STEPS_TEMPLATE = "build_steps_template.txt"
@@ -145,11 +146,14 @@ FN_BATCH_HELPER = "__batch_helper__.sh"
 FN_BATCH_NODE_ID = "__batch_node_id__.py"
 
 # if using docker, most cmds are stored in the secondary inner file
-FN_INNER_SH = "__inner__.sh"
-FN_INNER_BAT = "__inner__.bat"
+# FN_INNER_SH = "__inner__.sh"
+# FN_INNER_BAT = "__inner__.bat"
+
+FN_INNER_SCRIPT = "__inner_script__.sh"
+FN_NODE_SCRIPT = "__node_script__.sh"
 
 LOGS_DIR = "__logs__"
-FN_IMAGE_NAME =  "__docker_image_name.txt"
+FN_IMAGE_NAME =  "__docker_image_name__.txt"
 
 distribution_types = [
     "choice", "randint", 
@@ -175,4 +179,6 @@ HPARAM_RUNSETS = "hyperparameter-runsets"
 HPARAM_COMMAND = "command"
 
 # used to communicate user requests from XT controller to running scripts
-FN_USER_REQUEST = "~/.xt/cwd/__user_request__.txt"
\ No newline at end of file
+FN_USER_REQUEST = "~/.xt/cwd/__user_request__.txt"
+
+ONGOING_PREFIX = "__ongoing__"
