@@ -16,11 +16,13 @@ import sys
 import math
 import time
 import random
+import wandb
 import argparse
 import numpy as np
 
-print("------ miniMnist starting ------")
 print("current conda=", os.getenv("CONDA_DEFAULT_ENV"))
+print("WANDB_API_KEY", os.getenv("WANDB_API_KEY"))
+print("GROK_API_KEY", os.getenv("GROK_API_KEY"))
 
 import torch
 import torch.nn as nn
@@ -56,8 +58,9 @@ def parse_cmdline_args():
     # LOGGING
     parser.add_argument('--log-interval', type=int, default=1, help='how many epochs to wait before logging training status')
     parser.add_argument('--tensorboard', type=int, default=1, help='if tensorboard logging is enabled')
-    parser.add_argument('--xtlib', type=int, default=1, help='if xtlib usage is enabled')
-    parser.add_argument('--env-vars', type=int, default=1, help='=1 to show name/value of all environment variables')
+    parser.add_argument('--xtlib', type=int, default=1, help='if xtlib logging is enabled')
+    parser.add_argument('--wandb', type=int, default=0, help='if wandb logging is enabled')
+    parser.add_argument('--env-vars', type=int, default=0, help='=1 to show name/value of all environment variables')
     parser.add_argument('--tag-job', type=int, default=0, help='set =1 to tag associated with plotted_metrics=test-acc')
 
     # MINI MNIST
@@ -87,107 +90,32 @@ def parse_cmdline_args():
     args = parser.parse_args()
     return args
 
-class Mlp(nn.Module):
-    '''
-    MLP with specified middle layers and a RELU activation function.
-    '''
-
-    def __init__(self, in_dim, middle_dims, out_dim, batch_norm=False, dropout=0) -> None:
-        super().__init__()
-
-        next_in = in_dim
-        if not isinstance(middle_dims, list):
-            middle_dims = [middle_dims]
-
-        dims = middle_dims + [out_dim]
-        self.layers = nn.ModuleList()
-
-        if batch_norm:
-            self.batch_norms = nn.ModuleList()
-        else:
-            self.batch_norms = None
-
-        for dim in dims:
-            self.layers.append( nn.Linear(next_in, dim) )
-            if batch_norm:
-                self.batch_norms.append( nn.BatchNorm1d(dim) )
-            next_in = dim
-
-        self.last_layer = self.layers[-1]
-        self.dropout = dropout
+class SimpleCnn(nn.Module):
+    def __init__(self):
+        super(SimpleCnn, self).__init__()
 
+        self.conv1 = nn.Conv2d(1, 32, 3, 1)
+        self.conv2 = nn.Conv2d(32, 64, 3, 1)
+        self.dropout1 = nn.Dropout(0.25)
+        self.dropout2 = nn.Dropout(0.5)
+        self.fc1 = nn.Linear(9216, 128)
+        self.fc2 = nn.Linear(128, 10)
 
     def forward(self, x):
-        output = x
-        for i, layer in enumerate(self.layers):
-            output = layer(output)
-
-            if layer != self.last_layer:
-                if self.batch_norms:
-                    output = self.batch_norms[i](output)
-
-                if self.dropout:
-                    output = F.dropout(output, self.dropout)
-                output = F.relu(output)
-                
+        x = self.conv1(x)
+        x = F.relu(x)
+        x = self.conv2(x)
+        x = F.relu(x)
+        x = F.max_pool2d(x, 2)
+        x = self.dropout1(x)
+        x = torch.flatten(x, 1)
+        x = self.fc1(x)
+        x = F.relu(x)
+        x = self.dropout2(x)
+        x = self.fc2(x)
+        output = F.log_softmax(x, dim=1)
         return output
 
-class SimpleCNN(nn.Module):
-    def __init__(self, num_mid_conv=0, channels1=20, channels2=50, kernel_size=5, mlp_units=500, batch_norm=False, dropout=0, square_mlp=False):
-        super(SimpleCNN, self).__init__()
-
-        # input to conv1: channels=1 (grayscale), W=28, H=28
-        self.conv1 = nn.Conv2d(1, channels1, kernel_size, 1)
-        self.dropout = dropout
-
-        for i in range(num_mid_conv):
-            name = "conv" + str(2+i)
-            convX = nn.Conv2d(channels1, channels1, kernel_size, 1, padding=2)
-            setattr(self, name, convX)
-
-        self.conv_last = nn.Conv2d(channels1, channels2, kernel_size, 1)
-        self.channels2 = channels2
-        self.num_mid_conv = num_mid_conv
-
-        sz1 = 28-kernel_size+1
-        assert sz1 % 2 == 0
-        pool1 = sz1 // 2
-
-        sz2 = pool1-kernel_size+1
-        assert sz2 % 2 == 0
-        pool2 = sz2 // 2
-
-        self.factor = pool2*pool2*channels2
-        #print("sz1=", sz1, ", sz2=", sz2, ", factor=", self.factor)
-        
-        self.mlp = Mlp(self.factor, mlp_units, 10, batch_norm=batch_norm, dropout=dropout)
-
-    def forward(self, input):
-        dropout = self.dropout
-
-        # first CONV2D
-        x = F.relu(self.conv1(input))
-
-        # middle CONV2D's
-        for i in range(self.num_mid_conv):
-            name = "conv" + str(2+i)
-            convX = getattr(self, name)
-            x = F.relu(convX(x))
-            if dropout:
-                x = F.dropout(x, dropout)
-
-        x = F.max_pool2d(x, 2, 2)
-        x = F.relu(self.conv_last(x))
-        if dropout:
-            x = F.dropout(x, dropout)
-        x = F.max_pool2d(x, 2, 2)
-        x = x.view(-1, 4*4*self.channels2)
-
-        x = self.mlp(x)
-
-        return F.log_softmax(x, dim=1)
-
-
 class Trainer():
     def __init__(self):
         pass
@@ -306,7 +234,7 @@ class Trainer():
             total_steps += steps
 
             if epoch % args.log_interval == 0:
-                elapsed = time.time() - start
+                elapsed = max(1, time.time() - start)
                 steps_per_sec = epoch_steps/elapsed
                 #print("{} epoch(s) training took: {:.2f} secs".format(args.log_interval, elapsed))
 
@@ -341,10 +269,13 @@ class Trainer():
         # print to console
         print(msg)
 
+        if args.wandb:
+            wandb.log({"loss": train_loss, "acc": train_acc, "epoch": epoch})
+
         if xt_run:
             # log TRAINING stats
             train_loss = np.float64(train_loss)   # test XT's ability to convert np floats to python
-            xt_run.log_metrics({"epoch": epoch, "loss": train_loss, "acc": train_acc}, step_name="epoch", stage="train")
+            xt_run.log_metrics({"epoch": epoch, "train_loss": train_loss, "train_acc": train_acc}, step_name="epoch", stage=None)
 
             # log EVAL/TEST stats half as often
             if (epoch / args.log_interval ) % 2 == 0:
@@ -356,12 +287,15 @@ class Trainer():
                     # exit without error
                     sys.exit(0)
 
-    def test_model_and_log_metrics(self, run, model, device, epoch, args):
+    def test_model_and_log_metrics(self, xt_run, model, device, epoch, args):
         # TEST the model
         test_loss, test_acc = self.test(args, model, device)
         
         # log TEST METRICS
-        run.log_metrics({"epoch": epoch, "loss": test_loss, "acc": test_acc}, step_name="epoch", stage="test")
+        xt_run.log_metrics({"epoch": epoch, "dev_loss": test_loss, "dev_acc": test_acc}, step_name="epoch", stage=None)
+
+        if args.wandb:
+            wandb.log({"test_loss": test_loss, "test_acc": test_acc, "epoch": epoch})
 
         return test_loss, test_acc
 
@@ -425,7 +359,7 @@ class Trainer():
                 self.xt_run.tag_job( {"plotted_metric": "test_acc"} )
 
             # if "call search API" test was specified and if we are running under XT
-            if args.search_api and run.run_name:
+            if args.search_api and self.xt_run.run_name:
                 fn_sweeps = os.path.join(file_utils.get_my_file_dir(__file__), "miniSweeps.yaml")
                 sweeps = file_utils.load_yaml(fn_sweeps)
                 hp_space_dict = sweeps[constants.HPARAMS_DIR]
@@ -468,6 +402,9 @@ class Trainer():
         tb_path = "tb_logs"
         self.init_xt_run(logging, tb_path, args)
 
+        if args.wandb:
+            wandb.init(project="miniMnist", dir="../")
+
         self.init_datasets(data_dir, use_cuda, args)
 
         model = self.init_model(device, args)
@@ -477,9 +414,8 @@ class Trainer():
     def init_model(self, device, args):
         print("creating CNN model...")
 
-        model = SimpleCNN(num_mid_conv=args.mid_conv, channels1=args.channels1, channels2=args.channels2, kernel_size=args.kernel_size, 
-            mlp_units=args.mlp_units, batch_norm=args.batch_norm, dropout=args.dropout, square_mlp=args.square_mlp)
-            
+        model = SimpleCnn()
+
         gpu_count = torch.cuda.device_count()
 
         if args.cuda==0:
@@ -533,6 +469,18 @@ class Trainer():
         self.args = parse_cmdline_args()
         args = self.args
 
+        print("args.env_vars=", args.env_vars)
+
+        if args.env_vars:
+            print("environment variables:\n")
+            ev = dict(os.environ)
+            print(ev)
+            print("")
+
+        # run_name = os.getenv("XT_RUN_NAME")
+        # if run_name and run_name.endswith(".2"):
+        #     raise Exception("intentional error for runXXX.2")
+
         model, device, live_output_test_dir = self.init_stuff()
 
         start_epoch = 1
@@ -548,6 +496,9 @@ class Trainer():
         if xt_run:
             xt_run.log_hparams(self.hp_dict)
 
+        if args.wandb:
+            wandb.config.update(self.hp_dict)
+
         # print hyperparameters
         print("hyperparameters:", self.hp_dict)
         print()
