@@ -26,6 +26,21 @@ class KeyPressChecker:
 
         return self
 
+    def _get_windows_char(self, encoding):
+        bb = msvcrt.getch()
+
+        if bb == '\000' or bb == '\xe0':
+            # special function key indicator; next keycode is actual key
+            bb = msvcrt.getch()
+
+        try:
+            ch = bb.decode(encoding)
+        except BaseException as ex:
+            #console.print("KeyPressChecker exception decoding bb={}: {}".format(bb, ex))
+            ch = ""
+
+        return ch
+
     def getch_wait(self, encoding='utf-8'):
         '''
         Returns:
@@ -36,11 +51,7 @@ class KeyPressChecker:
         ch = None
         if self.is_windows:
             import msvcrt
-            bb = msvcrt.getch()
-            if bb == '\000' or bb == '\xe0':
-                # special function key indicator; next keycode is actual key
-                bb = msvcrt.getch()
-            ch = bb.decode(encoding)
+            ch = self._get_windows_char(encoding)
         else:
             # linux
             ch = sys.stdin.read(1)
@@ -57,15 +68,7 @@ class KeyPressChecker:
         if self.is_windows:
             import msvcrt
             if msvcrt.kbhit():
-                bb = msvcrt.getch()
-                if bb == '\000' or bb == '\xe0':
-                    # special function key indicator; next keycode is actual key
-                    bb = msvcrt.getch()
-                try:
-                    ch = bb.decode(encoding)
-                except BaseException as ex:
-                    console.print("KeyPressChecker exception decoding bb={}: {}".format(bb, ex))
-                    ch = ""
+                ch = self._get_windows_char(encoding)
         else:
             # linux
             if select.select([sys.stdin], [], [], 0) == ([sys.stdin], [], []):
