@@ -15,7 +15,7 @@ class Scanner():
         self.prev_index = 0
         #console.print("Scanner created, text=", text)
 
-    def scan(self, allow_extended_ids=True):
+    def scan(self, allow_extended_ids=True, force_string=False):
         self.prev_index = self.index
         
         text = self.text
@@ -40,14 +40,33 @@ class Scanner():
             else:
                 ch_next = None
 
-            if allow_extended_ids and ch.lower() in '~/\\_abcdefghijklmnopqrstuvwxyz*?$-' or (ch == "." and ch_next in [".", "/", "\\"]):
+            if ch == '"' or ch == "'" or ch == '`':
+                # scan a STRING
+                quote = ch
+                last_ch = ""
+                while self.index < self.len:
+                    if text[self.index] == quote and last_ch != "\\":
+                        break
+                    last_ch = text[self.index]
+                    self.index += 1
+
+                if text[self.index] != quote:
+                    errors.raise_error("Unterminated string at offset=" + str(start) + " in cmd: " + text)
+
+                self.token_type = "string"
+                self.index += 1        # skip over the ending quote
+                self.token = text[start+1:self.index-1]
+                # un-embed contained quotes
+                self.token = self.token.replace("\\" + quote, quote)
+                
+            elif allow_extended_ids and (force_string or ch.lower() in '~/\\_abcdefghijklmnopqrstuvwxyz*?$-' or (ch == "." and ch_next in [".", "/", "\\"])):
                 # scan an ID or FILENAME or a WILDCARD or box-addr
                 while self.index < self.len and text[self.index].lower() in '/@._-abcdefghijklmnopqrstuvwxyz0123456789?*:/\\':
                     self.index += 1
                 self.token_type = "id"
                 self.token = text[start:self.index]
 
-            elif not allow_extended_ids and ch.lower() in '_abcdefghijklmnopqrstuvwxyz': 
+            elif not allow_extended_ids and (force_string or ch.lower() in '_abcdefghijklmnopqrstuvwxyz'): 
                 # scan a simple ID
                 while self.index < self.len and text[self.index].lower() in '_abcdefghijklmnopqrstuvwxyz0123456789.-':
                     self.index += 1
@@ -70,25 +89,6 @@ class Scanner():
                     self.token_type = "number"
                     self.token = text[start:self.index]
 
-            elif ch == '"' or ch == "'" or ch == '`':
-                # scan a STRING
-                quote = ch
-                last_ch = ""
-                while self.index < self.len:
-                    if text[self.index] == quote and last_ch != "\\":
-                        break
-                    last_ch = text[self.index]
-                    self.index += 1
-
-                if text[self.index] != quote:
-                    errors.raise_error("Unterminated string at offset=" + str(start) + " in cmd: " + text)
-
-                self.token_type = "string"
-                self.index += 1        # skip over the ending quote
-                self.token = text[start+1:self.index-1]
-                # un-embed contained quotes
-                self.token = self.token.replace("\\" + quote, quote)
-                
             else:
                 # scan a special char
                 self.token_type = "special"
