@@ -438,14 +438,12 @@ class XTConfig():
         '''
         backend_name = backend.get_name()
         compute_def = backend.compute_def
+        desc = self.get_target_desc_from_def(target, backend_name, compute_def)
+        return desc
 
-        if backend_name == "philly":
-            cluster = backend.cluster
-            vc = backend.vc
-            queue = backend.queue
-            target_str = "target={}, backend={}, cluster={}, vc={}, queue={}".format(target, backend_name, cluster, vc, queue)
+    def get_target_desc_from_def(self, target, backend_name, compute_def):
 
-        elif backend_name in ["aml", "itp", "singularity"]:
+        if backend_name in ["aml", "itp", "singularity"]:
             service = compute_def["service"]
             compute = compute_def["compute"]
             target_str = "target={}, backend={}, service={}, compute={}".format(target, backend_name, service, compute)
