@@ -14,7 +14,7 @@ class HpSetFormatter():
     def parse_and_sort_hp_set(self, value):
         # fix up to be legal json
         if value:
-            hp_set_str = value.replace("'", '"')
+            hp_set_str = value.replace("'", '"').replace(": None", ": null")
             hp_set = json.loads(hp_set_str)
 
             # sort by hp names
@@ -47,12 +47,16 @@ class HpSetFormatter():
 
     def format_hpset_changed(self, hp_set):
         value = self.parse_and_sort_hp_set(hp_set)
-        value = str(value)
+        value_ = str(value)
 
         # if value not in self.hp_set_names:
         #     self.hp_set_names[value] = "hp_set_" + str(1 + len(self.hp_set_names))
 
-        new_value = self.changed_set_names[value]
+        new_value = self.changed_set_names[value_]
+
+        # if nothing changes, just pick the first hparam/value
+        if len(new_value) == 0:
+            new_value = next(iter(value.items()))
         return new_value
 
     def build_hp_set_names(self, records):
