@@ -671,7 +671,7 @@ class SearchHistory():
             # merge fd into hd
             hd.update(fd)
             records.append(hd)
-
+ 
             # calc data for plot
             y = trophy.runset.post_mean
             d = trophy.runset.post_st_dev
@@ -686,7 +686,8 @@ class SearchHistory():
         builder = ReportBuilder(self.xt_config, self.xtstore)
 
         # build report (borrow config file settings from run-reports)
-        text, row_count = builder.build_formatted_table(records, avail_cols=avail_cols, col_list=hd.keys(), 
+        col_list = list(hd.keys())
+        text, row_count = builder.build_formatted_table(records, avail_cols=avail_cols, col_list=col_list, 
             report_type="run-reports", max_col_width=50)
 
         # print report
