@@ -14,6 +14,7 @@ import numpy as np
 
 import xtlib.xt_run as xt_run
 from xtlib import cs_utils
+from xtlib import pc_utils as pc
 from xtlib import report_builder
 from xtlib.client import Client
 from xtlib.runner import Runner
@@ -238,11 +239,17 @@ class ImplCompute(ImplBase):
         if node_index is None:
             node_index = 0
 
+        # enable ANSI escape chars on Windows 10 for color output
+        pc.enable_ansi_escape_chars_on_windows_10()
+
         backend, job_info = job_helper.get_job_info_with_backend(self.store, self.core, workspace=workspace, 
             job_id=job_id, node_index=node_index)
 
         node_id = utils.node_id(node_index)
         service_info_by_node = job_info["service_info_by_node"]  
+        if not service_info_by_node:
+            errors.general_error("job is missing service_info_by_node table (terminated during submit?): {}".format(job_id))
+
         service_node_info = service_info_by_node[node_id]
         pool_info = job_info["pool_info"]
         node_count = len(service_info_by_node)
@@ -274,6 +281,7 @@ class ImplCompute(ImplBase):
         last_status_shown = None
         log_source = None
         request_dict = {"t": "predict_train", "e": "predict_eval", "?": "show_hparams", "=": "set_hparam", "h": "help"}
+        from_storage = None
 
         try:
             with KeyPressChecker() as checker:
@@ -333,6 +341,7 @@ class ImplCompute(ImplBase):
                         new_log = result["log_name"]
                         file_path = utils.safe_value(result, "file_path")
                         log_source = utils.safe_value(result, "log_source")
+                        from_storage = utils.safe_value(result, "from_storage")
 
                         if first_call or new_log != log_name:
                             log_name = new_log                
@@ -403,7 +412,11 @@ class ImplCompute(ImplBase):
         if show_final_status and last_status_shown != simple_status:
             console.print("==> node{} status: {} ({})".format(node_index, service_status, simple_status))
 
-        console.print("end of monitoring: {}".format(job_id))
+        source_msg = ""
+        if from_storage is not None:
+            source_msg = " (from storage)" if from_storage else " (from service)"
+
+        console.print("end of monitoring: {}{}".format(job_id, source_msg))
 
     def monitor_with_jupyter(self, workspace, run_name):
         if not self.is_aml_ws(workspace):
@@ -420,7 +433,7 @@ class ImplCompute(ImplBase):
 
     #---- COLLECT LOGS command ----
     @argument(name="run-names", type="str_list", help="comma-separated list of run names")
-    @argument(name="log_path", help="the wildcard path of the log files to collect")
+    @argument(name="log-path", help="the wildcard path of the log files to collect")
     @option("workspace", default="$general.workspace", type=str, help="the workspace that the runs reside within")
     @example(task="collect the log files matching *tfevent* from run68", text="xt collect logs run68, run69 *tfevent* --work=curious")
     @command(help="copy log files from specified runs (on blob store) to grok server")
@@ -618,9 +631,7 @@ class ImplCompute(ImplBase):
         for br in box_records:
             if "box_name" in br:
                 box_name = br["box_name"] 
-                if not box_name in runs_by_box:
-                    runs_by_box[box_name] = []
-                runs_by_box[box_name].append(br)
+                runs_by_box[box_name] = br
 
         return runs_by_box
 
@@ -635,11 +646,10 @@ class ImplCompute(ImplBase):
 
         cancel_results_by_box = {}
 
-        for box_name, runs in runs_by_box.items():
-            first_run = runs[0]
+        for box_name, box_run in runs_by_box.items():
 
-            run_names = [run["run_name"] for run in runs]
-            compute = first_run["compute"] if "compute" in first_run else "pool"    # some legacy support
+            run_names = [box_run["run_name"]]
+            compute = box_run["compute"] if "compute" in box_run else "pool"    # some legacy support
 
             backend = self.core.create_backend(compute)
             cancel_results = backend.cancel_runs_by_names(workspace, run_names, box_name)
@@ -652,7 +662,7 @@ class ImplCompute(ImplBase):
 
             else:
                 # single box name
-                job_id = first_run["job_id"]
+                job_id = box_run["job_id"]
                 node_index = 0
                 service_id = "pool"
 
@@ -676,28 +686,20 @@ class ImplCompute(ImplBase):
 
             if cancel:
                 console.print("cancelling {} (service: {}):".format(job_id, backend.get_name()))
+                node_end_times = None
 
                 try:
-                    result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
+                    result_by_node, node_end_times = backend.cancel_job(service_job_info, service_info_by_node)
 
                     for node_id, result in result_by_node.items():
                         console.print("  {}: {}".format(node_id, result))
                 except BaseException as ex:
                     console.print("Exception during service cancel call: {}".format(ex))
-                    result_by_node = service_info_by_node
-                # wrap up runs associated with each node
 
-            if wrapup:
+            if wrapup and node_end_times:
                 # whether or not the cancel succeeded, cleanup the RUNS
                 node_list = list(service_info_by_node.keys())
-                runs = run_helper.wrapup_runs_for_nodes(self.store, workspace, job_id, node_list, max_workers)
-
-                # cleanup the NODES
-                for node_index, node in enumerate(node_list):
-                    self.store.database.mark_node_cancelled(workspace, job_id, node_index)
-
-                # cleanup the JOB
-                self.store.database.mark_job_cancelled(workspace, job_id)
+                runs = run_helper.wrapup_runs_nodes_job(self.store, workspace, job_id, node_list, node_end_times, max_workers)
 
     #---- CANCEL REQUESTS command ----
     @argument("names", type="str_list", help="the names of the requests to be cancelled")
@@ -772,9 +774,10 @@ class ImplCompute(ImplBase):
     #---- APPROVE command ----
     @argument("names", type="str_list", help="the names of the requests to be approved")
     @option("workspace", default="$general.workspace", help="the workspace that contains the requests")
+    @flag("force", help="when specified, previously processed requests can be approved")
     @example(task="approve requests request3 and request5", text="xt approve request3, request5")
     @command(help="approves and submits a job for the specified requests ")
-    def approve(self, names, workspace):
+    def approve(self, names, workspace, force):
 
         filter_dict = {"request_id": {"$in": names}}
         records = self.store.database.get_info_for_requests(workspace, filter_dict)
@@ -782,7 +785,7 @@ class ImplCompute(ImplBase):
             status = record["status"]
             name = record["request_id"]
 
-            if status == "requested":
+            if force or status == "requested":
                 username = self.config.get("general", "username")
                 self.store.update_request_status(record, "approved", username)
                 console.print("request approved: {}".format(name))
@@ -790,7 +793,7 @@ class ImplCompute(ImplBase):
                 xt_cmd = record["cmd"]
                 console.print("  running: " + xt_cmd)
 
-                # run the command, capturing the output so we can extract the job_id
+                # submit the job by running xt_cmd, capturing the output so we can extract the job_id
                 console.set_capture(True, clear_captured=True)
                 xt_run.main(xt_cmd)
 
@@ -831,45 +834,45 @@ class ImplCompute(ImplBase):
             text = "  " + text.replace("\n", "\n  ")
             console.print(text)
 
-    #---- CANCEL QUEUES command ----
-    @argument("target", help="the name of the compute target")
-    @option("queues", type="str_list", values=["queued", "running", "completed"], help="the queues to cancel")
-    @option("username", default="$general.username", help="the username to filter entries with")
-    @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
-    @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers for this command")
-    @flag("cancel", default=1, help="call the backend service to cancel the job")
-    @flag("wrapup", default=1, help="wrapup the runs associated with the job")
-    @example(task="cancel my all of my queued jobs on the target philly-rr1", text="cancel queues philly-rr1  --queues=queued ")
-    @command(kwgroup="cancel", help="cancel the queue entries for the specified compute target")
-    def cancel_queues(self, target, username, queues, workspace, max_workers, cancel, wrapup):
-        backend = self.core.create_backend(target, username=username)
-        target_def = self.config.get_target_def(target)
+    # #---- CANCEL QUEUES command ----
+    # @argument("target", help="the name of the compute target")
+    # @option("queues", type="str_list", values=["queued", "running", "completed"], help="the queues to cancel")
+    # @option("username", default="$general.username", help="the username to filter entries with")
+    # @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
+    # @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers for this command")
+    # @flag("cancel", default=1, help="call the backend service to cancel the job")
+    # @flag("wrapup", default=1, help="wrapup the runs associated with the job")
+    # @example(task="cancel my all of my queued jobs on the target philly-rr1", text="cancel queues philly-rr1  --queues=queued ")
+    # @command(kwgroup="cancel", help="cancel the queue entries for the specified compute target")
+    # def cancel_queues(self, target, username, queues, workspace, max_workers, cancel, wrapup):
+    #     backend = self.core.create_backend(target, username=username)
+    #     target_def = self.config.get_target_def(target)
 
-        results_by_queue, columns = backend.get_queue_jobs(target_def, username, queues, workspace)
+    #     results_by_queue, columns = backend.get_queue_jobs(target_def, username, queues, workspace)
         
-        for queue, jobs in results_by_queue.items():
-            cancel_results = []
+    #     for queue, jobs in results_by_queue.items():
+    #         cancel_results = []
 
-            for job in jobs:
+    #         for job in jobs:
 
-                if cancel:
-                    job_id = job["job_id"]
-                    console.print("cancelling {} (service: {}):".format(job_id, backend.get_name()))
-                    result = backend.cancel_job_from_service_dict(job, workspace)
-                    cancel_results.append(result)
+    #             if cancel:
+    #                 job_id = job["job_id"]
+    #                 console.print("cancelling {} (service: {}):".format(job_id, backend.get_name()))
+    #                 result = backend.cancel_job_from_service_dict(job, workspace)
+    #                 cancel_results.append(result)
 
-                if wrapup:
-                    job_id = job["job_id"]
+    #             if wrapup:
+    #                 job_id = job["job_id"]
 
-                    try:
-                        service_job_info, service_info_by_node, backend = \
-                            job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
+    #                 try:
+    #                     service_job_info, service_info_by_node, backend = \
+    #                         job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
 
-                        node_list = list(service_info_by_node.keys())
-                        runs = run_helper.wrapup_runs_for_nodes(self.store, workspace, job_id, node_list, max_workers)
+    #                     node_list = list(service_info_by_node.keys())
+    #                     runs = run_helper.wrapup_runs_nodes_job(self.store, workspace, job_id, node_list, max_workers)
 
-                    except BaseException as ex:
-                        console.print("exception during wrapup: {}".format(ex))
+    #                 except BaseException as ex:
+    #                     console.print("exception during wrapup: {}".format(ex))
 
     #---- CANCEL ALL command ----
     @argument("target", help="the name of the compute target (or backend service) whose runs will be cancelled")
@@ -923,7 +926,7 @@ class ImplCompute(ImplBase):
 
     #---- VIEW CONTROLLER STATUS command ----
     @argument("job-id", help="the name of the job whose node will be restarted")
-    @option("node-index", default=0, help="the 0-based node index to be restarted")
+    @option("node-index", default=0, type=int, help="the 0-based node index to be restarted")
     @example(task="view the status of the 2nd node running job100", text="xt view controller status job100 --node=1")
     @command(kwgroup="view", help="uses the XT controller to view the status of the specified job/node")
     def view_controller_status(self, job_id, node_index):
@@ -1005,7 +1008,7 @@ class ImplCompute(ImplBase):
     @hidden("after-dirs", default="$after-files.after-dirs", help="the files and directories to upload after the run completes")
     @hidden("after-omit", default="$after-files.after-omit", help="the files and directories to omit from after uploading")
     @flag("after-upload", default="$after-files.after-upload", help="when true, the after files are upload when the run completes")
-    @option("aml_compute", type=str, default=DEFAULT_FROM_COMPUTE, help="specifies the name of the AML compute resource for the job")
+    @option("aml-compute", type=str, default=DEFAULT_FROM_COMPUTE, help="specifies the name of the AML compute resource for the job")
     @hidden("capture_setup_cmds", type=bool, default="$logging.capture-setup-cmds", help="specifies if some setup cmds should output to a log file")
     @option("cluster", help="the name of the Philly cluster to be used")
     @option("code", type=str, help="the source of a code snapshot (job id, file path, or blob path) to use in place of current code")
@@ -1014,7 +1017,7 @@ class ImplCompute(ImplBase):
     @flag("code-upload", default="$code.code-upload", help="when true, code is uploaded to job and download for each run of job")
     @hidden("code-zip", default="$code.code-zip", type=str, help="the type zip file to create for the CODE files: none/fast/compress")
     @option("concurrent", default="$hyperparameter-search.concurrent", type=int, help="the maximum concurrent runs to be allowed on each node")
-    @option("data-action", default="$data.data-action", help="the data action to take on the target, before run is started")
+    @option("data-action", default="$data.data-action", values=["none", "download", "mount"], help="the data action to take on the target, before run is started")
     @hidden("data-local", default="$data.data-local", help="the path on the local machine specifying where the data for this job resides")
     @hidden("data-omit", default="$data.data-omit", help="the list wildcard patterns to omit uploading from the data files")
     @hidden("data-share-path", default="$data.data-share-path", help="the path on the data share where data for this run will be stored")
@@ -1040,17 +1043,22 @@ class ImplCompute(ImplBase):
     @flag("fake-submit", help="when True, we skip creation of job and runs and the submit (used for internal testing)")
     @hidden("fn_generated_config", default="$hyperparameter-search.fn-generated-config", type=str, help="the name for the generated HP config file")
     @hidden("framework", default="$aml-options.framework", help="the name of the framework to be installed for the run (pytorch, tensorflow, chainer)")
+    @option("from-request", type=str, help="The name of the request this generated this run)")
     @hidden("fw-version", default="$aml-options.fw-version", help="the framework version number")
     @option("goal-metric", default="$hyperparameter-search.goal-metric", type=float, help="goal for primary metrics (stops CCD hp search when reached)")
+    @option("grid-repeat", type=int, help="the number of times to repeat each hyperparameter set")
     #@hidden("grok-server", default="$logging.grok-server", help="the ip address of the grok-server to be used for mirroring")
     @flag("hold", help="when True, the Azure Pool (VM's) are held open for debugging)")
     @option("hp-config", default="$hyperparameter-search.hp-config", type=str, help="the path of the hyperparameter config file")
     @flag("jupyter-monitor", help="when True, a Jupyter notebook is created to monitor the run")
+    @option("locations", type="str_list", default=DEFAULT_FROM_COMPUTE, help="specifies the region(s) in which to run the job")
     @hidden("log", default="$logging.log", help="specifes if run-related events should be logged")
-    @flag("log-reports", default="$after-files.log-file-uploads", help="if set, log each AFTER file/size before uploading")
-    @hidden("log-file-uploads", default="$logging.log-reports", help="specifies if CPU, GPU, and MEMORY reports should be included in setup log")
+    @flag("log-file-uploads", default="$after-files.log-file-uploads", help="if set, log each AFTER file/size before uploading")
+    @hidden("log-reports", default="$logging.log-reports", help="specifies if various systems reports should be included in setup log")
     @option("low-pri", type=bool, default=DEFAULT_FROM_COMPUTE, help="when true, use low-priority (preemptable) nodes for this job")
     @option("max-passes", default="$hyperparameter-search.max-passes", type=int, help="used to limit number of passes over hyperparameters in hp search")
+    @option("max-node-duration", default="$general.max-node-duration", type=str, help="the maximum time that the node can execute before being terminated")
+    @option("max-run-duration", default="$general.max-run-duration", type=str, help="the maximum time that a run can execute before being terminated")
     @option("max-minutes", default="$hyperparameter-search.max-minutes", type=int, help="the maximum number of minutes the run can execute before being terminated")
     @hidden("max-seconds", type=int, default="$aml-options.max-seconds", help="the maximum number of seconds this run will execute before being terminated")
     @option("max-run-delay", default="$database.max-run-delay", type=float, help="the maximum random delay applied to each run in the job")
@@ -1058,19 +1066,18 @@ class ImplCompute(ImplBase):
     @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers to use during run command")
     @hidden("maximize-metric", default="$general.maximize-metric", help="whether to minimize or maximize value of primary metric")
     @hidden("merge-batch-logs", type=int, default="$logging.merge-batch-logs", help="replaces STDOUT.txt and STDERR.txt with STDBOTH.txt")
-    @hidden("mirror-dest", default="$logging.mirror-dest", help="the location where mirrored information is stored (none, storage, grok)")
-    @hidden("mirror-files", default="$logging.mirror-files", help="the wildcard path that specifies files to be mirrored")
-    @hidden("mirror-log-files", default="$logging.mirror-log-files", help="when true, the job's log file directory is mirrored")
-    @option("model-action", default="$model.model-action", help="the model action to take on the target, before run is started")
+    @hidden("mirror-dest", default="$mirroring.mirror-dest", help="the location where mirrored information is stored (none, storage, grok)")
+    @hidden("mirror-files", default="$mirroring.mirror-files", help="the wildcard path that specifies files to be mirrored")
+    @hidden("mirror-log-files", default="$mirroring.mirror-log-files", help="when true, the job's log file directory is mirrored")
+    @option("model-action", default="$model.model-action",  values=["none", "download", "mount"], help="the model action to take on the target, before run is started")
     @hidden("model-local", default="$model.model-local", help="the path on the local machine specifying where the model for this job resides")
     @hidden("model-share-path", default="$model.model-share-path", help="the model share name for the model")
     @flag("model-writable", default="$model.model-writable", help="when true, a mounted model path can be written to")
     @option("monitor", default="$general.monitor", values=["new", "bg", "same", "none"], help="how to monitor primary run of the new job")
-    @hidden("mount-retry_count", type=int, default="$general.mount-retry-count", help="the number of times to retry each blobfuse mount command")
-    @hidden("mount-retry_interval", type=str, default="$general.mount-retry-interval", help="the time to sleep between retries of blobfuse mount command")
+    @hidden("mount-retry_count", type=int, default="$mounting.mount-retry-count", help="the number of times to retry each blobfuse mount command")
+    @hidden("mount-retry_interval", type=str, default="$mounting.mount-retry-interval", help="the time to sleep between retries of blobfuse mount command")
     @flag("multi-commands"   , help="the script file contains multiple run commands (one per line)")
     @option("node-delay", type=str, default="$general.node-delay", help="maximum time to randomly delay start of node execution")
-    @option("node-repeat", type=int, help="the number of times to repeat each hyperparameter set on each node")
     @hidden("node-script-path", type=str, default="$code.node-script-path", help="relative path to script on computing node")
     @option("nodes", type=int, help="the number of normal (non-preemptable) nodes to allocte for this run")
     @option("notify", type="str_list", help="the list of users to notify for notifications generated by this job")
@@ -1090,7 +1097,7 @@ class ImplCompute(ImplBase):
     @option("runs-per-set", default="$hyperparameter-search.runs-per-set", type=int, help="how many runs to do for each CCD hp set explored")
     @option("schedule", default="$general.schedule", values=["static", "dynamic"], help="specifies if runs are pre-assigned to each node or allocate on demand")
     @option("search-type", values=["random", "grid", "bayesian", "dgd", "ccd"], default="$hyperparameter-search.search-type", help="the type of hyperparameter search to perform")
-    @option("seed", default=None, help="the random number seed that can be used for reproducible HP searches")
+    @option("seed", type=int, default=None, help="the random number seed that can be used for reproducible HP searches")
     @option("service", type=str, default=DEFAULT_FROM_COMPUTE, help="the name of the xt config-specified compute service to use for this job")
     @option("setup", type=str, help="the name of the setup entry to use with the target (from one of the entries in the config file [setups] section")
     @option("sku", default=DEFAULT_FROM_COMPUTE, type=str, help="the name of the Philly SKU to be used (e.g, 'G1')")
@@ -1098,6 +1105,7 @@ class ImplCompute(ImplBase):
     @hidden("slack-factor", default="$early-stopping.slack-factor", type=float, help="(bandit only) specified as a ratio, the delta between this eval and the best performing eval")
     @hidden("slack-amount", default="$early-stopping.slack-amount", type=float, help="(bandit only) specified as an amount, the delta between this eval and the best performing eval")
     @option("sleep-on-exit", type=str, default="0", help="the amount of time to sleep (e.g., '45m'), before controller exits it's process")
+    @flag("snapshot-dirs", default="$logging.snapshot-dirs", help="when True, the node script logs contents of various directories during execution")
     @flag("static-search",  default="$hyperparameter-search.static-search", help="if true, hyperparameter searches are performed at job submit time, when possible")
     @hidden("storage", default=None, type=str, help="name of storage service to be used for this run")
     @option("submit-logs", default=None, help="specifies a directory to which log files for the submit are saved")
@@ -1114,7 +1122,7 @@ class ImplCompute(ImplBase):
     @flag("xtlib-upload", default="$code.xtlib-upload", help="when True, local source code for xtlib is included in the source code snapshot ")
     
     @example(task="run the script miniMnist.py", text="xt run miniMnist.py")
-    @example(task="run the linux command 'sleep3d' on philly", text="xt run --target=philly --code-upload=0 sleep 3d")
+    @example(task="run the linux command 'sleep3d' on singularity", text="xt run --target=singularity --code-upload=0 sleep 3d")
     @command(options_before_args=True, keyword_optional=False, pass_by_args=True, help="submits a script or executable file for running on the specified compute target")
     def run(self, args):
         '''
@@ -1125,7 +1133,7 @@ class ImplCompute(ImplBase):
         - pool (a specified set of computers managed by the user)
         - Azure Batch
         - Azure ML
-        - Philly
+        - Singularity
         '''
         #console.diag("run_cmd")
 
@@ -1172,7 +1180,7 @@ class ImplCompute(ImplBase):
 
             elif monitor in ["bg", "new"]:
                 store_name = self.config.data["store"]
-                cmd = "xt --echo --store={} monitor {} --workspace={}".format(store_name, job_id, workspace)
+                cmd = "xt --echo monitor {} ".format(job_id)
                 noactivate = (monitor=="bg")
                 process_utils.run_cmd_in_new_console(cmd, noactivate=noactivate)
 
