@@ -29,7 +29,7 @@ This module implements the following commands:
 '''     
 
 class ImplHelp(ImplBase):
-    def __init__(self, config, store):
+    def __init__(self, config, store=None):
         super(ImplHelp, self).__init__()
         self.core = CmdCore(config, None, None)
         self.config = config
