@@ -17,6 +17,7 @@ from xtlib import errors
 from xtlib import file_utils
 from xtlib import store_utils
 from xtlib import run_helper
+from xtlib import query_helper
 
 '''
 To add a new job info column (defined at job submit time):
@@ -33,17 +34,14 @@ To add a new job info column (defined at job submit time):
 
 '''
 
-# NEW COLS: sla, low_pri, vm_size, sku, service_name, service_type, aml_compute, compute_target, num_dgd_seeds, schedule
-# LEGACY column for old jobs: "compute" (logged as "target")
-
 job_info_props = {"_id": 1, "aml_compute": 1, "compute": 1, "compute_target": 1,"concurrent": 1, "exper_name": 1, "hold": 1, "job_id": 1, "job_num": 1,
-    "job_guid": 1, "job_secret": 1, "low_pri": 1, "node_count": 1, "num_dgd_seeds": 1, "primary_metric": 1, "run_count": 1, "repeat": 1, 
+    "job_guid": 1, "job_secret": 1, "location": 1, "low_pri": 1, "node_count": 1, "num_dgd_seeds": 1, "primary_metric": 1, "run_count": 1, "repeat": 1, 
     "schedule": 1, "search_type": 1, "search_style": 1, "service_name": 1, "service_type": 1, "username": 1, "xt_cmd": 1, "started": 1,
     "pool_info": 1, "service_job_info": 1, "sku": 1, "sla": 1, "schedule": 1, "sleep_on_exit": 1, "vm_size": 1, "ws_name": 1}
 
-job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_runs": 1, 
+job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_runs": 1, "run_started": 1,
     "running_nodes": 1, "total_runs": 1, "db_retries": 1, "storage_retries": 1,
-    "restarts": 1, "next_run_index": 1}
+    "restarts": 1, "next_run_index": 1, "queue_duration": 1, "run_duration": 1, "heartbeat": 1, "end_time": 1}
 
 all_job_props = {**job_info_props, **job_stats_props}
 
@@ -152,7 +150,7 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
         build_filter_part(fd, args, name, user_to_actual[name])
 
     # filter by filter_list
-    filter_exp_list = args["filter"]
+    filter_exp_list = args["filter"]          
     if filter_exp_list:
         builder.process_filter_list(fd, filter_exp_list, user_to_actual)
 
@@ -180,6 +178,13 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
         # or all of fany conditions together
         fd["$or"] = fany_list
 
+    # filter by tags_none
+    if "tags_none" in args:
+        tags_none = args["tags_none"]
+        if tags_none:
+            for tag in tags_none:
+                fd["tags." + tag] = {"$exists": False}
+
     return fd
 
 # NEW COLS: sla, low_pri, vm_size, sku, service_name, service_type, aml_compute, compute_target, num_dgd_seeds
@@ -187,7 +192,7 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
 
 user_to_actual = {"aml_compute": "aml_compute", "cluster": "cluster", "compute_target": "compute_target", 
     "docker": "environment", "experiment": "exper_name",
-    "hold": "hold", "hparams": "hparams", "job": "job_id","job_num": "job_num", "low_pri": "low_pri", "num_dgd_seeds": "num_dgd_seeds",
+    "hold": "hold", "hparams": "hparams", "job": "job_id","job_num": "job_num", "location": "location", "low_pri": "low_pri", "num_dgd_seeds": "num_dgd_seeds",
     "node_count": "node_count", "primary_metric": "primary_metric", "queue": "queue", 
     "repeat": "repeat", "schedule": "schedule", "search": "search_type", "service_name": "service_name", 
     "service_type": "service_type", "sku": "sku", "sleep_on_exit": "sleep_on_exit", 
@@ -200,6 +205,8 @@ user_to_actual = {"aml_compute": "aml_compute", "cluster": "cluster", "compute_t
     "running_runs": "running_runs", "error_runs": "error_runs", "completed_runs": "completed_runs",
     "total_runs": "total_runs", "db_retries": "db_retries", "storage_retries": "storage_retries",
     "restarts": "restarts", "next_run_index": "next_run_index",
+    "queue_duration": "queue_duration", "run_duration": "run_duration", "heartbeat": "heartbeat", "end_time": "end_time",
+    "run_started": "run_started",
     }
 
 std_cols_desc = {
@@ -212,6 +219,7 @@ std_cols_desc = {
     "hold": "if the Azure Batch pool for this job was requested to be held open after runs were completed",
     "job": "the job name",
     "job_num": "the sort compatible, numeric portion of the job_id",
+    "location": "the location (aka region) of the compute for the job",
     "low_pri": "if the nodes requested for this job were specified as preemptable",
     "node_count": "the number of compute nodes requested for this job",
     "num_dgd_seeds": "the number of random HP configurations used to initialize the DGD search",
@@ -219,6 +227,7 @@ std_cols_desc = {
     "queue": "the Philly queue that the job was submitted to",
     "repeat": "the repeat count specified for the job",
     "runs": "the total number of runs for the job",
+    "run_started": "the datetime when the first node of the job was started",
     "schedule": "how runs are assigned to nodes (static or dynamic)",
     "search": "the type of hyperparameter search requested",
     "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
@@ -235,16 +244,20 @@ std_cols_desc = {
     "workspace": "the workspace associated with the job",
 
     # dynamic properties (get updated at various stages of job)
-    "db_retries": "the total number of database retries performed by the runs of this node",
-    "status": "one of: submitted, running, completed",
-    "running_nodes": "the number of nodes in the running state",
-    "running_runs": "the number of this job's runs that are current running",
     "completed_runs": "the number of runs that have completed (with or without errors)",
     "error_runs": "the number of runs that have terminated with an error",
+    "db_retries": "the total number of database retries performed by the runs of this node",
+    "end_time": "the datetime when the job was completed",
+    "heartbeat": "the last time the job was updated",
+    "next_run_index": "the number to be used for the next dynamic run within the job",
+    "queue_duration": "the amount of time the job spent in the queue",
+    "restarts": "the number of times a node has been preempted and restarted",
+    "run_duration": "the amount of time the job spent running",
+    "running_nodes": "the number of nodes in the running state",
+    "running_runs": "the number of this job's runs that are current running",
+    "status": "one of: submitted, running, completed",
     "storage_retries": "the total number of storage retries performed by the runs of this node",
     "total_runs": "the total number of runs specified for the job",
-    "restarts": "the number of times a node has been preempted and restarted",
-    "next_run_index": "the number to be used for the next dynamic run within the job"
 }
 
 def get_job_property_dicts():
@@ -306,13 +319,18 @@ def list_jobs(store, config, args, compare=False):
     console.diag("job_helper.list_jobs")
 
     available = args["available"]
+    if available in ["0", "none"]:
+        available = None
+    elif available in ["", "1", "all"]:
+        available = "all"
+
     report_type = "job-reports"
     user_columns = utils.get_user_columns(args["columns"], report_type, config)
 
     if compare:
         args["columns"] = ["job", "hparams.*"]
 
-    records, using_default_last, last, user_to_actual, builder \
+    records, limiter, limiter_value, user_to_actual, builder \
         = get_list_jobs_records(store, config, user_columns=user_columns, args=args)
 
     console.diag("after get_list_jobs_records")
@@ -323,7 +341,7 @@ def list_jobs(store, config, args, compare=False):
         std_cols = list(user_to_actual.keys())
         tag_cols = extract_tag_cols(records)
         lines = builder.available_cols_report("job", std_cols, std_cols_desc, hparams_list=hparams_cols, 
-            tags_list=tag_cols)
+            tags_list=tag_cols, filter=available)
 
         for line in lines:
             console.print(line)
@@ -336,30 +354,7 @@ def list_jobs(store, config, args, compare=False):
         avail_list = list(user_to_actual.keys())
         lines, row_count, was_exported = builder.build_report(records, user_columns=user_columns, report_type=report_type, args=args)
 
-        store_name = config.get("store")
-        workspace = args["workspace"]
-        console.print("jobs on {}/{}:".format(store_name, workspace))
-
-        if was_exported:
-            console.print("")
-
-            for line in lines:
-                console.print(line)
-        else:
-            # console.print the report
-            if row_count > 0:
-                console.print("")
-
-                for line in lines:
-                    console.print(line)
-
-                if row_count > 1:
-                    if using_default_last:
-                        console.print("total jobs listed: {} (limited by --last={})".format(row_count, last))
-                    else:
-                        console.print("total jobs listed: {}".format(row_count))
-            else:
-                console.print("no matching jobs found")
+        query_helper.print_report(lines, "jobs", config, store, args, builder, was_exported, row_count, limiter, limiter_value)
 
 def extract_tag_cols(records):
     tag_dict = {}
