@@ -589,7 +589,7 @@ class PlotBuilder():
 
     def calc_actual_layout(self, count, layout):
         if not "x" in layout:
-            errors.syntax_error("layout string must be of form RxC (R=# rows, C=# cols)")
+            errors.user_error("layout string must be of form RxC (R=# rows, C=# cols)")
 
         r,c = layout.split("x", 1)
 
@@ -1386,7 +1386,7 @@ class PlotBuilder():
             ax.hist(y_values, bins=self.bins, **kwargs)  
 
         else:
-            errors.syntax_error("unknown plot type={}".format(self.plot_type))
+            errors.user_error("unknown plot type={}".format(self.plot_type))
 
         # this must be called AFTER plotting
         if self.plot_titles:
