@@ -44,9 +44,10 @@ if is_windows:
 else:
     CWD = os.path.expanduser("~/.xt/cwd")
 
+# NOTE: we hard code names here to avoid pulling in the constants.py file
 PSM = "psm.py"
 CURRENT_RUNNING_ENTRY = "__current_running_entry__.txt"
-FN_WRAPPER = os.path.join(CWD, "__wrapped__.bat" if is_windows else "__wrapped__.sh")
+FN_NODE_SCRIPT = os.path.join(CWD, "__node_script__.sh")                
 CONTROLLER_NAME_PATTERN = "xtlib.controller"
 PY_RUN_CONTROLLER ="__run_controller__.py"
 
@@ -69,10 +70,7 @@ def get_controller_wrapped_counts():
     controller_count = 0
     wrapped_count = 0
 
-    if is_windows:
-        WRAPPED_PARTIAL = ".xt\\cwd\\__wrapped__.bat"
-    else:
-        WRAPPED_PARTIAL = ".xt/cwd/__wrapped__.sh"
+    WRAPPED_PARTIAL = ".xt/cwd/__node_script__.sh"
 
     #log_print("  WRAPPED_PARTIAL: " + WRAPPED_PARTIAL)
 
@@ -198,7 +196,7 @@ def start_entry(fn_entry):
         with open(fn_current_entry, "wt") as outfile:
             outfile.write(fn_entry)
 
-        fn_wrapper = FN_WRAPPER
+        fn_wrapper = FN_NODE_SCRIPT
         if is_windows:
             # fix slashes
             fn_wrapper = fn_wrapper.replace("/", "\\")
