@@ -182,7 +182,7 @@ class RemotePsmClient():
         output = self.run_cmd(box_cmd)
 
         #console.print("result=\n", output)
-        target_text = ".xt/cwd/" + constants.FN_WRAPPED_SH
+        target_text = ".xt/cwd/" + constants.FN_NODE_SCRIPT
         targets = [text for text in output.split("\n") if target_text in text]
         #console.print("targets=", targets)
         process_id = None
@@ -203,7 +203,7 @@ class RemotePsmClient():
             containers = output.split("\n")[1:]
 
             # filter to matches of our image name
-            containers = [con for con in containers if image_name in con]
+            containers = [con for con in containers if fn_image_name in con]
             docker_count = len(containers)
 
         return docker_count
