@@ -31,6 +31,9 @@ Root:
     will be they can be accepted for any command.
 '''
 
+# GLOBALS
+# TODO: remove these
+
 # structure that is used for storing all cmd_info's and
 # for locating a cmd_info one word at a time
 commands = {}               # nested dictionary, each level index by next command keyword
@@ -1020,6 +1023,15 @@ class Dispatcher():
         value = True if value in ["true", "on", "1"] else False
         return value
 
+    def scan_relational_op(self, scanner):
+        op = scanner.scan()
+
+        if scanner.token_type != "special":
+            if op not in ["in", "not-in"]:
+                self.syntax_error("expected relational operator in filter expression: " + op)
+
+        return op
+
     def process_prop_op_value(self, value):
         
         # mini parse of the value: <prop> <op> <value>
@@ -1028,9 +1040,7 @@ class Dispatcher():
         if scanner.token_type != "id":
             self.syntax_error("expected property name in filter expression: " + prop)
 
-        op = scanner.scan()
-        if scanner.token_type != "special":
-            self.syntax_error("expected relational operator in filter expression: " + op)
+        op = self.scan_relational_op(scanner)
 
         # expressions can be complicated; we want the rest of the string containing the filter
         value = scanner.get_rest_of_text()
@@ -1561,14 +1571,32 @@ class Dispatcher():
 
     def parse_cmd_option(self, match, scanner, tok, arg_dict):            
 
-        tok = scanner.scan()        # skip over name
+        option_type = match["type"]
+        force_string = option_type in ["str"]
+
         value = True
         found_equals = False
         name = match["name"]        # full name
 
-        if tok == "=":
+        # need to peek here to set force_string correctly
+        next_tok = scanner.peek()
+
+        if next_tok == "=":
+            tok = scanner.scan()
             found_equals = True
-            tok = scanner.scan()        # skip over equals
+            tok = scanner.scan(force_string=force_string)        # skip over equals
+        else:
+            tok = scanner.scan(force_string=force_string)        # skip over name
+
+        value = tok
+        if value and value.startswith("--"):
+            # this is a new option, not a value
+            value = None
+
+        # special handling for options with no value
+        required = utils.safe_value(match, "required")
+        if value is None and not required:
+            value = 1
 
         if name == "layer":
             # layer's job is to accumulate all previous args into a layer dict, and append to self.layers
@@ -1582,7 +1610,6 @@ class Dispatcher():
         else:
             values = match["values"] if "values" in match else None
             #value = self.process_option_value(name, match["type"], value, values)
-            required = utils.safe_value(match, "required")
 
             value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner, 
                 required)
@@ -1593,9 +1620,17 @@ class Dispatcher():
                 self.unknown_dest = match["dest"]
 
             else:
+                self.add_to_arg_dict(arg_dict, name, value)
                 arg_dict[name] = value
 
         return tok
+    
+    def add_to_arg_dict(self, arg_dict, name, value):
+        # preserve order of arguments: if duplicate, del first occurance and append to dict
+        if name in arg_dict:
+            del arg_dict[name]
+
+        arg_dict[name] = value
 
     def parse_option_value(self, name, opt_type, keywords, found_equals, tok, scanner, required=False):
 
@@ -1605,6 +1640,8 @@ class Dispatcher():
                 required = True
 
             user_value = self.expand_system_values(tok)
+            if user_value is None:
+                user_value = ""
 
             tok = scanner.scan()
             value = self.match_keyword(user_value, keywords)
@@ -1679,7 +1716,9 @@ class Dispatcher():
                 else:
                     self.syntax_error("Illegal value for boolean option: " + str(value))
             elif opt_type == "str":
-                #value = scanner.get_value_until_space()
+                if value is None:
+                    # to distinguish between unspecified option and one specified with no value
+                    value = ""
                 value = self.expand_system_values(value)
 
             else:
