@@ -83,10 +83,11 @@ class Run():
                 from tensorboard.compat import tf
                 delattr(tf.io.gfile.LocalFileSystem, 'append')
             except:
-                import tensorflow as tf
-                import tensorboard as tb
-                tf.io.gfile = tb.compat.tensorflow_stub.io.gfile
-                delattr(tf.io.gfile.LocalFileSystem, 'append')
+                # import tensorflow as tf
+                # import tensorboard as tb
+                # tf.io.gfile = tb.compat.tensorflow_stub.io.gfile
+                # delattr(tf.io.gfile.LocalFileSystem, 'append')
+                pass     # don't assume tensorflow is installed
 
             self.init_tensorboard()
 
@@ -270,8 +271,6 @@ class Run():
             # avoid calling close twice (once by app, once by auto_close)
             return
 
-        print("run.close() starting...")
-
         # careful not to trigger store method if not needed so that local script runs
         # don't trigger need for authentication unnecessarily 
         has_store = False
@@ -315,8 +314,9 @@ class Run():
         if has_store and self.store.database:        
             self.store.database.on_run_close(self.ws_name, self.run_name)
 
-            print("RUN: database call stats")
-            self.store.database.print_call_stats()
+            if self.context and self.context.log_db_stats:
+                print("RUN: database call stats")
+                self.store.database.print_call_stats()
 
         if self.train_writer:
             self.train_writer.close()
@@ -338,7 +338,7 @@ class Run():
             exit_code = 0
             hparams_dict = None
             metrics_rollup_dict = None
-            end_time = utils.get_arrow_time_str()
+            end_time = utils.get_arrow_now_str()
             log_records = []
 
             self.store.end_run(self.ws_name, self.run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
@@ -394,7 +394,9 @@ class Run():
                 if name == step_name:
                     dd[name] = value
                 else:
-                    dd[stage+"-"+name] = value
+                    # don't add the "-" if user supplied his own delimiter
+                    stage_name = stage + name if not stage[-1].isalpha() else stage + "-" + name
+                    dd[stage_name] = value
 
         # update ordered list of metric names
         metric_names_changed = False
@@ -429,6 +431,12 @@ class Run():
             for name, value in dd.items():
                 self.aml_run.log(name, value)
 
+        self.log_to_tensorboard(dd, step_name, step_num, stage)
+
+        if self.store and self.xt_logging:
+            self.process_notification_events("log_run")
+
+    def log_to_tensorboard(self, data_dict, step_name, step_num, stage=None):
         # log to TENSORBOARD
         if stage is None or stage == "train":
 
@@ -460,9 +468,6 @@ class Run():
                         self.test_writer2.add_scalar(name, value, global_step=step_num)
                 self.test_writer2.flush()
 
-        if self.store and self.xt_logging:
-            self.process_notification_events("log_run")
-
     def process_notification_events(self, current_event_when):
         if self.job_events:
 
