@@ -9,6 +9,7 @@ import fnmatch
 import pandas as pd
 from os import fdopen
 from datetime import timedelta
+from collections import defaultdict
 
 from xtlib import qfe
 from xtlib import utils
@@ -20,6 +21,7 @@ from xtlib import store_utils
 from xtlib.console import console
 from xtlib.helpers import file_helper
 from xtlib.helpers.feedbackParts import feedback as fb
+from xtlib import query_helper 
 
 run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "cmd_line_args": 1, "create_time": 1, "description": 1, 
     "display_name": 1, "exper_name": 1,
@@ -28,7 +30,7 @@ run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "cmd_lin
     "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
     "xt_build": 1, "xt_version": 1, "xt_cmd": 1, "cluster": 1, "vc":1, "service_run_id": 1}
 
-run_stats_props = {"status": 1, "start_time": 1, "last_time": 1, "exit_code": 1, "restarts": 1, 
+run_stats_props = {"status": 1, "start_time": 1, "last_time": 1, "exit_code": 1, "error_msg": 1, "restarts": 1, 
     "end_time": 1, "queue_duration": 1, "run_duration": 1, "end_id": 1, "metric_names": 1,
      "db_retries": 1, "storage_retries": 1}
 
@@ -190,6 +192,7 @@ def remove_run_stats(dd: dict):
 
 user_to_actual = {"box": "box_name", "cmd_line_args": "cmd_line_args", "created": "create_time", "child": "is_child", "cluster": "cluster",
     "description": "description", "display_name": "display_name", "experiment": "exper_name", "exit_code": "exit_code", 
+    "error_msg": "error_msg",
     "from_host": "from_computer_name", "from_ip": "from_ip", 
     "guid": "run_guid", "hp_set": "hp_set", "job": "job_id", "last_time": "last_time",
     "node_index": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
@@ -225,6 +228,7 @@ std_cols_desc = {
     "end_id": "a unique serial number assigned to each run when it has ended",
     "ended": "when the run completed",
     "experiment": "the name of the experiment associated with the run",
+    "error_msg": "set to the last stacktrace error seen in runs that terminate with an error",
     "exit_code": "the integer value returned by the run",
     "from_host": "the name of the computer that submitted the run",
     "from_ip": "the IP address of the computer that submitted the run",
@@ -350,25 +354,40 @@ def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, arg
     option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", 
         "outer", "username", "node_index"]  
 
-    explict_options = qfe.get_explicit_options()
-    explict_options = {key.replace("-", "_"): value for key,value in explict_options.items()}
-
-    # handle properties that are exceptions to needing to be explicit
-    for key in ["child", "parent"]:
-        if (key in args) and (args[key] is not None) and (key not in explict_options):
-            explict_options[key] = args[key]
-
-    # # parent is an exception - its default=0 needs to be filtered on
-    # if "parent" not in explict_options:
-    #     explict_options["parent"] = args["parent"]
+    # special hanndling for parent/child options
+    parent = utils.safe_value(args, "parent")
+    child = utils.safe_value(args, "child")
+    if parent and child:
+        # both specified, so we need to do an OR
+        # best accomplished by turning them both off
+        args["parent"] = None
+        args["child"] = None
 
     if run_list:
         build_filter_from_mixed_run_list(store, ws_name, run_list, fd)
 
-    # filter by specified options
-    for name in option_filters:
-        if name in explict_options:
-            build_filter_part(fd, args, name, user_to_actual[name])
+    use_explict_options = False
+
+    if use_explict_options:
+        # older approach: do we still need this?
+        explict_options = qfe.get_explicit_options()
+        explict_options = {key.replace("-", "_"): value for key,value in explict_options.items()}
+
+        # # handle properties that are exceptions to needing to be explicit
+        # for key in ["child", "parent"]:
+        #     if (key in args) and (args[key] is not None) and (key not in explict_options):
+        #         explict_options[key] = args[key]
+
+        # filter by specified options
+        for name in option_filters:
+            if name in explict_options:
+                build_filter_part(fd, args, name, user_to_actual[name])
+
+    else:
+        # filter by specified options
+        for name in option_filters:
+            if name in args:
+                build_filter_part(fd, args, name, user_to_actual[name])
 
     # filter by filter_list
     if "filter" in args:
@@ -378,7 +397,7 @@ def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, arg
 
     # filter by tag
     if "tag" in args:
-        tag_name = args["tag"]
+        tag_name = args["tag"] 
         if tag_name:
                 fd["tags." + tag_name] = {"$exists": True}
 
@@ -401,6 +420,13 @@ def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, arg
             # or all of fany conditions together
             fd["$or"] = fany_list
 
+    # filter by tags_none
+    if "tags_none" in args:
+        tags_none = args["tags_none"]
+        if tags_none:
+            for tag in tags_none:
+                fd["tags." + tag] = {"$exists": False}
+
     return fd
 
 def extract_dotted_cols(records, prefix):
@@ -442,7 +468,16 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
 
     # optional
     pool = utils.safe_value(args, "target")
+
+    # correct value of available (can be on/off flag, or string to be used type of available column)
     available = utils.safe_value(args, "available")
+    if available in ["0", "none"]:
+        available = None
+    elif available in ["", "1", "all"]:
+        available = "all"
+
+    args["available"] = available
+        
     workspace = utils.safe_value(args, "workspace")
 
     # if not col_dict:
@@ -533,7 +568,7 @@ def list_runs(store, config, args, compare=False):
         tags_cols = extract_dotted_cols(records, "tags")
 
         lines = builder.available_cols_report("run", std_cols, std_cols_desc, hparams_list=hparams_cols, 
-            metrics_list=metrics_cols, tags_list=tags_cols)
+            metrics_list=metrics_cols, tags_list=tags_cols, avail_filter=available)
 
         for line in lines:
             console.print(line)
@@ -554,40 +589,7 @@ def list_runs(store, config, args, compare=False):
         #avail_list = list(user_to_actual.keys())
         lines, row_count, was_exported = builder.build_report(records, user_columns=user_columns, report_type=report_type, args=args)
 
-        store_name = config.get("store")
-        workspace = args["workspace"]
-        console.print("runs on {}/{}:".format(store_name, workspace))
-
-        # print common hyperparameters on top of report
-        if utils.safe_value(args, "common") and builder.hp_set_formatter.hp_sets_processed:
-            console.print("\nCommon hyperparameters:")
-            for hp,value in builder.hp_set_formatter.hp_unchanged_dict.items():
-                console.print("  {}: {}".format(hp, value))
-            console.print("")
-
-        if was_exported:
-            console.print("")
-
-            for line in lines:
-                console.print(line)
-        else:
-            # console.print the report
-            if row_count > 0:
-                console.print("")
-
-                for line in lines:
-                    console.print(line)
-
-                if row_count > 1:
-                    
-                    if limiter:
-                        avail_count = store.database.avail_row_count
-                        avail_text = "available: {:}, ".format(avail_count) if avail_count is not None else ""
-                        console.print("total runs listed: {:} ({}limited by --{}={:})".format(row_count, avail_text, limiter, limiter_value))
-                    else:
-                        console.print("total runs listed: {:}".format(row_count))
-            else:
-                console.print("no matching runs found")
+        query_helper.print_report(lines, "runs", config, store, args, builder, was_exported, row_count, limiter, limiter_value)
 
 def get_run_record(store, workspace, run_name, fields_dict = None):
     run_records = get_run_records_simple(store, workspace, [run_name], fields_dict)
@@ -646,7 +648,7 @@ def expand_run_range(store, ws_name, low, high, run_names):
     high, high_prefix = get_rightmost_run_num(high)
 
     if low_prefix != high_prefix:
-        errors.syntax_error("for run name range, prefixes must match: {} vs. {}".format(low_prefix, high_prefix))
+        errors.user_error("for run name range, prefixes must match: {} vs. {}".format(low_prefix, high_prefix))
 
     for rx in range(low, high+1):
         rxx = low_prefix + str(rx)
@@ -1002,12 +1004,15 @@ def get_job_context(store, job_id, workspace):
     mrc_data = json.loads(text)
     return mrc_data
 
-def get_wrapup_runs_for_specified_nodes(store, ws_name, job_id, nodes):
+def get_wrapup_runs_for_node_list(store, ws_name, job_id, node_list, max_workers):
 
-    node_indexes = [utils.node_index(node_id) for node_id in nodes]
+    mrc_data = get_job_context(store, job_id, ws_name)
+    context_by_nodes = mrc_data["context_by_nodes"]
 
-    filter_dict = {"job_id": job_id, "node_index": {"$in": node_indexes}}
-    fields_dict = {"status": 1, "run_index": 1, "run_name": 1, "node_index": 1 }
+    node_indexes = [utils.node_index(node_id) for node_id in node_list]
+
+    filter_dict = {"job_id": job_id, "node_index": {"$in": node_indexes}, "status": {"$nin": ["error", "completed", "cancelled"]}}
+    fields_dict = {"status": 1, "run_index": 1, "run_name": 1, "node_index": 1, "create_time": 1, "start_time": 1, "is_parent": 1 }
 
     # get runs for specified nodes
     runs = store.database.get_info_for_runs(ws_name, filter_dict, fields_dict)
@@ -1022,77 +1027,94 @@ def get_wrapup_runs_for_specified_nodes(store, ws_name, job_id, nodes):
         if status in unwrapped:
             filtered_runs.append(run)
 
-    return filtered_runs
+    return runs, context_by_nodes
 
-def get_wrapup_runs_for_all_nodes(store, ws_name, job_id, node_list, max_workers):
-
-    mrc_data = get_job_context(store, job_id, ws_name)
-    context_by_nodes = mrc_data["context_by_nodes"]
-
-    # gather the runs on worker threads
-    next_progress_num = 1
-    all_runs = []
-
-    from threading import Lock
-    worker_lock = Lock()
-
-    def thread_worker(nodes, store, ws_name, job_id):
-
-        runs = get_wrapup_runs_for_specified_nodes(store, ws_name, job_id, nodes)
-
-        with worker_lock:
-            nonlocal all_runs, next_progress_num
-
-            all_runs += runs
-
-            for node in nodes:
-                node_msg = "gathering runs from nodes: {}/{}".format(next_progress_num, len(node_list))
-                fb.feedback(node_msg, id="gather_msg")  
-                next_progress_num += 1
-
-    utils.run_on_threads(thread_worker, node_list, max_workers, [store, ws_name, job_id])
-    return all_runs, context_by_nodes
-
-def wrapup_runs_for_nodes(store, ws_name, job_id, nodes, max_workers):
+def wrapup_runs_nodes_job(store, ws_name, job_id, node_ids, node_end_times, max_workers):
     '''
-    wrap up a run from azure batch.  run may have started, or may have completed.  
+    wrap up a run.  run may have started, or may have completed.  
     '''
     fb.reset_feedback()
+    node_ended_run_counts = defaultdict(int)
 
-    run_list, context_by_nodes = get_wrapup_runs_for_all_nodes(store, ws_name, job_id, nodes, max_workers)
+    run_list, context_by_nodes = get_wrapup_runs_for_node_list(store, ws_name, job_id, node_ids, max_workers)
     if run_list:
-        # wrapup a subset of run_list on each worker thread
+        # mark each run as cancelled
         next_progress_num = 1
 
         from threading import Lock
         worker_lock = Lock()
 
-        def thread_worker(runs, store, context_by_nodes):
+        def run_thread_worker(runs, store):
 
             for run in runs:
                 node_index = run["node_index"]
                 node_id = utils.node_id(node_index)
+                run_name = run["run_name"]
+                run_status = "cancelled"
+                create_time = run["create_time"]
+                start_time = run["start_time"]
+                node_end_time = node_end_times[node_id]
 
-                context_plus = context_by_nodes[node_id]
-                context = context_plus["runs"][0]
-
-                wrapup_run_with_context(store, run, context, job_id=job_id)
+                store.database.mark_run_ended_in_past(ws_name, run_name, run_status, create_time=create_time, start_time=start_time, end_time=node_end_time, exit_code=None, error_msg=None)
 
                 with worker_lock:
                     nonlocal next_progress_num
                     node_msg = "wrapping up runs: {}/{}".format(next_progress_num, len(run_list))
-
                     fb.feedback(node_msg, id="wrapup_msg")  
+
+                    next_progress_num += 1
+                    node_ended_run_counts[node_id] += 1
+
+        utils.run_on_threads(run_thread_worker, run_list, max_workers, [store])
+        fb.feedback("done", is_final=True)
+    
+    if node_ids:
+        # mark each node as cancelled
+        next_progress_num = 1
+
+        from threading import Lock
+        worker_lock = Lock()
+
+        # get nodes for specified node_ids
+        filter_dict = {"ws_name": ws_name, "job_id": job_id, "node_id": {"$in": node_ids}}
+        fields_dict = {"job_id": 1, "target": 1, "node_index": 1, "node_id": 1, "node_status": 1, "ws_name": 1,
+                       "create_time": 1, "prep_start_time": 1, "app_start_time": 1, "post_start_time": 1, "post_end_time": 1,
+                       "completed_runs": 1, "error_runs": 1}
+        node_list = store.database.get_info_for_nodes(ws_name, filter_dict, fields_dict)
+
+        def node_thread_worker(nodes, store):
+
+            for node in nodes:
+                node_id = node["node_id"]
+                node_end_count = node_ended_run_counts[node_id]
+                node_end_time = node_end_times[node_id]
+                node_status = "cancelled"
+
+                #wrapup_run_with_context(store, run, context, job_id=job_id)
+                store.database.mark_node_ended_in_past(ws_name, job_id, node, node_status, node_end_time, node_end_count, 0)
+
+                with worker_lock:
+                    nonlocal next_progress_num
+                    node_msg = "wrapping up nodes: {}/{}".format(next_progress_num, len(node_list))
+                    fb.feedback(node_msg, id="wrapup_msg")  
+
                     next_progress_num += 1
 
-        utils.run_on_threads(thread_worker, run_list, max_workers, [store, context_by_nodes])
-    else:
-        fb.feedback("no runs needing wrapup found")
+        utils.run_on_threads(node_thread_worker, node_list, max_workers, [store])
+        fb.feedback("done", is_final=True)
 
     # finally, mark the job as CANCELLED
-    #store.database.set_job_status(ws_name, job_id, "cancelled")
+    filter_dict = {"ws_name": ws_name, "job_id": job_id}
+    fields_dict = {"job_id":1, "started": 1, "run_started": 1, "completed_runs": 1, "error_runs": 1}
+    jobs = store.database.get_info_for_jobs(ws_name, filter_dict, fields_dict)
+    job = jobs[0]
+    job_status = "cancelled"
+    job_end_time = max(node_end_times.values())
+    job_runs_ended = sum(node_ended_run_counts.values())
 
-    fb.feedback("done", is_final=True)
+    store.database.mark_job_ended_in_past(ws_name, job_id, job_status, job, end_time_secs=job_end_time, 
+        new_completed_runs=job_runs_ended, new_error_runs=None)
+    console.print("wrapped up job: {}".format(job_id))
 
 def wrapup_run_with_context(store, run, context_dict, job_id=None):
     context = utils.dict_to_object(context_dict)
@@ -1174,6 +1196,15 @@ def is_run_name(name):
 
     return is_valid
 
+def is_parent_run(name):
+    is_parent = False
+    if "_" in name:
+        parts = name.split("_")
+        if parts[-1].isdigit():
+            is_parent = True
+            
+    return is_parent
+
 def get_run_records_simple(store, ws_name, run_names, fields_dict=None):
     ''' return run records for specified run names'''
 
