@@ -16,7 +16,6 @@ from threading import Lock
 from xtlib import mirror_worker, utils
 from xtlib import errors
 from xtlib import file_utils
-from xtlib.utils import log_info, log_title
 
 from xtlib.console import console
 
@@ -25,7 +24,7 @@ logger = logging.getLogger(__name__)
 class RunInfo():
     def __init__(self, run_name, workspace, cmd_parts, run_script, repeat, context, status, show_output=True, parent_name=None, 
         parent_prep_needed=False, mirror_close_func=None, node_id=None, run_index=None, max_delay=0,
-        store=None, is_parent=False):
+        store=None, is_parent=False, log_info=None):
 
         if isinstance(repeat, str):
             repeat = int(repeat)
@@ -68,6 +67,11 @@ class RunInfo():
         self.killed_for_restart = False
         self.max_delay = max_delay
         self.delay_started = None
+        self.last_run_exception = None    # from stack trace in console output      
+        self.last_error_msg = None        # might not be fatal
+        self.error_msg = None             # fatal error msg
+        self.run_started = None
+        self.log_info = log_info
 
     def set_console_fn(self, console_fn):
         console_fn = os.path.expanduser(console_fn)
@@ -77,6 +81,24 @@ class RunInfo():
         #     os.remove(console_fn)
         file_utils.ensure_dir_exists(file=console_fn)
 
+    def check_for_time_exceeded(self):
+        # limit run time to specified value
+        run_elapsed_secs = time.time() - self.run_started
+        max_dur_text = self.context.max_run_duration
+
+        if max_dur_text:
+            max_secs = utils.shell_time_str_to_secs(max_dur_text)
+
+            if max_secs and (run_elapsed_secs > max_secs):
+
+                suffix = max_dur_text[-1]
+                max_actual_text = utils.secs_to_shell_time(run_elapsed_secs, suffix)
+                console.print("** Error: specified run max duration ({}) exceeded run's actual elapsed running time ({}: {})".format( \
+                    max_dur_text, self.run_name, max_actual_text))
+
+                # kill this run
+                self.kill()
+
     def process_run_output(self, text_msg, run_info_is_locked=False):
 
         # if self.context.scrape:
@@ -90,6 +112,15 @@ class RunInfo():
 
         # append to output file (open/close file each time to ensure it is written to local file)
 
+        # first console output of run defines start time (for now)
+        if not self.run_started:
+            self.run_started = time.time()
+
+        # save any message containing "error" for runs that terminate with an error
+        if "error" in text_msg.lower():
+            self.last_error_msg = text_msg
+            print("setting last_error to: {}".format(self.last_error_msg))
+
         if self.console_fn:
             if os.path.isdir(self.console_fn):
                 console.print("INTERNAL ERROR: dir found at: {}", self.console_fn)
@@ -103,8 +134,7 @@ class RunInfo():
         # send output to attached clients
         if self.show_output:
             # console.print output on CONTROLLER console
-            sys.stdout.write(self.run_name + ": ")
-            sys.stdout.write(text_msg)
+            console.print(self.run_name + ": " + text_msg, end="")
 
             # if run_info_is_locked:
             #     list2 = list(self.acallbacks)
@@ -173,7 +203,7 @@ class RunInfo():
         '''wrap-up the run (logging, capture)'''
         #console.print("run_wrapup, self.killed_for_restart={}".format(self.killed_for_restart))
         
-        log_info("run_wrapup: mirror_workers", self.mirror_workers)
+        self.log_info("run_wrapup: mirror_workers", self.mirror_workers)
         if self.mirror_workers:
                 self.mirror_close_func(self)
 
@@ -187,18 +217,30 @@ class RunInfo():
             # for "xt restart controller", do not log "cancelled" for runs
             # in spirit of a box that has just been preempted
 
-            log_info("runinfo.is_parent", self.is_parent)
-            log_info("runinfo.repeat", self.repeat)
+            self.log_info("runinfo.is_parent", self.is_parent)
+            self.log_info("runinfo.repeat", self.repeat)
 
-            log_info("calling WRAPUP_RUN", self.run_name)
+            self.log_info("calling WRAPUP_RUN", self.run_name)
             log_upload_files = context.log_file_uploads
 
+            # print("ri.run_wrap_up: detected exit_code: {}, last_run_exception: {}, last_error_msg: {}".format(\
+            #     self.exit_code, self.last_run_exception, self.last_error_msg))
+
+            if not self.last_run_exception:
+                self.last_run_exception = self.last_error_msg
+
+            # record the reason for the error exit as the run_stats std col "error_msg"
+            self.error_msg = self.last_run_exception
+
+            #print("calling wrapup_run with error_msg: {}".format(self.error_msg))
+
             store.wrapup_run(context.ws, run_name, context.aggregate_dest, context.dest_name, 
                 status=self.status, exit_code=self.exit_code, primary_metric=context.primary_metric, 
                 maximize_metric=context.maximize_metric, report_rollup=context.report_rollup, rundir=self.rundir, 
                 after_files_list=context.after_files_list, after_omit_list=context.after_omit_list, log_events=context.log, 
                 capture_files=context.after_upload, job_id=self.job_id, is_parent=self.is_parent, node_id=self.node_id, 
-                run_index=self.run_index, start_time=self.started, log_upload_files=log_upload_files)
+                run_index=self.run_index, start_time=self.started, log_upload_files=log_upload_files, 
+                error_msg=self.error_msg)
 
     def set_status(self):
         if self.killing:
@@ -217,13 +259,13 @@ class RunInfo():
 
             # wait for process to completely terminate
             if wait_if_needed:
-                log_info("waiting for terminate", self.run_name)
+                self.log_info("waiting for terminate", self.run_name)
                 self.process.wait()
 
             presult = self.process.poll()
             exited = (presult != None)
-            log_info("process.poll() result=", presult)
-            log_info("exited=", exited)
+            self.log_info("process.poll() result=", presult)
+            self.log_info("exited=", exited)
 
             if exited or wait_if_needed:
                 self.exit_code = self.process.wait()
