@@ -66,7 +66,7 @@ class Runner():
         target_file = file_utils.fix_slashes(target_file, is_linux=target_is_linux)
 
         if os.path.isabs(target_file):
-            errors.syntax_error("path to app file must be specified with a relative path: {}".format(target_file))
+            errors.user_error("path to app file must be specified with a relative path: {}".format(target_file))
 
         is_rerun = "is_rerun" in args
         if is_rerun:
@@ -172,7 +172,7 @@ class Runner():
             if file_ext != expected_ext:
                 errors.combo_error("{} file ext='{}' doesn't match box.os='{}'".format(script_name, file_ext, box_info.box_os))
 
-    def build_first_run_for_node(self, node_index, run_count, total_run_count, box_name, run_script_path, parent_script_path, 
+    def build_primary_run_for_node(self, node_index, run_count, total_run_count, box_name, run_script_path, parent_script_path, 
             using_hp, use_aml_hparam, run_specs, job_id, parent_name, cmds, compute_def, repeat_count, fake_submit, 
             search_style, box_secret, node_count, run_name=None, args=None):
             
@@ -188,7 +188,7 @@ class Runner():
             if parent_script_path:
                 self.ensure_script_ext_matches_box("parent script", parent_script_path, box_info)
 
-        node_id = "node" + str(node_index)
+        #node_id = "node" + str(node_index)
         # if using_hp:
         #     if not node_id in cmds_by_node:
         #         errors.combo_error("you specified more nodes/boxes than hyperparameter search runs")
@@ -196,6 +196,15 @@ class Runner():
         cmd_parts = run_specs["cmd_parts"]
         actual_parts = None
 
+        is_direct_run = self.config.get("general", "direct-run")
+        if is_direct_run and cmds:
+            # adjust cmd in each job_run to use cmds[node_index]
+            cmd = cmds[node_index]
+            cmd_parts = cmd.split()
+
+            run_specs["cmd_parts"] = cmd_parts
+            run_specs["run_cmd"] = cmd
+
         if cmd_parts:
             actual_parts = list(cmd_parts)
             if box_info.box_os == "linux" and actual_parts[0] == "docker":
@@ -224,7 +233,7 @@ class Runner():
         box_name, run_name, secret, service_info, exper_name, args):
 
         compute_target = args["target"]
-        str_now = utils.get_arrow_time_str()
+        str_now = utils.get_arrow_now_str()
 
         job_num = job_helper.get_job_number(job_id)
         node_num = job_num*1000*1000 + node_index
@@ -248,7 +257,7 @@ class Runner():
         self.store.database.update_collection("node_info", ws_name, dd)
 
     def write_node_stats_data(self, _id, ws_name):
-        str_now = utils.get_arrow_time_str()
+        str_now = utils.get_arrow_now_str()
 
         dd = {"_id": _id, "node_status": "created", 
             "completed_runs": 0, "error_runs": 0, "running_runs": 0,
@@ -411,19 +420,18 @@ class Runner():
         last_run = None
 
         # for each node
-        for run_data_list in job_runs:
-            for run_data in run_data_list:   
-                box_name = run_data["box_name"]
+        for run_data in job_runs:
+            box_name = run_data["box_name"]
 
-                # process a run for box_name
-                if not box_name in runs_by_box:
-                    runs_by_box[box_name] = [] 
+            # process a run for box_name
+            # if not box_name in runs_by_box:
+            #     runs_by_box[box_name] = [] 
 
-                # create as dict; we will later add "service_run_id" to the dict (for philly, batch, aml)
-                rr = {"ws_name": workspace, "run_name": run_data["run_name"], "box_index": run_data["box_index"]}
+            # create as dict; we will later add "service_run_id" to the dict (for philly, batch, aml)
+            rr = {"ws_name": workspace, "run_name": run_data["run_name"], "box_index": run_data["box_index"]}
 
-                runs_by_box[box_name].append(rr)
-                last_run = run_data["run_name"]
+            runs_by_box[box_name] = rr
+            last_run = run_data["run_name"]
 
         return runs_by_box, last_run
 
@@ -545,39 +553,23 @@ class Runner():
 
         options = "--rm"
 
-        # collect env vars 
-        env_vars["XT_IN_DOCKER"] = 1
-        env_vars["XT_USERNAME"] = pc_utils.get_username()
-        
-        scriptor.add_controller_env_vars(env_vars, self.config, job_secret, "node0")
-
         full_image = login_server + "/" + docker_image if login_server else docker_image
 
-        # don't use data_local or model_local mappings for docker because docker
-        # always runs linux where blobfuse is available 
+        # inherit ENV VARS from running environment
+        env_vars["XT_IN_DOCKER"] = 1
+        env_vars["XT_USERNAME"] = pc_utils.get_username()
 
-        # # build a mapping for data
-        # if target == "local":
-        #     data_local = args["data_local"]
-        #     if data_local:
-        #         if "$scriptdir" in data_local:
-        #             data_local = data_local.replace("$scriptdir", script_dir)
-
-        #         data_local = os.path.realpath(data_local)
-        #         mappings += " -v {}:/usr/data".format(data_local)
-        #         env_vars["XT_DATA_DIR"] = "/usr/data"
+        # we have to be careful here to correctly transfer env vars that are node-specific
+        # this is done in the NodeScriptor.gen_docker_run()
 
         # write env vars to file in bootstrap_dir 
-        fn_env_var = os.path.join(bootstrap_dir, constants.FN_EV)
+        fn_env_var = os.path.join(bootstrap_dir, constants.FN_DOCKER_ENV)
         lines = [name + "=" + str(value) for name,value in env_vars.items()]
         text = "\n".join(lines)
         file_utils.write_text_file(fn_env_var, text)
 
         # specify env var file (in current directory) to docker
-        options += " --env-file={}".format(constants.FN_EV)
-
-        # inherit ENV VARS from running environment
-        options += " -e XT_RUN_NAME -e XT_WORKSPACE_NAME -e XT_EXPERIMENT_NAME"
+        options += " --env-file={}".format(constants.FN_DOCKER_ENV)
 
         # flags needed by blobfuse usage
         #options += " --cap-add SYS_ADMIN --device /dev/fuse --security-opt apparmor:unconfined"
@@ -597,14 +589,9 @@ class Runner():
         pass_args = "%*" if for_windows else "$*"
         sh_prefix = "bash --login"
 
-        if for_windows:
-            # launching docker from windows
-            docker_cmd = "{}docker run {} {} {} {} {}/{} {}".format(sudo, mappings, options, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
-        else:
-            # launching docker from linux
-            # we set DOCKER_RUN just before this command is executed (see backend_base.py)
-            docker_cmd = "{}$DOCKER_RUN {} {} {} {} {}/{} {}".format(sudo, mappings, options, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
-            #docker_cmd = "{}docker run --gpus all {} {} {} {} {}/{} {}".format(sudo, options, mappings, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
+        # launching docker from linux
+        # we set DOCKER_RUN just before this command is executed (see backend_base.py)
+        docker_cmd = "{}$DOCKER_RUN {} {} {} {} {}/{} {}".format(sudo, mappings, options, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SCRIPT, pass_args)
 
         # store for use by backend_base.wrap_user_cmds()
         args["docker_cmd"] = docker_cmd
@@ -750,7 +737,11 @@ class Runner():
             errors.user_error("when creating a request, the reason for the job must be specified with --description")
 
         # create a new request in database
-        rd = {"status": "requested", "requested_by": username, "description": description, 
+        # include env_vars, which might have captured values from requestor's env vars
+        env_vars = args["env_vars"]
+        env_vars_text = json.dumps(env_vars)
+
+        rd = {"status": "requested", "requested_by": username, "description": description, "env_vars": env_vars_text,
             "target": target, "nodes": nodes, "runs": runs}
 
         request_id = self.store.create_request(ws_name, rd)
@@ -761,15 +752,22 @@ class Runner():
 
         from xtlib.xt_cmds import orig_xt_cmd
         run_cmd = orig_xt_cmd
-        code_option = " --code=$requests/{}/code ".format(request_id) 
+        new_options = " --code=$requests/{}/code".format(request_id) 
+
+        # add the request id, so run command can fetch other info as needed related to the request
+        new_options += " --from-request=" + request_id
+
+        # add requestor's name as username
+        new_options += " --username=" + username
+        new_options += " "
 
         # replace the "--request" option with code_option
         index = run_cmd.index("--req")
         index2 = run_cmd.find(" ", index+5)
         if index2 > -1:
-            run_cmd = run_cmd[0:index].strip() + code_option + run_cmd[index2:].strip()
+            run_cmd = run_cmd[0:index].strip() + new_options + run_cmd[index2:].strip()
         else:
-            run_cmd = run_cmd[0:index].strip() + code_option
+            run_cmd = run_cmd[0:index].strip() + new_options
 
         # update the request record
         rd["cmd"] = run_cmd
@@ -933,10 +931,21 @@ class Runner():
         # expand needed args (used by docker command, runner, and batch/pool backends)
         # TODO: centralize all this env var processing
         if env_vars:
-            for name, value in env_vars.items():
-                if value.startswith("$$"):
-                    value = os.getenv(value[2:])
-                    env_vars[name] = value
+
+            request_id = args["from_request"]
+            if request_id:
+                # use environment variables from the request
+                filter_dict = {"request_id": request_id}
+                records = self.store.database.get_info_for_requests(workspace, filter_dict)
+                rr = records[0]
+                env_vars_text = rr["env_vars"]
+                env_vars = json.loads(env_vars_text)
+
+            else:
+                for name, value in env_vars.items():
+                    if value.startswith("$$"):
+                        value = os.getenv(value[2:])
+                        env_vars[name] = value
 
             # update args copy
             args["env_vars"] = env_vars
@@ -1054,6 +1063,8 @@ class Runner():
         secrets_by_node = {}
         remote_control = args["remote_control"]
 
+        job_create_time = utils.get_arrow_now_str()
+
         self.build_all_node_runs(boxes, remote_control, run_count, total_run_count, 
                 target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
                 parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, job_runs, 
@@ -1085,9 +1096,6 @@ class Runner():
         # create a job guid to uniquely identify this job across all XT instances
         job_guid = str(uuid.uuid4())
 
-        # we add with "node0" and "job_secret", but backend service will override for each node
-        scriptor.add_controller_env_vars(env_vars, self.config, None, "node0")
-
         data_local = args["data_local"]
         if "$scriptdir" in data_local:
             data_local = os.path.realpath(data_local.replace("$scriptdir", script_dir))
@@ -1098,8 +1106,14 @@ class Runner():
             model_local = os.path.realpath(model_local.replace("$scriptdir", script_dir))
             args["model_local"] = model_local
 
-        # ADJUST CMDS: this allows backend to write scripts to snapshot dir, if needed, as a way of adjusting/wrapping run commands
-        self.backend.adjust_run_commands(job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args=args)
+        is_direct_run = self.config.get("general", "direct-run")
+        if is_direct_run:
+            run_count = len(cmds) if cmds else args["runs"]
+            if len(job_runs) != run_count:
+                errors.user_error("When specifying --direct-run=1, the number of runs must match the number of nodes.")
+
+        # allows backend to build the node script (shared by all nodes of the run)
+        self.backend.build_node_script_and_adjust_runs(job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, cmds=cmds, args=args)
 
         # upload CODE from snapshot_dir
         code_upload = args["code_upload"]
@@ -1184,7 +1198,7 @@ class Runner():
 
         # create the JOB_INFO record (and fully initialize JOB_STATS)
         self.create_job_info(job_id, search_style, total_run_count, runs_by_box, experiment, job_guid, job_secret, repeat_count, 
-            compute_def, workspace, secrets_by_node, args)
+            compute_def, workspace, secrets_by_node, job_create_time, args)
 
         # SUBMIT JOB to backend 
         # backend requirement: update each node_info ASAP with service_info_by_node
@@ -1192,11 +1206,14 @@ class Runner():
             self.backend.submit_job(job_id, job_runs, workspace, compute_def, resume_name, 
                 repeat_count, using_hp, runs_by_box, experiment, code_uploaded_dir, adjustment_scripts, args)
 
+        # note: singularity/AML/ITP have no job-level info, so service_job_info is {} for them
+        #assert service_job_info
+
         # POST SUBMIT processing
         if not fake_submit:
 
             # update job_info with: service_job_info, service_info_by_node
-            self.update_job_info(job_id, workspace, service_job_info, service_info_by_node)
+            self.update_job_info_post_submit(job_id, workspace, service_job_info, service_info_by_node)
 
         fb.feedback("done", is_final=True)
 
@@ -1213,8 +1230,15 @@ class Runner():
         # return values for API support (X)
         return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, compute_def, job_id 
 
+    def get_location(self, backend) -> str:
+        cd = backend.compute_def
+        service = cd["service"]
+
+        location = backend.get_location(service)
+        return location 
+    
     def create_job_info(self, job_id, search_style, total_run_count, runs_by_box, experiment, job_guid, job_secret, repeat_count, 
-            compute_def, workspace, secrets_by_node, args):
+            compute_def, workspace, secrets_by_node, job_create_time, args):
 
             fb.feedback("logging job")
 
@@ -1235,11 +1259,18 @@ class Runner():
 
             tag_dict = args["tags"]
 
+            locations = utils.safe_value(compute_def, "locations")
+            if locations and (not self.backend.supports_setting_location()):
+                console.print("warning: backend: {} does not support setting compute location".format(self.backend.get_name()))
+
+            if not locations:
+                # get the location associated with the backend compute being used
+                locations = [ self.get_location(self.backend) ]
+
             # NEW COLS: sla, low_pri, vm_size, sku, service_name, service_type, aml_compute, compute_target, num_dgd_seeds
     
             dd = {
                 # JOB_INFO: flat READONLY props (18)
-                # new cols
                 "sla": utils.safe_value(compute_def, "sla"),
                 "low_pri": utils.safe_value(compute_def, "low-pri"),
                 "vm_size": utils.safe_value(compute_def, "vm-size"),
@@ -1248,6 +1279,7 @@ class Runner():
                 "service_type": utils.safe_value(args, "service_type"),
                 "aml_compute": utils.safe_value(compute_def, "compute"),
                 "compute_target": utils.safe_value(compute_def, "name"),
+                "location": locations[0] if locations else None,           # for now, we only log primary location 
 
                 # old cols
                 "concurrent": concurrent,
@@ -1277,13 +1309,16 @@ class Runner():
 
                 # JOB_STATS: flat UPDATABLE props (8)
                 "completed_runs": 0, 
-                "dynamic_runs_remaining": dynamic_runs_remaining, 
-                "error_runs": 0, 
-                "job_status": "submitted", 
                 "running_nodes": 0, 
                 "running_runs": 0, 
-                "started": utils.get_arrow_time_str(), 
+                "error_runs": 0, 
+                "restarts": 0,
+                "db_retries": 0,
+                "storage_retries": 0,
                 "next_run_index": 0,
+                "dynamic_runs_remaining": dynamic_runs_remaining, 
+                "job_status": "submitted", 
+                "started": job_create_time,
 
                 # SERVICE_INFO: embedded READONLY props BY NODE (2)
                 "runs_by_box": runs_by_box, 
@@ -1302,7 +1337,7 @@ class Runner():
 
             self.store.log_job_info(workspace, job_id, dd)
 
-    def update_job_info(self, job_id, workspace, service_job_info, service_info_by_node):
+    def update_job_info_post_submit(self, job_id, workspace, service_job_info, service_info_by_node):
 
         dd = {"ws_name": workspace, "job_id": job_id, "service_job_info": service_job_info, "service_info_by_node": service_info_by_node}
         self.store.log_job_info(workspace, job_id, dd, update_job_stats=False)
@@ -1310,10 +1345,9 @@ class Runner():
     def log_run_and_node(self, workspace, box_name, runs_by_box, job_id, total_run_count, secrets_by_node, 
         service_info_by_node, args):
 
-        runs_for_box = runs_by_box[box_name]
-        first_run = runs_for_box[0]
-        run_name = first_run["run_name"]
-        node_index = first_run["box_index"]
+        box_run = runs_by_box[box_name]
+        run_name = box_run["run_name"]
+        node_index = box_run["box_index"]
         node_count = len(runs_by_box)
         node_id = utils.node_id(node_index)
 
@@ -1400,18 +1434,18 @@ class Runner():
                 # generate a box secret for talking to XT controller for this node
                 box_secret = str(uuid.uuid4()) if remote_control else ""
                 run_name = run_names[node_index]
+                node_run_specs = dict(run_specs)    # make a copy that this node can own
 
-                run_data = self.build_first_run_for_node(node_index, run_count, total_run_count, 
-                    boxes[node_index], target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+                run_data = self.build_primary_run_for_node(node_index, run_count, total_run_count, 
+                    boxes[node_index], target_file, ps_path, using_hp, using_aml_hparam, node_run_specs, job_id, 
                     parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, box_secret, 
                     node_count, run_name=run_name, args=args)
 
                 with thread_lock:
                     nonlocal next_progress_num
 
-                    # for now, adhere to the more general design of multiple runs per box (to be removed)
-                    box_runs = [run_data]      
-                    job_runs.append(box_runs)
+                    # NEW CHANGE: we now only have 1 run per box/node (the primary/parent run)
+                    job_runs.append(run_data)
 
                     node_id = utils.node_id(node_index)            
                     secrets_by_node[node_id] = box_secret
@@ -1456,7 +1490,7 @@ class Runner():
         self.store.database.set_insert_buffering(0)
 
         # sort job_runs to normal order
-        job_runs.sort(key=lambda r: r[0]["box_index"])
+        job_runs.sort(key=lambda r: r["box_index"])
 
     def get_cmd_line_args_from_file(self, fn):
         with open(fn, "rt") as infile:
@@ -1716,7 +1750,7 @@ class Runner():
 
         runs = args["runs"]
         max_runs = args["max_runs"]
-        node_repeat = args["node_repeat"]
+        grid_repeat = args["grid_repeat"]
         runs_per_set = args["runs_per_set"]
 
         # USER MULTI CMDS
@@ -1755,12 +1789,12 @@ class Runner():
             cmds = [new_run_cmd]
             show_run_report = False
 
-        if node_repeat:
+        if grid_repeat:
             # duplicate cmds so they are repeated N times on each node
             # nodes are assigned runs in round-robin fashion
             new_cmds = []
             for cmd in cmds:
-                new_cmds += [cmd]*node_repeat
+                new_cmds += [cmd]*grid_repeat
             
             cmds = new_cmds
             run_count = len(cmds)
@@ -1782,13 +1816,13 @@ class Runner():
 
             console.print("{} {}runs{}:".format(search_style, stype, dr))
 
-            if node_repeat:
+            if grid_repeat:
                 runsets = None
                 search_style = "static"
                 node_index = -1
 
                 for i, run_cmd_parts in enumerate(cmds):
-                    if i % node_repeat == 0:
+                    if i % grid_repeat == 0:
                         node_index += 1
 
                     node_id = utils.node_id(node_index)
@@ -1869,7 +1903,9 @@ class Runner():
                 if (search_type == "grid") or (static_search and search_type in ["random"]):
                     runsets = hp_client.generate_hp_sets(dd, search_type, num_runs, max_runs, node_count)
         
-                    if option_prefix and option_prefix in cmd_line:
+                    # why was this conditional on option_prefix being found in cmd_line?
+                    #if option_prefix and option_prefix in cmd_line:
+                    if option_prefix:
                         # apply runsets to run_cmd to generate cmd list
                         cmd_line_base = run_cmd
                         run_cmds = hp_client.generate_runs(runsets, cmd_line_base)
@@ -1888,9 +1924,9 @@ class Runner():
 
 # flat functions
 def update_compute_def_from_cmd_options(compute_def, hold=False):
-
+    # support for DEFAULT_FROM_COMPUTE:
     compute_props = ["vm-size", "azure-image", "low-pri", "box-class", "docker", "setup", "boxes", 
-        "vc", "cluster", "queue", "sku", "nodes", "sla", "aml_compute", "service"]
+        "vc", "cluster", "queue", "sku", "nodes", "sla", "aml_compute", "service", "locations"]
 
     # apply any compute-def properties found in explicit args to compute_def
     explict_opts = qfe.get_explicit_options()
