@@ -780,7 +780,7 @@ class MongoDB2(implements(DbInterface)):
         #print("dd=", dd)
         self.disallow_embedded_props(dd)
         
-        run_stats["last_time"] = utils.get_arrow_time_str()
+        run_stats["last_time"] = utils.get_arrow_now_str()
         
         _id = self.make_key(ws_name, run_name)
 
