@@ -315,7 +315,7 @@ class StoreV2():
     def log_workspace_event(self, ws_name, event_name, data_dict):
         ''' log the specifed event_name and key/value pairs in the data_dict to the workspace log file.
         '''
-        record_dict = {"time": utils.get_arrow_time_str(), "event": event_name, "data": data_dict}
+        record_dict = {"time": utils.get_arrow_now_str(), "event": event_name, "data": data_dict}
         rd_text = json.dumps(record_dict)
 
         # append to workspace log file
@@ -665,7 +665,7 @@ class StoreV2():
         run_guid = str(uuid.uuid4())
         run_num = run_helper.get_run_number(run_name)
         script = os.path.basename(path) if path else None
-        create_time = utils.get_arrow_time_str()
+        create_time = utils.get_arrow_now_str()
 
         build_parts = constants.BUILD.split(",")
         xt_version = build_parts[0].split(":")[1].strip()
@@ -717,7 +717,7 @@ class StoreV2():
         console.diag("start_run: after log_run_event")
 
         # append "start" record to workspace summary log
-        dd["time"] = utils.get_arrow_time_str()
+        dd["time"] = utils.get_arrow_now_str()
         dd["event"] = "created"
         text = json.dumps(dd) + "\n"
 
@@ -750,7 +750,7 @@ class StoreV2():
         utils.log_info("end_run", run_name)
 
         if not end_time:
-            end_time = utils.get_arrow_time_str()
+            end_time = utils.get_arrow_now_str()
 
         self.log_run_event(ws_name, run_name, "ended", {"status": status, "exit_code": exit_code, \
             "metrics_rollup": metrics_rollup_dict}, event_time=end_time, job_id=job_id)
@@ -834,7 +834,7 @@ class StoreV2():
 
         utils.log_info("rollup_and_end_run", run_name)
 
-        end_time = utils.get_arrow_time_str()
+        end_time = utils.get_arrow_now_str()
 
         # write run to ALLRUNS file
         if aggregate_dest and aggregate_dest != "none":
@@ -948,7 +948,7 @@ class StoreV2():
         #console.print("log_run_event: ws_name={}, run_name={}, event_name={}".format(ws_name, run_name, event_name))
 
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
@@ -978,7 +978,7 @@ class StoreV2():
 
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
-        after_omit_list=None, node_id=None, run_index=None, start_time=None, log_upload_files=False):
+        after_omit_list=None, node_id=None, run_index=None, start_time=None, log_upload_files=False, error_msg=None):
 
         utils.log_info("wrapup_run", run_name)
 
@@ -1024,19 +1024,8 @@ class StoreV2():
             utils.log_info("RUN db_retries", db_retries)
             utils.log_info("RUN storage_retries", storage_retries)
             
-            self.database.run_exit(ws_name, run_name, status, exit_code, db_retries, storage_retries, 
-                 start_time)
-
-            if not is_parent:
-                # tell database this job has a completed run
-                node_index = utils.node_index(node_id)
-                run_was_started = (start_time is not None)
-
-                utils.log_info("calling DATABASE job_run_exit", run_name)
-                self.database.job_run_exit(ws_name, job_id, exit_code, run_was_started)
-
-                utils.log_info("calling DATABASE node_run_end", run_name)
-                self.database.node_run_end(ws_name, job_id, node_index, exit_code, run_was_started)
+            self.database.mark_run_completed(ws_name, job_id, node_id, run_name, status, exit_code, db_retries, storage_retries, 
+                start_time, error_msg, is_parent)
 
     def copy_run_files_to_run(self, ws_name, from_run, run_wildcard, to_run, to_path):
         return self.helper.copy_run_files_to_run(ws_name, from_run, run_wildcard, to_run, to_path)
@@ -1185,7 +1174,7 @@ class StoreV2():
 
         record["status"] = new_status
         record["action_by"] = username
-        record["action_time"] = utils.get_arrow_time_str()
+        record["action_time"] = utils.get_arrow_now_str()
 
         self.database.update_requests_record(ws_name, request_id, record)
 
@@ -1213,15 +1202,16 @@ class StoreV2():
         return self.helper.read_job_info_file(ws_name, job_id)
 
     def log_job_info(self, ws_name, job_id, dd, update_job_stats=True):
+        # this is called multiple times during a job run, so we append to the file
         text = json.dumps(dd, indent=4)
-        self.helper.write_job_info_file(ws_name, job_id, text)
+        self.helper.append_job_info_file(ws_name, job_id, text)
 
         # update database-db for fast access
         self.database.create_db_job(dd, update_job_stats=update_job_stats)
 
     def log_job_event(self, ws_name, job_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
@@ -1234,7 +1224,7 @@ class StoreV2():
 
     def log_node_event(self, ws_name, job_id, node_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
