@@ -162,7 +162,7 @@ class StoreBlobObjs():
     # ---- INTERNAL HELPERS ----
     def _check_ws_name(self, ws_name):
         if not self.is_legal_workspace_name(ws_name):
-           errors.syntax_error("error: Illegal Azure workspace name (must be >= 3 alphanumeric chars, dashes OK, no space or underscore chars)")
+           errors.user_error("error: Illegal Azure workspace name (must be >= 3 alphanumeric chars, dashes OK, no space or underscore chars)")
 
     def _get_job_dir(self, job_name):
         return constants.JOBS_DIR + "/" + job_name 
@@ -410,7 +410,7 @@ class StoreBlobObjs():
         if not container:
             # get a list of all containers is a special case 
             if path:
-                errors.syntax_error("path can not be set when the container is set to '/'")
+                errors.user_error("path can not be set when the container is set to '/'")
                 
             folder, folder_names = self._get_root_folders()
             folders = [ folder ]
@@ -830,9 +830,9 @@ class StoreBlobObjs():
 
         return text
 
-    def write_job_info_file(self, ws_name, job_name, text):
+    def append_job_info_file(self, ws_name, job_name, text):
         fn_job = self._get_job_path_fn(job_name, constants.JOB_INFO_FN)
-        self._create_blob(ws_name, fn_job, text)
+        self._append_blob(ws_name, fn_job, text)
 
     def does_job_exist(self, ws_name, job_id):
         fn_job_info = self._get_job_path_fn(job_id, constants.JOB_INFO_FN)
@@ -994,7 +994,7 @@ class BlobsObject():
         #path = self._expand_path(fn)
         container, path, wc_target = self._get_container_path_target(fn)
         if wc_target:
-            errors.syntax_error("wildcard characters not allowed in filename")
+            errors.user_error("wildcard characters not allowed in filename")
 
         return self.store.provider.create_blob_from_path(container, path, source_fn, progress_callback=progress_callback)
 
@@ -1105,7 +1105,7 @@ class BlobsObject():
             path = os.path.dirname(path)
 
             if "*" in path or "?" in path:
-                errors.syntax_error("wildcard characters are not allowed in directory part of store paths")
+                errors.user_error("wildcard characters are not allowed in directory part of store paths")
         else:
             wc_target = None
             # # treat filename at base like a wildcard spec
@@ -1181,7 +1181,7 @@ class BlobsObject():
     def delete_file(self, filename):
         container, path, wc_target = self._get_container_path_target(filename)
         if wc_target:
-            errors.syntax_error("wildcard not supported here: " + filename)
+            errors.user_error("wildcard not supported here: " + filename)
         self.store.provider.delete_blob(container, path)
         return True
 
