@@ -424,7 +424,7 @@ class MongoDB(implements(DbInterface)):
             if clear:
                 update_doc = { "$unset": dd}
             else:
-                dd["last_time"] = utils.get_arrow_time_str()
+                dd["last_time"] = utils.get_arrow_now_str()
                 update_doc = { "$set": dd}
 
             # update, create prop if needed
