@@ -210,7 +210,7 @@ class StoreV1():
     def log_workspace_event(self, ws_name, event_name, data_dict):
         ''' log the specifed event_name and key/value pairs in the data_dict to the workspace log file.
         '''
-        record_dict = {"time": utils.get_arrow_time_str(), "event": event_name, "data": data_dict}
+        record_dict = {"time": utils.get_arrow_now_str(), "event": event_name, "data": data_dict}
         rd_text = json.dumps(record_dict)
 
         # append to workspace log file
@@ -531,7 +531,7 @@ class StoreV1():
 
         run_num = run_helper.get_run_number(child_name)
         script = os.path.basename(path) if path else None
-        create_time = utils.get_arrow_time_str()
+        create_time = utils.get_arrow_now_str()
 
         build_parts = constants.BUILD.split(",")
         xt_version = build_parts[0].split(":")[1].strip()
@@ -550,7 +550,7 @@ class StoreV1():
         self.log_run_event(ws_name, child_name, "created", dd)
 
         # append "start" record to workspace summary log
-        dd["time"] = utils.get_arrow_time_str()
+        dd["time"] = utils.get_arrow_now_str()
         dd["event"] = "created"
         text = json.dumps(dd) + "\n"    
 
@@ -610,7 +610,7 @@ class StoreV1():
         run_guid = str(uuid.uuid4())
         run_num = run_helper.get_run_number(run_name)
         script = os.path.basename(path) if path else None
-        create_time = utils.get_arrow_time_str()
+        create_time = utils.get_arrow_now_str()
 
         build_parts = constants.BUILD.split(",")
         xt_version = build_parts[0].split(":")[1].strip()
@@ -633,7 +633,7 @@ class StoreV1():
         console.diag("start_run: after log_run_event")
 
         # append "start" record to workspace summary log
-        dd["time"] = utils.get_arrow_time_str()
+        dd["time"] = utils.get_arrow_now_str()
         dd["event"] = "created"
         text = json.dumps(dd) + "\n"
 
@@ -660,7 +660,7 @@ class StoreV1():
         aggregate_dest=None, dest_name=None):
 
         if not end_time:
-            end_time = utils.get_arrow_time_str()
+            end_time = utils.get_arrow_now_str()
 
         self.log_run_event(ws_name, run_name, "ended", {"status": status, "exit_code": exit_code, \
             "metrics_rollup": metrics_rollup_dict}, event_time=end_time)
@@ -695,7 +695,7 @@ class StoreV1():
         # else:
         #     end_time = utils.get_time()
 
-        end_time = utils.get_arrow_time_str()
+        end_time = utils.get_arrow_now_str()
 
         # write run to ALLRUNS file
         if aggregate_dest and aggregate_dest != "none":
@@ -797,7 +797,7 @@ class StoreV1():
 
     def log_node_event(self, ws_name, job_id, node_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
@@ -814,7 +814,7 @@ class StoreV1():
         #console.print("log_run_event: ws_name={}, run_name={}, event_name={}".format(ws_name, run_name, event_name))
 
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
@@ -1026,7 +1026,7 @@ class StoreV1():
 
     def log_job_event(self, ws_name, job_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_arrow_time_str()
+            event_time = utils.get_arrow_now_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
