@@ -4,7 +4,6 @@
 #
  # py: common functions shared among XT modules
 
-from logging import error
 import os
 import re
 import sys
@@ -19,8 +18,11 @@ import shutil
 import datetime
 import traceback
 import importlib
+import itertools
 from threading import Thread, Lock
 from multiprocessing import Pool
+from collections import OrderedDict
+from functools import total_ordering
 
 from xtlib import errors
 from xtlib import constants
@@ -130,6 +132,17 @@ def elapsed_time(start):
 
     return elapsed
 
+def str_is_int(value):
+    is_int = False
+    if value is not None:
+        value = str(value)    # ensure its a str
+        if value[0] in "+-":
+            is_int = value[1:].isdigit()
+        else:
+            is_int = value.isdigit()
+
+    return is_int
+
 def str_is_float(value):
     is_float = False
     try:
@@ -349,7 +362,10 @@ def get_python_value_from_possible_text_list(input):
 
     return output
 
-def get_arrow_time_str():
+def get_arrow_str_from_time(time):
+    return str(arrow.get(time))
+
+def get_arrow_now_str():
     ''' 
     returns NOW as an arrow time string
     '''
@@ -631,8 +647,11 @@ def run_on_processes(worker_func, workitems, max_workers, args_for_worker):
     # wait for all processes to complete
     results = [p.get() for p in res]
 
+add_time_to_log_info = False
+log_info_enabled = True
+
 def log_title(title, double=False):
-    if pc_utils.is_compute_node():
+    if log_info_enabled and pc_utils.is_compute_node():
         if double:
             line = "=========================================="
         else:
@@ -642,18 +661,10 @@ def log_title(title, double=False):
         console.print(title)
         console.print(line)
 
-add_time_to_log_info = False
-
 def log_info(title, value=None):
-    if pc_utils.is_compute_node():
+    if log_info_enabled and pc_utils.is_compute_node():
         title = (title + ":").ljust(25)
 
-        # add time for platforms that don't log time for console output
-        if add_time_to_log_info:
-            now = datetime.datetime.now()
-            now_str = now.strftime("%Y-%m-%d %H:%M:%S")
-            title = "@{}: {}".format(now_str, title)
-
         console.print("{} \t{}".format(title, value))
 
 def log_info_to_text(title, value):
@@ -743,27 +754,45 @@ def tag_dict_from_list(tag_list):
     return td
 
 def shell_time_str_to_secs(text):
-    # convert "3s" or "5m" or 6h" to seconds
-    if text[-1].isdigit():
-        # if units not specified, assume seconds
-        value = int(text)
-    else:
-        value = int(text[:-1])
-        units = text[-1]
+    value = 0
 
-        if units == "s":
-            pass
-        elif units == "m":
-            value *= 60
-        elif units == "h":
-            value *= 60*60
-        elif units == "d":
-            value *= 60*60*60
+    if text:
+        # convert "3s" or "5m" or 6h" to seconds
+        if text[-1].isdigit():
+            # if units not specified, assume seconds
+            value = float(text)
         else:
-            errors.ConfigError("shell time has unrecognized unit: {}".format(text))
+            value = float(text[:-1])
+            units = text[-1]
+
+            if units == "s":
+                pass
+            elif units == "m":
+                value *= 60
+            elif units == "h":
+                value *= 60*60
+            elif units == "d":
+                value *= 60*60*60
+            else:
+                errors.ConfigError("shell time has unrecognized unit: {}".format(text))
 
     return value
 
+def secs_to_shell_time(secs, suffix):
+    if suffix == "s":
+        value = secs
+    elif suffix == "m":
+        value = secs/60
+    elif suffix == "h":
+        value = secs/3600
+    elif suffix == "d":
+        value = secs/(24*3600)
+    else:
+        errors.internal_error("unknown shell time suffix: {}".format(suffix))
+
+    text = "{:.2f}{}".format(value, suffix)
+    return text
+
 def get_user_columns(columns_arg, report_name, config):
     report_columns = config.get(report_name, "columns")
 
@@ -857,10 +886,24 @@ def friendly_duration_format(value, ongoing):
         secs = float(value)   # in case its a string
         secs_per_day = 60*60*24
         days = secs/secs_per_day
+        weeks = days/7
+        months = days/30
+        years = days/365
+
         plus = "+" if ongoing else ""
 
-        if days >= 1:
+        if years >= 1:
+            value = "{}{:,.1f} years".format(plus, years)
+
+        elif months >= 1:
+            value = "{}{:,.1f} months".format(plus, months)
+
+        elif weeks >= 1:
+            value = "{}{:,.1f} weeks".format(plus, weeks)
+
+        elif days >= 1:
             value = "{}{:,.1f} days".format(plus, days)
+            
         else:
             hrs = days*24
             if hrs >= 1:
@@ -874,3 +917,68 @@ def friendly_duration_format(value, ongoing):
                     value = "{}{:,.1f} secs".format(plus, secs)
 
     return value
+
+
+def set_openai_key(config, service_name):
+    import openai
+
+    openai.api_key = config.get("external-services", service_name, "key")
+    openai.organization = config.get("external-services", service_name, "org")
+    a = 3
+
+# define a minimum value for sorting (substitutes for None)
+@total_ordering
+class MinSortItem(object):
+    def __le__(self, other):
+        return True
+
+    def __eq__(self, other):
+        return (self is other)
+
+min_sort_item = MinSortItem()    
+
+def sort_dict_by_keys(dd, reverse=False, first=None, last=None):
+    # do keys need to be converted to floats?
+    convert_to_float = True
+    for key in dd:
+        if (isinstance(key, str) and not str_is_float(key)):
+            convert_to_float = False
+            break
+
+    if convert_to_float:
+        dd = {None if key is None else float(key):value for key, value in dd.items()}
+
+    dd = OrderedDict(sorted(dd.items(), key=lambda k: min_sort_item if k[0] is None else k[0], reverse=reverse))
+
+    if first:
+        dd = OrderedDict(itertools.islice(dd.items(), first))
+        
+    elif last:
+        dd = OrderedDict(itertools.islice(dd.items(), len(dd) - last, None))
+
+    return dd
+
+def sort_tuple_list_by_key(tuple_list, reverse=False, first=None, last=None):
+    # do keys need to be converted to floats?
+    convert_to_float = True
+    for key, value in tuple_list:
+        if (isinstance(key, str) and not str_is_float(key)):
+            convert_to_float = False
+            break
+
+    if convert_to_float:
+        tuple_list = [ (None if key is None else float(key),value) for key, value in tuple_list ]
+        
+    tuple_list = sorted(tuple_list, key=lambda k: min_sort_item if k[0] is None else k[0], reverse=reverse)
+    text = ""
+
+    # apply first, last
+    if first:
+        tuple_list = tuple_list[:first]
+
+    if last:  
+        tuple_list = tuple_list[-last:]
+
+    return tuple_list
+
+
