@@ -81,59 +81,87 @@ def get_fn_local_config(cmd):
 
     return fn
 
+def args_to_dict(args):
+    arg_dict = {}
+    a = 0
+
+    while a < len(args):
+        arg = args[a]
+        assert arg.startswith("--")
+
+        arg = arg[2:]
+        if "=" in arg:
+            name, value = arg.split("=", 1)
+            part = value
+        else:
+            name = arg
+            value = 1
+            part = None
+
+        # handle space delimited str list  (e.g., a, b, c)
+        while part and part.endswith(",") and a < len(args)-1:
+            a += 1
+            part = args[a]
+            value += " " + part
+
+        arg_dict[name] = value
+        a += 1
+
+    return arg_dict
+
 def process_template(config, cmd):
+    '''
+    replace a template invocation and specified args with the corresponding XT command.
+    '''
     args = cmd_utils.xt_cmd_split(cmd)
 
     template_name = args[0][1:]
-    args = args[1:]
+    template_actual_args = args[1:]
+    arg_dict = args_to_dict(template_actual_args)
 
     td = config.get("templates", template_name)
     if not td:
-        errors.syntax_error("Template not found: {}".format(template_name))
+        errors.user_error("Template not found: {}".format(template_name))
 
     hidden = utils.safe_value(td, "hidden", False)
     if hidden:
-        errors.syntax_error("cannot use a template that is marked as hidden: {}".format(template_name))
+        errors.user_error("cannot use a template that is marked as hidden: {}".format(template_name))
 
-    templ = td["command"]
-    targs = templ.split()
-    tats = [a for a in targs if a.startswith("@")]
+    template_cmd = td["command"]
+    cmd_parts = template_cmd.split()
+    template_formal_args = [part for part in cmd_parts if part.startswith("@")]
+    opt_arg_dict = utils.safe_value(td, "optional-args", {})
 
-    if len(tats) > len(args):
-        errors.syntax_error("templates requires {} args, but only {} were supplied".format(len(tats), len(args)))
+    required_arg_count = len(template_formal_args) - len(opt_arg_dict)
 
-    for i, targ in enumerate(targs):
-        if targ.startswith("@"):
-            targs[i] = args.pop(0)
+    if required_arg_count > len(template_actual_args):
+        errors.user_error("template requires {} args, but only {} were supplied".format(len(template_formal_args), 
+            len(template_actual_args)))
 
-    cmd = " ".join(targs)
+    for i, part in enumerate(cmd_parts):
+        if part.startswith("@"):
+            name = part[1:]
+            if name in arg_dict:
+                cmd_parts[i] = arg_dict[name]
+                del arg_dict[name]
+            else:
+                # argument not specified by user
+                if name in opt_arg_dict:
+                    # use value from optional args dict
+                    cmd_parts[i] = opt_arg_dict[name]
+                else:
+                    errors.user_error("template argument not specified: --{}".format(name))
+
+    cmd = "xt " + " ".join(cmd_parts)
 
     # add any remaining args to the end of the command
-    cmd += " " + " ".join(args)
+    for name, value in arg_dict.items():
+        cmd += " --{}={}".format(name, value)
 
-    print("template expansion: {}".format(cmd))
-    args = cmd.split()
-
-    return args
-
-def main(cmd, new_start_time=None, capture_output=False, basic_mode=False, raise_syntax_exception=True):
-    '''
-    Parse and execute the specified cmd.
-    '''
-    # when executing multiple commands, reset the feedback for each command
-    feedback.reset_feedback()
-
-    #console.print("cmd=", cmd, ", args=", args)
-    console.diag("in xt_cmds.main")
-
-    #console.print("config=", config)
-    fn_local_config = get_fn_local_config(cmd)
-
-    impl_shared = ImplShared()
-    config = impl_shared.init_config(fn_local_config)
-    store = impl_shared.store
-    basic_mode = not config.get("general", "advanced-mode")
+    print("[{}]".format(cmd))
+    return cmd
 
+def build_xt_cmds(config, store, basic_mode=False):
     cmd_providers = config.get("providers", "command")
     impl_dict = {}
 
@@ -149,17 +177,9 @@ def main(cmd, new_start_time=None, capture_output=False, basic_mode=False, raise
         if name == "help":
             impl.set_basic_mode(basic_mode)
 
-    # process template invocations
-    if cmd and cmd.startswith("xt "):
-        cmd = cmd[3:].strip()
-
-    if cmd.startswith("@"):
-        cmd = process_template(config, cmd)
-
-    # this parses args and calls the correct command function with its args and options correctly set.
-    # the config object supplies the default value for most options and flags.
-    dispatcher = qfe.Dispatcher(impl_dict, config, preprocessor=impl_shared.pre_dispatch_processing)
+    return impl_dict
 
+def hide_commands(dispatcher, basic_mode=False):
     if basic_mode:
         # a dict of commands + arg/options to be surfaced (None means use all args/options)
         show_commands = {
@@ -199,6 +219,44 @@ def main(cmd, new_start_time=None, capture_output=False, basic_mode=False, raise
     hide_commands.append("generate_help")
     dispatcher.hide_commands(hide_commands)
 
+def main(cmd, new_start_time=None, capture_output=False, basic_mode=False, raise_syntax_exception=True):
+    '''
+    Parse and execute the specified cmd.
+    '''
+    # when executing multiple commands, reset the feedback for each command
+    feedback.reset_feedback()
+
+    global orig_xt_cmd
+    orig_xt_cmd = cmd
+
+    #console.print("cmd=", cmd, ", args=", args)
+    console.diag("in xt_cmds.main")
+
+    #console.print("config=", config)
+    fn_local_config = get_fn_local_config(cmd)
+
+    impl_shared = ImplShared()
+    config = impl_shared.init_config(fn_local_config)
+    store = impl_shared.store
+    basic_mode = not config.get("general", "advanced-mode")
+
+    if cmd and cmd.startswith("xt "):
+        cmd = cmd[3:].strip()
+
+    # process template invocations
+    if cmd.startswith("@"):
+        cmd = process_template(config, cmd)
+        if cmd and cmd.startswith("xt "):
+            cmd = cmd[3:].strip()
+
+    impl_dict = build_xt_cmds(config, store, basic_mode)
+
+    # this parses args and calls the correct command function with its args and options correctly set.
+    # the config object supplies the default value for most options and flags.
+    dispatcher = qfe.Dispatcher(impl_dict, config, preprocessor=impl_shared.pre_dispatch_processing)
+
+    hide_commands(dispatcher, basic_mode)
+
     # expand symbols like $lastjob, $lastrun
     cmd = impl_shared.expand_xt_symbols(cmd)
 
