@@ -25,13 +25,12 @@ import socket
 import sys
 import os
 import json
-import shlex
 import logging
 
 from xtlib import utils
 from xtlib import constants
+from xtlib import cmd_utils
 from xtlib import file_utils
-
 from xtlib.console import console
 
 # turn off console output until we parse our root options
@@ -68,7 +67,9 @@ def run_cmd_on_server(cmd_dict, retry_count):
             console.print(msg, end="", flush=True)    
 
 def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False):
-
+    '''
+    This is the XT entrypoint (also used by the debug.py script)
+    '''
     utils.init_logging(constants.FN_XT_EVENTS, logger, "XT session")
     # print("cmd=", cmd)
     # print("sys.argv[1:]", sys.argv[1:])
@@ -77,45 +78,23 @@ def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False)
     if cmd:
         # treat as if it came from the shell (for consistent debugging/support)
         console.diag("orig cmd={}".format(cmd))
+        args = cmd_utils.xt_cmd_split(cmd)
 
-        # shlex and LINUX SHELL lose single quotes around strings, but windows does not
-        # shlex also zaps single backslashes
-        cmd = cmd.replace("\\", "\\\\")    # double backslashes to prevent their removal
-
-        orig_args = shlex.split(cmd)
-
-        # now, undouble backslashes
-        for a, arg in enumerate(orig_args):
-            if "\\\\" in arg:
-                orig_args[a] = arg.replace("\\\\", "\\")
-
-        console.diag("shlex args={}".format(orig_args))
     else:
+        # drop "xt" name from sys.argv
         orig_args = sys.argv[1:]
         console.diag("orig_args={}".format(orig_args))
+        args = cmd_utils.add_back_quotes(orig_args)
 
-    # add back quotes where needed 
-    for a, arg in enumerate(orig_args):
-        if " " in arg:
-            if "=" in arg:
-                name, value = arg.split("=")
-                # ensure it is not already quoted
-                if value and not value[0] in "'`\"":
-                    orig_args[a] = "{}='{}'".format(name, value)
+    cmd = cmd_utils.join_args(args)
+    #from this point all, we should keep cmd as string (not args)
 
-            # ensure it is not already quoted
-            elif arg and not arg[0] in "'`\"":
-                orig_args[a] = "'" + arg + "'"
+    use_server = False   # "--quic" in cmd
+    # if not use_server:
+    #     from .helpers.xt_config import get_merged_config
 
-    cmd = " ".join(orig_args)
-    cmd = cmd.strip()
-
-    use_server = "--quic" in cmd
-    if not use_server:
-        from .helpers.xt_config import get_merged_config
-
-        config = get_merged_config()
-        use_server = config.get("general", "quick-start")
+    #     config = get_merged_config()
+    #     use_server = config.get("general", "quick-start")
 
     mid_elapsed = time.time() - xt_start_time
     #console.print("mid_elapsed={:.2f}".format(mid_elapsed))
