@@ -131,13 +131,15 @@ class XTVault():
                 credential = DeviceCodeCredential() 
 
             else:
-                errors.syntax_error("unrecognized authentication type '{}'".format(authentication))
+                errors.user_error("unrecognized authentication type '{}'".format(authentication))
 
             if not deserialized_record:
                 # serialize to our cred to disk
                 record = credential.authenticate()
                 cred_json = record.serialize()
 
+                file_utils.ensure_dir_exists(file=fn)
+                
                 with open(fn, "wt") as outfile:
                     outfile.write(cred_json)
 
